//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CAppBase.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CAPPBASE_H
#define CAPPBASE_H

#include "CWinUtil.h"
#include "CWinInit.h"
#include "CDirectX11.h"
#include "CDXStates.h"
#include "CInput.h"
#include "CInputControls.h"
#include "..\Globals\CGlobals.h"
#include "..\Utilities\CDebugError.h"
#include "..\Utilities\CMemoryFree.h"
#include "..\Utilities\CTimer.h"
#include "..\Utilities\CScreenshot.h"
#include "..\Utilities\CSystemInfo.h"
#include "..\Graphics\CPostScreenQuad.h"
#include "..\Graphics\CTextureList.h"
#include "..\Graphics\CRenderingData.h"

#include "..\Audio\CAudioList.h"
#include "..\Audio\CXAudio2.h"

namespace App
{
	// CAppBase class
	// Primarily used as a base class for the main application/game class.
	class CAppBase
	{
	public:
		CAppBase();
		CAppBase(const CAppBase& other);
		virtual ~CAppBase();

		virtual int							Initialize();
		int									Run();

		virtual int							PreResize();
		virtual int							PostResize();

		virtual void						Release();

		inline HWND							WindowHandle() const;

	protected:
		virtual int							Setup() { return Util::Err::NONE; }
		int									InitializePostQuad();
		virtual int							Update() { return Util::Err::NONE; }
		virtual int							Render() { return Util::Err::NONE; }

		virtual int							MessageHandler(MSG msg) { return Util::Err::NONE; }
		
		virtual LRESULT CALLBACK			WinProc(HWND hWnd, u32 msg, WPARAM wParam, LPARAM lParam);
		static LRESULT CALLBACK				DisplayWinProc(HWND hWnd, u32 msg, WPARAM wParam, LPARAM lParam);

		// Variables.
		bool								m_bInitialized;
		bool								m_bInFocus;
		bool								m_bMinimized;
		bool								m_bQuitting;

		Util::CTimer*						m_pTimer;

		CWinInit*							m_pWindow;
		WINDATA*							m_pWinData;
		CDirectX11*							m_pDirectX;
		CDXStates*							m_pDXStates;
		DX11DATA*							m_pDX11Data;

		CInput*								m_pInput;

		GFX::CPostScreenQuad*				m_pPostQuad;
		GFX::RENDERDATA						m_renderData;

		Audio::CXAudio2Engine*				m_pAudioEngine;

	private:
		int									AdjustToResize();
	};

	// Window handle getter.

	inline HWND CAppBase::WindowHandle() const
	{
		if(!m_pWinData)
			return NULL;

		return m_pWinData->hWnd;
	}
};

#endif