//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CDirectX11.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDIRECTX11_H
#define CDIRECTX11_H

#include <d3d11.h>
#include "..\Globals\CGlobals.h"
#include "..\Utilities\CDebugError.h"
#include "..\Utilities\CDebugLog.h"
#include "..\Utilities\CMemoryFree.h"
#include "CWinData.h"
#include "CDX11Data.h"

#pragma comment(lib, "d3d11.lib")
#pragma comment(lib, "d3dx11.lib")
#pragma comment(lib, "dxguid.lib")
#pragma comment(lib, "dxgi.lib")

namespace App
{
	// CDirectX11 class.
	// Class for initializing and running DirectX with this application.
	class CDirectX11
	{
	public:
		CDirectX11();
		CDirectX11(const CDirectX11& other);
		virtual ~CDirectX11();

		int									Initialize(WINDATA* winData);

		int									BeginScene();
		int									EndScene();

		int									ResizeWindow();

		int									FindScreenMode();
		int									UpdateScreenMode();
		int									UpdateScreenMode(bool fullscreen);

		void								Release();

		inline DX11DATA*					DXData() const;
		inline void							DXData(DX11DATA* pDX11Data);

	private:
		int									HandleDisplayData(u32* numeratorOut, u32* denominatorOut);
		int									CreateSwapChainAndDevices(u32 numerator, u32 denominator);
		int									CreateRenderAndDepthBuffers();

		WINDATA*							m_pWinData;
		DX11DATA*							m_pDX11Data;
	};

	// DirectX 11 data getter.

	inline DX11DATA* CDirectX11::DXData() const
	{
		return m_pDX11Data;
	}

	inline void CDirectX11::DXData(DX11DATA* pDX11Data)
	{
		m_pDX11Data = pDX11Data;
	}
};

#endif
