//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CInput.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CINPUT_H
#define CINPUT_H

#include <string>
#include "..\Math\CMathVector2.h"
#include "..\Globals\CGlobals.h"
#include "..\Utilities\CDebugError.h"

// Mouse input name string constants.
const std::string MOUSE_INPUTS[] {
	"Left Mouse",
	"Right Mouse",
	"Middle Mouse",
};

// Keyboard input name string constants.
const std::string KEY_INPUTS[] = {
	"Left Button",
	"Right Button",
	"Cancel",
	"Middle Button",
	"X Button1",
	"X Button2",
	"0x06",
	"0x07",
	"Backspace",
	"Tab",
	"0x0A",
	"0x0B",
	"Clear",
	"Enter",
	"0x0E",
	"0x0F",
	"Shift",
	"Ctrl",
	"Alt",
	"Pause",
	"Caps",
	"Kana",
	"0x16",
	"Junja",
	"Final",
	"Hanja",
	"0x1A",
	"Esc",
	"Conv",
	"Non-Conv",
	"Accept",
	"Mode",
	"Space",
	"Pg Up",
	"Pg Down",
	"End",
	"Home",
	"Left",
	"Up",
	"Right",
	"Down",
	"Sel",
	"Prt",
	"Exe",
	"PrtScr",
	"Ins",
	"Del",
	"Help",
	"0",
	"1",
	"2",
	"3",
	"4",
	"5",
	"6",
	"7",
	"8",
	"9",
	"0x3A",
	"0x3B",
	"0x3C",
	"0x3D",
	"0x3E",
	"0x3F",
	"0x40",
	"A",
	"B",
	"C",
	"D",
	"E",
	"F",
	"G",
	"H",
	"I",
	"J",
	"K",
	"L",
	"M",
	"N",
	"O",
	"P",
	"Q",
	"R",
	"S",
	"T",
	"U",
	"V",
	"W",
	"X",
	"Y",
	"Z",
	"Win",
	"Win",
	"App",
	"0x5E",
	"Sleep",
	"NUM 0",
	"NUM 1",
	"NUM 2",
	"NUM 3",
	"NUM 4",
	"NUM 5",
	"NUM 6",
	"NUM 7",
	"NUM 8",
	"NUM 9",
	"Mul",
	"Add",
	"Sep",
	"Sub",
	"Dec",
	"Div",
	"F1",
	"F2",
	"F3",
	"F4",
	"F5",
	"F6",
	"F7",
	"F8",
	"F9",
	"F10",
	"F11",
	"F12",
	"F13",
	"F14",
	"F15",
	"F16",
	"F17",
	"F18",
	"F19",
	"F20",
	"F21",
	"F22",
	"F23",
	"F24",
	"0x88",
	"0x89",
	"0x8A",
	"0x8B",
	"0x8C",
	"0x8D",
	"0x8E",
	"0x8F",
	"Num",
	"Scroll",
	"0x92",
	"0x93",
	"0x94",
	"0x95",
	"0x96",
	"0x97",
	"0x98",
	"0x99",
	"0x9A",
	"0x9B",
	"0x9C",
	"0x9D",
	"0x9E",
	"0x9F",
	"LShift",
	"RShift",
	"LCtrl",
	"RCtrl",
	"LMenu",
	"RMenu",
	"Back",
	"Forward",
	"Refresh",
	"Stop",
	"Search",
	"Fav",
	"Home",
	"Vol Mute",
	"Vol Down",
	"Vol Up",
	"Media Next",
	"Media Prev",
	"Media Stop",
	"Media Play",
	"Launch Mail",
	"Launch Media",
	"Launch App1",
	"Launch App2",
	"0xB8",
	"0xB9",
	"US ;:",
	"Plus",
	"Comma",
	"Minus",
	"Period",
	"US /?",
	"US ~",
	"0xC1",
	"0xC2",
	"0xC3",
	"0xC4",
	"0xC5",
	"0xC6",
	"0xC7",
	"0xC8",
	"0xC9",
	"0xCA",
	"0xCB",
	"0xCC",
	"0xCD",
	"0xCE",
	"0xCF",
	"0xD0",
	"0xD1",
	"0xD2",
	"0xD3",
	"0xD4",
	"0xD5",
	"0xD6",
	"0xD7",
	"0xD8",
	"0xD9",
	"0xDA",
	"US [{",
	"US \\|",
	"US ]}",
	"US \'\"",
	"OEM 8",
	"0xE0",
	"OEM",
	"OEM 102",
	"OEM",
	"OEM",
	"Process Key",
	"OEM",
	"Packet",
	"E8",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"OEM",
	"ATtn",
	"Crsel",
	"Exsel",
	"Erase",
	"Play",
	"Zoom",
	"No Name",
	"PA 1",
	"OEM Clear",
};

// Mouse id constants.
const u8 VM_MOUSE_LEFT		= 0;
const u8 VM_MOUSE_RIGHT		= 1;
const u8 VM_MOUSE_MIDDLE	= 2;

// Keyboard id constants.
const u8 VK_KEY_0			= 0x30;
const u8 VK_KEY_1			= 0x31;
const u8 VK_KEY_2			= 0x32;
const u8 VK_KEY_3			= 0x33;
const u8 VK_KEY_4			= 0x34;
const u8 VK_KEY_5			= 0x35;
const u8 VK_KEY_6			= 0x36;
const u8 VK_KEY_7			= 0x37;
const u8 VK_KEY_8			= 0x38;
const u8 VK_KEY_9			= 0x39;
const u8 VK_KEY_A			= 0x41;
const u8 VK_KEY_B			= 0x42;
const u8 VK_KEY_C			= 0x43;
const u8 VK_KEY_D			= 0x44;
const u8 VK_KEY_E			= 0x45;
const u8 VK_KEY_F			= 0x46;
const u8 VK_KEY_G			= 0x47;
const u8 VK_KEY_H			= 0x48;
const u8 VK_KEY_I			= 0x49;
const u8 VK_KEY_J			= 0x4A;
const u8 VK_KEY_K			= 0x4B;
const u8 VK_KEY_L			= 0x4C;
const u8 VK_KEY_M			= 0x4D;
const u8 VK_KEY_N			= 0x4E;
const u8 VK_KEY_O			= 0x4F;
const u8 VK_KEY_P			= 0x50;
const u8 VK_KEY_Q			= 0x51;
const u8 VK_KEY_R			= 0x52;
const u8 VK_KEY_S			= 0x53;
const u8 VK_KEY_T			= 0x54;
const u8 VK_KEY_U			= 0x55;
const u8 VK_KEY_V			= 0x56;
const u8 VK_KEY_W			= 0x57;
const u8 VK_KEY_X			= 0x58;
const u8 VK_KEY_Y			= 0x59;
const u8 VK_KEY_Z			= 0x5A;

const u32 MAX_MOUSE_BUTTONS = 4;
const u32 MAX_KEYS = 256;

namespace App
{
	// Structure for storing mouse data.
	struct MOUSE_DATA
	{
		Math::CVector2 pos;
		float x, y;
		float xPrev, yPrev;
		float wheel;
		u32 button[MAX_MOUSE_BUTTONS];

		MOUSE_DATA()
		{
			x			= 0.0f;
			y			= 0.0f;
			xPrev		= 0.0f;
			yPrev		= 0.0f;
			wheel		= 0.0f;
		}
	};

	// CInput class.
	// Class for handling mouse and keyboard input.
	class CInput
	{
	public:
		CInput();
		CInput(const CInput& other);
		virtual ~CInput();

		int									Initialize();
		int									Update();
		int									PostUpdate();
		int									MessageHandler(MSG msg);
		
		inline bool							Key(u32 key);
		inline bool							KeyDown(u32 key);
		inline bool							KeyUp(u32 key);

		inline bool							MouseDClick(u32 button);
		inline bool							Mouse(u32 button);
		inline bool							MouseDown(u32 button);
		inline bool							MouseUp(u32 button);

		inline MOUSE_DATA					MouseData() const;

		inline void							hWnd(HWND handle) { m_hWnd = handle; }

	private:
		inline void							UpdateKey(u32 key, bool down);
		inline void							UpdateMouse(u32 button, bool down);
		inline void							UpdateMouseDClick(u32 button);

		void								UpdateMouseData();
		inline void							UpdateMouseWheel(float wheel);

		bool								m_bKeyDown[MAX_KEYS];
		bool								m_bKeyUp[MAX_KEYS];
		bool								m_bKey[MAX_KEYS];

		bool								m_bMouseDown[MAX_MOUSE_BUTTONS];
		bool								m_bMouseUp[MAX_MOUSE_BUTTONS];
		bool								m_bMouse[MAX_MOUSE_BUTTONS];
		bool								m_bMouseDClick[MAX_MOUSE_BUTTONS];

		MOUSE_DATA							m_mouseData;

		RAWINPUT*							m_pRawInput;
		HWND								m_hWnd;
	};

	//
	// Input inline methods.
	//
	
	// Key methods.
	inline bool CInput::Key(u32 key)
	{
		return m_bKey[key];
	}

	inline bool CInput::KeyDown(u32 key)
	{
		return m_bKeyDown[key];
	}

	inline bool CInput::KeyUp(u32 key)
	{
		return m_bKeyUp[key];
	}

	// Mouse methods.
	inline bool CInput::MouseDClick(u32 button)
	{
		return m_bMouseDClick[button];
	}

	inline bool CInput::Mouse(u32 button)
	{
		return m_bMouse[button];
	}

	inline bool CInput::MouseDown(u32 button)
	{
		return m_bMouseDown[button];
	}

	inline bool CInput::MouseUp(u32 button)
	{
		return m_bMouseUp[button];
	}

	// Update key methods.
	inline void CInput::UpdateKey(u32 key, bool down)
	{
		if(down)
			m_bKeyDown[key] = m_bKey[key] ? false : true;
		else
			m_bKeyUp[key] = m_bKey[key] ? true : false;

		m_bKey[key] = down;
	}
	
	// Update mouse methods.
	inline void CInput::UpdateMouse(u32 button, bool down)
	{
		if(down)
			m_bMouseDown[button] = m_bMouse[button] ? false : true;
		else
			m_bMouseUp[button] = m_bMouse[button] ? true : false;

		m_bMouse[button] = down;
	}

	inline void CInput::UpdateMouseDClick(u32 button)
	{
		m_bMouseDClick[button] = true;
	}

	inline void CInput::UpdateMouseWheel(float wheel)
	{
		m_mouseData.wheel = wheel;
	}

	inline MOUSE_DATA CInput::MouseData() const
	{
		return m_mouseData;
	}
};

#endif