//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CInputControls.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CINPUTCONTROLS_H
#define CINPUTCONTROLS_H

#include "CInput.h"
#include "..\Utilities\CTimer.h"
#include "..\Utilities\CDataStack.h"

namespace App
{
	const u32 KEY_BIND_NAME_MAX = 64;

	// Enumerator for key bind input device type.
	enum class KEY_BIND_TYPE
	{
		Keyboard,
		Mouse,
	};

	// Structure for storing data about a key bind option.
	struct KEY_BIND_OPTION
	{
		// Structure for storing data about a key bind sub-option.
		struct KEY_BIND
		{
			u32					code;
			KEY_BIND_TYPE		bindType;

			bool				activated;
			bool				allowHold;
			float				timer;

			KEY_BIND()
			{
				ZeroMemory(this, sizeof(*this));
			}

			KEY_BIND(u32 Code, const KEY_BIND_TYPE& BindType, bool AllowHold)
			{
				code			= Code;
				bindType		= BindType;
				allowHold		= AllowHold;

				activated		= false;
				timer			= 0.0f;
			}
		};

		u32						id;
		char					name[KEY_BIND_NAME_MAX];

		KEY_BIND				primary;
		KEY_BIND				secondary;

		KEY_BIND_OPTION()
		{
			ZeroMemory(this, sizeof(*this));
		}

		KEY_BIND_OPTION(u32 Id, const std::string& Name, const KEY_BIND& Primary, const KEY_BIND& Secondary)
		{
			id			= Id;
			primary		= Primary;
			secondary	= Secondary;
			
			memset(name, 0, KEY_BIND_NAME_MAX);
			memcpy(name, Name.c_str(), min(KEY_BIND_NAME_MAX, Name.length()));
		}
	};

	// CControls class (singleton)
	// Class for creating a list of keybinds specific to an application.
	class CControls
	{
	public:
		friend class CAppBase;

	public:
		static CControls& Instance()
		{
			static CControls instance;
			return instance;
		}

	public:
		void								AddKeyBind(const KEY_BIND_OPTION& keyBind);
		bool								CheckForKeyBind(const KEY_BIND_OPTION& keyBind, KEY_BIND_OPTION* output = NULL);

		bool								FindKeyBind(u32 id, KEY_BIND_OPTION& output);
		bool								FindKeyBind(const std::string& name, KEY_BIND_OPTION& output);

		bool								Activated(u32 id);
		bool								Activated(const std::string& name);

		inline u32							KeyBindCount();

	private:
		CControls();
		CControls(const CControls&) = delete;
		virtual ~CControls();
		
		void								Initialize(CInput*& input, Util::CTimer*& timer);
		void								Update();
		void								UpdateKeyBind(KEY_BIND_OPTION::KEY_BIND& keyBind);
		void								Release();

		Util::CStack<KEY_BIND_OPTION>		m_sKeyBindStack;

		CInput*								m_pInput;
		Util::CTimer*						m_pTimer;
	};

	// Key binding count getter.
	inline u32 CControls::KeyBindCount()
	{
		return m_sKeyBindStack.Length();
	}
};

#endif