//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CInputTyping.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CINPUTTYPING_H
#define CINPUTTYPING_H

#include <string>
#include "CInput.h"
#include "..\Utilities\CTimer.h"
#include "..\Math\CMathFloat.h"

namespace App
{
	const float TYPING_KEY_HELD_TIME		= 0.35f;
	const float TYPING_KEY_SENSITIVITY		= 0.035f;

	// Typing flag.
	enum TYPING_FLAG : u32
	{
		TYPING_DISABLED			= 0,

		TYPING_INTEGER			= 1,
		TYPING_NUMERIC_SIGNS	= 2,
		TYPING_RATIONAL			= 3,

		TYPING_LOWER			= 4,
		TYPING_UPPER			= 8,
		TYPING_ALPHA			= 12,

		TYPING_SPECIAL			= 18,

		TYPING_EVERYTHING		= ~(u32)0,
	};

	// CInputTyping class
	// Class used to generate a sting based on the users keystrokes.
	class CInputTyping
	{
	public:
		CInputTyping();
		CInputTyping(const CInputTyping& other);
		virtual ~CInputTyping();

		int									Initialize(CInput*& input, Util::CTimer* timer);
		bool								Update(int& caretIndex, std::string* outputText = NULL);

		inline std::string					Text() const;
		inline void							Text(const std::string& text);

		inline TYPING_FLAG					TypingFlag() const;
		inline void							TypingFlag(const u32& flag);

	private:
		void								TestNumeric(std::string& text);
		void								TestAlpha(std::string& text);
		void								TestSpecial(std::string& text);

		char								GetSpecialFromKeyStroke(int key);
		char								GetSpecialFromLower(int lower);

		void								HandleKeyStroke(std::string& text, int key, char character);
		void								HandleBackAndDelete(std::string& text, int& caretIndex);
		void								HandleArrowKeys(const std::string& text, int& caretIndex);

		std::string							m_text;

		bool								m_bCaps;
		bool								m_bNumLock;

		float								m_keyTimers[MAX_KEYS];

		TYPING_FLAG							m_typingFlag;

		CInput*								m_pInput;
		Util::CTimer*						m_pTimer;
	};

	// Typing text getter and setter.

	inline std::string CInputTyping::Text() const
	{
		return m_text;
	}

	inline void CInputTyping::Text(const std::string& text)
	{
		m_text = text;
	}

	// Typing flag getter and setter.
	
	inline TYPING_FLAG CInputTyping::TypingFlag() const
	{
		return m_typingFlag;
	}

	inline void CInputTyping::TypingFlag(const u32& flag)
	{
		m_typingFlag = (TYPING_FLAG)flag;
	}
}

#endif