//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CWinInit.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CWinInit.h"

namespace App
{
	CWinInit::CWinInit()
	{
		m_pWinData = new WINDATA();
	}

	CWinInit::CWinInit(const CWinInit& other)
	{
	}

	CWinInit::~CWinInit()
	{
	}

	// Method for creating a window to be used with this application.
	// Notice: child window creation isn't fully implemented.
	int CWinInit::Create(WNDPROC winProc, long application, HWND parent)
	{
		int retCode = Util::Err::NONE;

		// If the window title is still null, set a default value to it.
		if(!m_pWinData->pWindowTitle)
			m_pWinData->pWindowTitle = "Untitled";

		WNDCLASSEX wc;
		ZeroMemory(&wc, sizeof(WNDCLASSEX));

		// Fill the window class structure.
		wc.cbSize							= sizeof(WNDCLASSEX);
		wc.style							= CS_HREDRAW | CS_VREDRAW | CS_OWNDC | (m_pWinData->bAllowDoubleClicks ? CS_DBLCLKS : 0);
		wc.lpfnWndProc						= winProc;
		wc.hInstance						= m_pWinData->hInstance = (HINSTANCE)GetModuleHandle(0);
		wc.hCursor							= LoadCursor(NULL, IDC_ARROW);
		wc.hIcon							= LoadIcon(wc.hInstance, m_pWinData->pIconName);
		wc.hIconSm							= wc.hIcon;
		wc.hbrBackground					= (HBRUSH)GetStockObject(BLACK_BRUSH);
		wc.lpszClassName					= m_pWinData->pWindowTitle;

		m_pWinData->menu					= LoadMenu(wc.hInstance, m_pWinData->pMenuName);

		RegisterClassEx(&wc);

		m_pWinData->maxWidth				= GetSystemMetrics(SM_CXSCREEN);
		m_pWinData->maxHeight				= GetSystemMetrics(SM_CYSCREEN);

		// Check for window dimension support.
		if(m_pWinData->maxWidth < m_pWinData->minWidth || m_pWinData->maxHeight < m_pWinData->minHeight)
		{
			return Util::Err::ERR_WINDOW_DISPLAY_TO_SMALL;
		}

		Math::Clamp(&m_pWinData->width, m_pWinData->minWidth, m_pWinData->maxWidth);
		Math::Clamp(&m_pWinData->height, m_pWinData->minHeight, m_pWinData->maxHeight);

		m_pWinData->hWnd = CreateWindowEx(
			NULL,
			wc.lpszClassName,
			wc.lpszClassName,
			m_pWinData->bResizable ? WS_OVERLAPPEDWINDOW : WS_OVERLAPPED | WS_MINIMIZEBOX | WS_SYSMENU,
			m_pWinData->posX,
			m_pWinData->posY,
			m_pWinData->width,
			m_pWinData->height,
			parent,
			m_pWinData->menu,
			wc.hInstance,
			NULL
		);

		if(m_pWinData->hWnd == NULL)
		{
			return Util::Err::ERR_WINDOW_HANDLE_NULL;
		}

		SetWindowLong(m_pWinData->hWnd, GWLP_USERDATA, application);

		// Display and set focus to this application window.
		ShowWindow(m_pWinData->hWnd, m_pWinData->bStartMaximized ? SW_MAXIMIZE : SW_SHOW);
		SetForegroundWindow(m_pWinData->hWnd);
		SetFocus(m_pWinData->hWnd);

		UpdateRects();

		return retCode;
	}

	// Method for updating the window and client rectangles for this application.
	void CWinInit::UpdateRects()
	{
		if(!m_pWinData->hWnd)
			return;

		GetClientRect(m_pWinData->hWnd, &m_pWinData->clientRect);
		GetWindowRect(m_pWinData->hWnd, &m_pWinData->windowRect);

		m_pWinData->posX = m_pWinData->clientRect.left;
		m_pWinData->posY = m_pWinData->clientRect.top;
		m_pWinData->width = m_pWinData->clientRect.right - m_pWinData->clientRect.left;
		m_pWinData->height = m_pWinData->clientRect.bottom - m_pWinData->clientRect.top;

		m_pWinData->winWidth = m_pWinData->windowRect.right - m_pWinData->windowRect.left;
		m_pWinData->winHeight = m_pWinData->windowRect.bottom - m_pWinData->windowRect.top;
	}

	// Method for removing this window from memory.
	void CWinInit::Release()
	{
		if(m_pWinData->hWnd)
		{
			DestroyWindow(m_pWinData->hWnd);
			m_pWinData->hWnd = NULL;
		}

		UnregisterClass(m_pWinData->pWindowTitle, m_pWinData->hInstance);

		SAFE_DELETE(m_pWinData);
	}
};