//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CDebugBase.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDEBUGBASE_H
#define CDEBUGBASE_H

#include "CMatDebug.h"
#include "CGraphicalObject.h"
#include "CRenderingData.h"
#include "..\Utilities\CDataQueue.h"

namespace GFX
{
	// CDebugBase class.
	// Base class for creating line-list based geometry that is primarily used for debug purposes.
	class CDebugBase : public CGraphicalObject
	{
	public:
		// Default debug vertex structure.
		struct DEBUG_VERTEX
		{
			Math::CVector3					Position;
		};

		// Default debug line structure.
		struct DEBUG_LINE
		{
			Math::CVector3					p0;
			Math::CVector3					p1;

			DEBUG_LINE()
			{
			}

			DEBUG_LINE(Math::CVector3 P0, Math::CVector3 P1)
			{
				p0 = P0;
				p1 = P1;
			}
		};

	public:
		CDebugBase();
		CDebugBase(const CDebugBase& other);
		virtual ~CDebugBase();

		int									Initialize() override;
		int									Render() override;
		void								Release() override;

		inline bool							SetCBuffer() const;
		inline void							SetCBuffer(bool setCBuffer);

		inline RENDERDATA*					RenderData() const;
		inline void							RenderData(RENDERDATA* pRenderData);

		DEBUG_CBUFFER						m_debugCBuffer;

	protected:
		void								AddLine(DEBUG_LINE* line);
		
		bool								m_bSetCBuffer;

		RENDERDATA*							m_pRenderData;

	private:
		Util::CPtrQueue						m_lineQueue;
		u32									m_lineCount;

		DEBUG_VERTEX*						m_pVertices;
		u32									m_vertexCount;
		
		ID3D11Buffer*						m_pVertexBuffer;
	};

	// Set constant buffer getter and setter.

	inline bool CDebugBase::SetCBuffer() const
	{
		return m_bSetCBuffer;
	}

	inline void CDebugBase::SetCBuffer(bool setCBuffer)
	{
		m_bSetCBuffer = setCBuffer;
	}

	// Render data getter and setter.

	inline RENDERDATA* CDebugBase::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CDebugBase::RenderData(RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}
};

#endif