//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CFBXLoader.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CFBXLOADER_H
#define CFBXLOADER_H

#include "..\Globals\CGlobals.h"
#include "..\Utilities\CDebugError.h"
#include "..\Utilities\CDebugLog.h"
#include "..\Utilities\CDataStack.h"
#include "..\Utilities\CSortInsertion.h"
#include "CMeshScene.h"

namespace GFX
{
	// Structure for FBX imported triangles.
	struct FBX_TRIANGLE
	{
		int				polygonIndex;
		int				subsetIndex;
		int				dccVertexIndex[3];
		Math::CVector3	point[3];
		Math::CVector3	normal[3];
		Math::CVector3	binormal[3];
		Math::CVector3	tangent[3];
		Math::CColor	color[3];
		Math::CVector2	uv[3][8];

		// Function for calculating the tangent and binormal of a triangle.
		void CalculateTangentBinormal()
		{
			Math::CVector3 v2v1, v3v1;
			
			v2v1 = (point[1] - point[0]);
			v3v1 = (point[2] - point[0]);

			float c2c1t = uv[1][0].x - uv[0][0].x;
			float c2c1b = uv[1][0].y - uv[0][0].y;

			float c3c1t = uv[2][0].x - uv[0][0].x;
			float c3c1b = uv[2][0].y - uv[0][0].y;
			
			tangent[0] = Math::CVector3(c3c1b * v2v1.x - c2c1b * v3v1.x, c3c1b * v2v1.y - c2c1b * v3v1.y, c3c1b * v2v1.z - c2c1b * v3v1.z);
			binormal[0] = normal[0].Cross(tangent[0]).Normalized();
			tangent[0] = binormal[0].Cross(normal[0]).Normalized();

			tangent[1] = tangent[2] = tangent[0];
			binormal[1] = binormal[2] = binormal[0];
		}
	};

	// CFBXLoader class
	// Class for loading a fbx mesh, and its materials into the game.
	// Notice: Only works with static meshes currently.
	class CFBXLoader
	{
	public:
		CFBXLoader();
		CFBXLoader(const CFBXLoader& other);
		virtual ~CFBXLoader();

		int									LoadFBXFile(char* filename, CMeshScene*& meshScene, float scale = 0.1f);

		void								Release();

	private:
		int									LoadScene(char* filename);

		int									ParseNode(FbxNode* node);
		int									ParseMesh(FbxNode* node, FbxMesh* mesh);
		int									ParseMaterialInLayer(FbxMesh* mesh, CMesh* gameMesh, FbxLayer* layer, int materialIndex);

		int									ExtractTextures(FbxProperty prop, const char* paramName, CMatMesh* material, u32 flags);
		void								AddTextureParameter(CMatMesh* material, const char* paramName, int index, const char* filename, u32 flags);
		
		FbxAMatrix							CalculateGlobalTransform(FbxNode* node);

		CMatMesh*							FindMaterial(u64 id);
		CMatMesh*							FindMaterial(const std::string& name);

		float								m_scale;

		FBX_TRIANGLE						m_triangle;

		CMeshScene*							m_pMeshScene;
		FbxScene*							m_pScene;
	};
};

#endif