//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CGraphicalObject.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CGRAPHICALOBJECT_H
#define CGRAPHICALOBJECT_H

#include "..\Application\CDX11Object.h"
#include "..\Math\CMathTransform.h"
#include "CMaterial.h"

namespace GFX
{
	// CGraphicalObject class
	// Base class for objects that are graphical objects that require a transform and a material.
	class CGraphicalObject : public App::CDX11Object
	{
	public:
		CGraphicalObject();
		CGraphicalObject(const CGraphicalObject& other);
		virtual ~CGraphicalObject();

		virtual int							Initialize();
		virtual int							Render();
		virtual void						Release();

		inline Math::CTransformSharedPtr	Transform() const;
		inline void							Transform(Math::CTransformSharedPtr pTransform);
		inline CMaterial*					Material() const;
		inline void							Material(CMaterial* pMaterial);

	protected:
		Math::CTransformSharedPtr			m_pTransform;
		CMaterial*							m_pMaterial;
	};

	// Transform getter and setter.

	inline Math::CTransformSharedPtr CGraphicalObject::Transform() const
	{
		return m_pTransform;
	}

	inline void CGraphicalObject::Transform(Math::CTransformSharedPtr pTransform)
	{
		m_pTransform = pTransform;
	}

	// Material getter and setter.

	inline CMaterial* CGraphicalObject::Material() const
	{
		return m_pMaterial;
	}

	inline void CGraphicalObject::Material(CMaterial* pMaterial)
	{
		m_pMaterial = pMaterial;
	}
};

#endif