//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CLightPoint.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CLightPoint.h"

namespace GFX
{
	CLightPoint::CLightPoint()
	{
		m_pointLightCount					= 0;

		m_pDebugSphere						= NULL;

		m_shadowMapSize						= 256;
		
		m_pointShadowMapCount				= 0;
		m_pPointLightArray					= NULL;
		m_pPointLightPOVs					= NULL;
		m_pPointShadowMaps					= NULL;

		m_pMatShadowMap						= NULL;
		m_pRenderData						= NULL;
		
		m_pPointLightCBuffer				= NULL;

		m_pMatDebug							= NULL;
		m_pCurrentCamera					= NULL;
	}

	CLightPoint::CLightPoint(const CLightPoint& other)
	{
		
	}

	CLightPoint::~CLightPoint()
	{
		
	}
	
	// Method for initializing all point lights for the game.
	int CLightPoint::Initialize()
	{
		int retCode = Util::Err::NONE;

		u32 i;

		// Point lights.
		if(m_pointLightCount > 0)
		{
			m_pPointLightArray		= new POINT_LIGHT[m_pointLightCount];
			m_pPointLightPOVs		= new Math::CCamera*[m_pointLightCount];

			ZeroMemory(m_pPointLightArray, sizeof(POINT_LIGHT) * m_pointLightCount);
			ZeroMemory(m_pPointLightPOVs, sizeof(Math::CCamera*) * m_pointLightCount);
			
			i = 0;
			POINT_LIGHT_QUEUE* data;
			while((data = (POINT_LIGHT_QUEUE*)m_qPointLightQueue.Pop()))
			{
				m_pPointLightArray[i].color						= data->color;
				m_pPointLightArray[i].position					= data->position;
				m_pPointLightArray[i].attenuation				= data->attenuation;
				m_pPointLightArray[i].range						= data->range + 2.5f;
				m_pPointLightArray[i].strength					= data->strength;
				m_pPointLightArray[i].shadowsOn					= data->shadowsOn;
				m_pPointLightArray[i].shadowStrength			= data->shadowStrength;
				if(data->lightPtr)
					*data->lightPtr								= &m_pPointLightArray[i];
				
				m_pPointLightPOVs[i] = new Math::CCamera();
				m_pPointLightPOVs[i]->CameraMode(Math::CAMERA_MODE::Perspective);
				m_pPointLightPOVs[i]->FarView(m_pPointLightArray[i].range);
				m_pPointLightPOVs[i]->NearView(m_pPointLightArray[i].range / 27.5f);
				m_pPointLightPOVs[i]->Width(50.0f);
				m_pPointLightPOVs[i]->Height(50.0f);
				m_pPointLightPOVs[i]->FieldOfView(Math::_PiOver2);
				m_pPointLightPOVs[i]->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
				m_pPointLightPOVs[i]->Initialize(1.0f);
				m_pPointLightPOVs[i]->Update();
				
				SAFE_DELETE(data);
				i++;
			}
			
			m_pointShadowMapCount = 0;

			for(i = 0; i < m_pointLightCount; i++)
			{
				if(m_pPointLightArray[i].shadowsOn)
				{
					m_pointShadowMapCount++;
				}
			}
			
			if(m_pointShadowMapCount > 0)
			{
				m_pPointShadowMaps = new CTextureRenderTo();
				m_pPointShadowMaps->DirectXData(m_pDX11Data);

				if((retCode = m_pPointShadowMaps->Initialize(m_shadowMapSize, m_shadowMapSize, m_pointShadowMapCount * 6, true)) != Util::Err::NONE)
					return retCode;
			}
		}

		// Store the light counts in strings so that they can be used as shader macros.
		m_pointLightCountStr = std::to_string(m_pointLightCount);
		
		if(m_pointLightCount > 0)
		{
			m_pPointLightCBuffer	= new CConstantBuffer();

			m_pPointLightCBuffer->DirectXData(m_pDX11Data);
			if((retCode = m_pPointLightCBuffer->Initialize(sizeof(POINT_LIGHT) * m_pointLightCount, true)) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}

	int CLightPoint::ResetBuffer()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pPointLightCBuffer) if((retCode = m_pPointLightCBuffer->ResetBuffer(m_pPointLightArray)) != Util::Err::NONE) return retCode;

		return retCode;
	}

	int CLightPoint::BindBuffer()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pPointLightCBuffer) if((retCode = m_pPointLightCBuffer->BindBufferPS(12)) != Util::Err::NONE) return retCode;

		return retCode;
	}
	
	// Method for rendering point light shadow maps.
	int CLightPoint::RenderMaps(void* data, int (*RenderWorldObjects)(void*, CMaterial*))
	{
		int retCode = Util::Err::NONE;
		
		u32 i;
		u32 index;

		// Point lights.
		if(m_pointLightCount > 0)
		{
			index = 0;
			int j;
			
			for(i = 0; i < m_pointLightCount; i++)
			{
				if(m_pPointLightArray[i].shadowsOn)
				{
					m_pPointLightPOVs[i]->Transform()->Position(*(Math::CVector3*)&m_pPointLightArray[i].position);

					for(j = 0; j < 6; j++)
					{
						switch(j)
						{
							case 0:
								m_pPointLightPOVs[i]->Transform()->Euler(Math::CVector3(Math::_PiOver2, 0, 0));
								break;
							case 1:
								m_pPointLightPOVs[i]->Transform()->Euler(Math::CVector3(-Math::_PiOver2, 0, 0));
								break;
							case 2:
								m_pPointLightPOVs[i]->Transform()->Euler(Math::CVector3(0, Math::_PiOver2, 0));
								break;
							case 3:
								m_pPointLightPOVs[i]->Transform()->Euler(Math::CVector3(0, Math::_PiOver2 * 2, 0));
								break;
							case 4:
								m_pPointLightPOVs[i]->Transform()->Euler(Math::CVector3(0, Math::_PiOver2 * 3, 0));
								break;
							case 5:
								m_pPointLightPOVs[i]->Transform()->Euler(Math::CVector3(0, Math::_PiOver2 * 4, 0));
								break;
							default:
								break;
						}

						m_pPointLightPOVs[i]->Update();

						m_pPointLightArray[i].viewProj[j] = m_pPointLightPOVs[i]->ProjectionMatrix() * m_pPointLightPOVs[i]->ViewMatrix();

						m_pPointShadowMaps->SetAsRenderTarget(true, index);
						m_pPointShadowMaps->ClearRenderTarget(DX11_CLEAR_COLOR, Math::CColor(0), true, index);

						m_pMatShadowMap->m_shadowCBuffer.shadowPerspective = m_pPointLightArray[i].viewProj[j];
						m_pMatShadowMap->Camera(m_pPointLightPOVs[i]);

						if((retCode = RenderWorldObjects(data, m_pMatShadowMap)) != Util::Err::NONE)
							return retCode;

						index++;
					}
				}
			}
			
			if(m_pPointShadowMaps)
			{
				m_pPointShadowMaps->ResetRenderTarget(false);

				m_pDevCon->PSSetShaderResources(31, 1, m_pPointShadowMaps->ppShaderResourceView());
			}
		}

		return retCode;
	}
	
	// Method for render point light debug elements.
	int CLightPoint::Render()
	{
		int retCode = Util::Err::NONE;

		// Render point light debug spheres.
		for(u32 i = 0; i < m_pointLightCount; i++)
		{
			m_pDebugSphere->Transform()->Position(*(Math::CVector3*)&m_pPointLightArray[i].position);
			m_pDebugSphere->Transform()->Update();

			m_pDebugSphere->TargetCamera(m_pCurrentCamera);

			m_pDebugSphere->m_debugCBuffer.color = m_pPointLightArray[i].color;
			m_pDebugSphere->m_debugCBuffer.scale = Math::CVector3(0.25f, 0.25f, 0.25f);
			m_pDebugSphere->LookAtCamera(true);
			if((retCode = m_pDebugSphere->Render()) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}	
	
	// Method for release all point light data.
	void CLightPoint::Release()
	{
		SAFE_RELEASE_DELETE(m_pPointLightCBuffer);
		
		// Release point lights.
		SAFE_RELEASE_DELETE(m_pPointShadowMaps);
		
		if(m_pPointLightPOVs)
			for(u32 i = 0; i < m_pointLightCount; i++)
				SAFE_DELETE(m_pPointLightPOVs[i]);
				
		SAFE_DELETE_ARRAY(m_pPointLightPOVs);
		SAFE_DELETE_ARRAY(m_pPointLightArray);
		
		m_qPointLightQueue.Release();
	}
};