//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CLightSpot.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CLightSpot.h"

namespace GFX
{
	CLightSpot::CLightSpot()
	{
		m_spotlightCount					= 0;

		m_pDebugSphere						= NULL;

		m_shadowMapSize						= 256;
		
		m_spotShadowMapCount				= 0;
		m_pSpotlightArray					= NULL;
		m_pSpotlightPOVs					= NULL;
		m_pSpotShadowMaps					= NULL;

		m_pMatShadowMap						= NULL;
		m_pRenderData						= NULL;
		
		m_pSpotlightCBuffer					= NULL;

		m_pMatDebug							= NULL;
		m_pCurrentCamera					= NULL;
	}

	CLightSpot::CLightSpot(const CLightSpot& other)
	{
		
	}

	CLightSpot::~CLightSpot()
	{
		
	}

	// Method for initializing all spotlights for the game.
	int CLightSpot::Initialize()
	{
		int retCode = Util::Err::NONE;

		u32 i;

		// Spotlights.
		if(m_spotlightCount > 0)
		{
			m_pSpotlightArray		= new SPOTLIGHT[m_spotlightCount];
			m_pSpotlightPOVs		= new Math::CCamera*[m_spotlightCount];

			ZeroMemory(m_pSpotlightArray, sizeof(SPOTLIGHT) * m_spotlightCount);
			ZeroMemory(m_pSpotlightPOVs, sizeof(Math::CCamera*) * m_spotlightCount);
			
			i = 0;
			SPOTLIGHT_QUEUE* data;

			// Store and create all spotlight data.
			while((data = (SPOTLIGHT_QUEUE*)m_qSpotlightQueue.Pop()))
			{
				m_pSpotlightArray[i].viewProj					= data->viewProj;
				m_pSpotlightArray[i].color						= data->color;
				m_pSpotlightArray[i].direction					= data->direction;
				m_pSpotlightArray[i].position					= data->position;
				m_pSpotlightArray[i].attenuation				= data->attenuation;
				m_pSpotlightArray[i].range						= data->range + 2.5f;
				m_pSpotlightArray[i].strength					= data->strength;
				m_pSpotlightArray[i].falloff					= data->falloff;
				m_pSpotlightArray[i].phi						= cos(data->phi * 0.5f);
				m_pSpotlightArray[i].theta						= cos(data->theta * 0.5f);
				m_pSpotlightArray[i].shadowsOn					= data->shadowsOn;
				m_pSpotlightArray[i].shadowStrength				= data->shadowStrength;
				if(data->lightPtr)
					*data->lightPtr								= &m_pSpotlightArray[i];

				m_pSpotlightPOVs[i] = new Math::CCamera();
				m_pSpotlightPOVs[i]->CameraMode(Math::CAMERA_MODE::Perspective);
				m_pSpotlightPOVs[i]->FarView(m_pSpotlightArray[i].range);
				m_pSpotlightPOVs[i]->NearView(m_pSpotlightArray[i].range / 27.5f);
				m_pSpotlightPOVs[i]->Width(50.0f);
				m_pSpotlightPOVs[i]->Height(50.0f);
				m_pSpotlightPOVs[i]->FieldOfView(data->phi);
				m_pSpotlightPOVs[i]->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
				m_pSpotlightPOVs[i]->Initialize(1.0f);
				m_pSpotlightPOVs[i]->Update();
				
				SAFE_DELETE(data);
				i++;
			}
			
			m_spotShadowMapCount = 0;

			for(i = 0; i < m_spotlightCount; i++)
			{
				if(m_pSpotlightArray[i].shadowsOn)
				{
					m_spotShadowMapCount++;
				}
			}
			
			// Generate shadow map render to texture if any lights have shadows enabled.
			if(m_spotShadowMapCount > 0)
			{
				m_pSpotShadowMaps = new CTextureRenderTo();
				m_pSpotShadowMaps->DirectXData(m_pDX11Data);

				if((retCode = m_pSpotShadowMaps->Initialize(m_shadowMapSize, m_shadowMapSize, m_spotShadowMapCount, true)) != Util::Err::NONE)
					return retCode;
			}
		}

		// Store the light counts in strings so that they can be used as shader macros.
		m_spotlightCountStr = std::to_string(m_spotlightCount);
		
		if(m_spotlightCount > 0)
		{
			m_pSpotlightCBuffer		= new CConstantBuffer();

			m_pSpotlightCBuffer->DirectXData(m_pDX11Data);
			if((retCode = m_pSpotlightCBuffer->Initialize(sizeof(SPOTLIGHT) * m_spotlightCount, true)) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}

	int CLightSpot::ResetBuffer()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pSpotlightCBuffer) if((retCode = m_pSpotlightCBuffer->ResetBuffer(m_pSpotlightArray)) != Util::Err::NONE) return retCode;

		return retCode;
	}

	int CLightSpot::BindBuffer()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pSpotlightCBuffer) if((retCode = m_pSpotlightCBuffer->BindBufferPS(13)) != Util::Err::NONE) return retCode;

		return retCode;
	}

	// Method for rendering spotlight shadow maps.
	int CLightSpot::RenderMaps(void* data, int (*RenderWorldObjects)(void*, CMaterial*))
	{
		int retCode = Util::Err::NONE;
		
		u32 i;
		u32 index;

		// Spotlights.
		if(m_spotlightCount > 0)
		{
			index = 0;

			for(i = 0; i < m_spotlightCount; i++)
			{
				if(m_pSpotlightArray[i].shadowsOn)
				{
					m_pSpotlightPOVs[i]->Transform()->Position(*(Math::CVector3*)&m_pSpotlightArray[i].position);
					m_pSpotlightPOVs[i]->Transform()->LookAt(*(Math::CVector3*)&m_pSpotlightArray[i].direction);
					m_pSpotlightPOVs[i]->Update();
					m_pSpotlightArray[i].viewProj = m_pSpotlightPOVs[i]->ProjectionMatrix() * m_pSpotlightPOVs[i]->ViewMatrix();
					
					m_pSpotShadowMaps->SetAsRenderTarget(true, index);
					m_pSpotShadowMaps->ClearRenderTarget(DX11_CLEAR_COLOR, Math::CColor(0), true, index);

					m_pMatShadowMap->m_shadowCBuffer.shadowPerspective = m_pSpotlightArray[i].viewProj;
					m_pMatShadowMap->Camera(m_pSpotlightPOVs[i]);

					if((retCode = RenderWorldObjects(data, m_pMatShadowMap)) != Util::Err::NONE)
						return retCode;

					index++;
				}
			}

			if(m_pSpotShadowMaps)
			{
				m_pSpotShadowMaps->ResetRenderTarget(false);

				m_pDevCon->PSSetShaderResources(32, 1, m_pSpotShadowMaps->ppShaderResourceView());
			}
		}

		return retCode;
	}
	
	// Method for render spotlight debug elements.
	int CLightSpot::Render()
	{
		int retCode = Util::Err::NONE;

		for(u32 i = 0; i < m_spotlightCount; i++)
		{
			m_pDebugSphere->Transform()->Position(*(Math::CVector3*)&m_pSpotlightArray[i].position);
			m_pDebugSphere->Transform()->Update();

			m_pDebugSphere->TargetCamera(m_pCurrentCamera);

			m_pDebugSphere->m_debugCBuffer.color = m_pSpotlightArray[i].color;
			m_pDebugSphere->m_debugCBuffer.scale = Math::CVector3(0.25f, 0.25f, 0.25f);
			m_pDebugSphere->LookAtCamera(true);
			if((retCode = m_pDebugSphere->Render()) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}	
	
	// Method for release all spotlight data.
	void CLightSpot::Release()
	{
		SAFE_RELEASE_DELETE(m_pSpotlightCBuffer);
		
		SAFE_RELEASE_DELETE(m_pSpotShadowMaps);
		
		if(m_pSpotlightPOVs)
			for(u32 i = 0; i < m_spotlightCount; i++)
				SAFE_DELETE(m_pSpotlightPOVs[i]);
				
		SAFE_DELETE_ARRAY(m_pSpotlightPOVs);
		SAFE_DELETE_ARRAY(m_pSpotlightArray);
		
		m_qSpotlightQueue.Release();
	}
};