//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatMesh.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatMesh.h"

namespace GFX
{
	CMatMesh::CMatMesh()
	{
		m_bTransparent						= false;
		m_bufferCount						= 3;
		m_textureCount						= 5;
		m_elementCount						= 9;
		m_bUseInverse						= true;

		ZeroMemory(&m_colorBuffer, sizeof(m_colorBuffer));
		ZeroMemory(&m_lightingBuffer, sizeof(m_lightingBuffer));
	}

	CMatMesh::CMatMesh(const CMatMesh& other)
	{
	}

	CMatMesh::~CMatMesh()
	{
	}
	
	// Method for setting up the mesh material shader.
	int CMatMesh::SetupShader()
	{
		int retCode = Util::Err::NONE;

		m_pIed									= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];
		
		int index = 0;
		m_pIed[index++]	= { "POSITION",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "NORMAL",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "BINORMAL",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "TANGENT",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "COLOR",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		
		for(u32 i = 0; i < 4; i++)
			m_pIed[index++]	= { "TEXCOORD",		i,		DXGI_FORMAT_R32G32_FLOAT,			0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		
		m_pMacros = new D3D_SHADER_MACRO[2];
		m_pMacros[0] = { "SPECULAR_ON", "true" };
		m_pMacros[1] = { NULL, NULL };
		
		m_pVertexResName	= MAKEINTRESOURCE(HLSL_VMESH);
		m_pPixelResName		= MAKEINTRESOURCE(HLSL_PMESH);
		m_pLogFilename		= ".\\MeshShader.txt";
		
		// Setup the constant buffers.
		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(MESH_COLOR_BUFFER), true)) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pShader->InitializeConstantBuffer(2, sizeof(MESH_LIGHTING_BUFFER), true)) != Util::Err::NONE)
			return retCode;

		// Setup textures.
		m_textureCount = 5;
		m_pTextureArray = new CTexture*[m_textureCount];
		for(u32 i = 0; i < m_textureCount; i++)
			m_pTextureArray[i] = NULL;

		if((retCode = OpenTextureFiles()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for opening all inputted textures from filenames.
	int CMatMesh::OpenTextureFiles()
	{
		int retCode = Util::Err::NONE;

		m_textureFlags = 0;

		CMeshMaterialParameter param;
		for(u32 i = 0; i < m_sMaterialParamStack.Length(); i++)
		{
			param = m_sMaterialParamStack[i];
			
			if(param.flags & MMP_TEXTURE)
			{
				CTexture* tex = CTextureList::Instance().AddTexture(param.valString);
				if(!tex) return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D;

				m_textureFlags |= param.flags & MMP_TEXTURE;

				// Apply the texture to the correct texture array index.
				switch(param.flags & MMP_TEXTURE)
				{
					case MMP_ALPHA_MAP:
						m_pTextureArray[0] = tex;
						break;
					case MMP_DIFFUSE_MAP:
						m_pTextureArray[1] = tex;
						break;
					case MMP_BUMP_MAP:
						m_pTextureArray[2] = tex;
						break;
					case MMP_SPECULAR_MAP:
						m_pTextureArray[3] = tex;
						break;
					case MMP_EMISSIVE_MAP:
						m_pTextureArray[4] = tex;
						break;
					default:
						m_pTextureArray[1] = tex;
						break;
				}
			}
		}

		m_colorBuffer.textureFlags = m_textureFlags;

		return retCode;
	}
	
	// Method for setting the mesh material for render.
	int CMatMesh::Render()
	{
		int retCode = Util::Err::NONE;

		m_lightingBuffer.cameraPoint = m_pCamera->Transform()->Position();
		
		if((retCode = m_pShader->ResetConstantBuffer(1, &m_colorBuffer)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->ResetConstantBuffer(2, &m_lightingBuffer)) != Util::Err::NONE) return retCode;

		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;

		// Bind constant buffers to shader.
		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->BindCBuffer(2, m_shaderType)) != Util::Err::NONE) return retCode;
		
		m_mtxInvCBuffer.projectionInv = m_pCamera->ProjectionMatrixInv();
		m_mtxInvCBuffer.viewInv = m_pCamera->ViewMatrixInv();
		m_mtxInvCBuffer.worldInv = m_pTransform->WorldMatrixInv();

		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CMatMesh::Release()
	{
		SAFE_DELETE_ARRAY(m_pTextureArray);
		SAFE_DELETE_ARRAY(m_pMacros);

		CMaterial::Release();
	}

	// Method for checking for a name parameter,
	bool CMatMesh::CheckForParameter(const std::string& name)
	{
		for(u32 i = 0; i < m_sMaterialParamStack.Length(); i++)
		{
			if(m_sMaterialParamStack[i].name == name)
			{
				return true;
			}
		}

		return false;
	}
};