//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatSkydome.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatSkydome.h"

namespace GFX
{
	CMatSkydome::CMatSkydome()
	{
		m_elementCount						= 1;
		m_bufferCount						= 2;
		m_textureCount						= 1;
	}

	CMatSkydome::CMatSkydome(const CMatSkydome& other)
	{
	}

	CMatSkydome::~CMatSkydome()
	{
	}
	
	// Method for setting up the skydome material shader.
	int CMatSkydome::SetupShader()
	{
		int retCode = Util::Err::NONE;

		m_pIed			= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];

		m_pIed[0]	= { "POSITION",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		0,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		
		m_pVertexResName		= MAKEINTRESOURCE(HLSL_VSKYDOME);
		m_pPixelResName			= MAKEINTRESOURCE(HLSL_PSKYDOME);
		m_pLogFilename			= ".\\SkydomeShader.txt";
		
		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(SKYDOME_CBUFFER), true)) != Util::Err::NONE)
			return retCode;

		ZeroMemory(&m_skydomeBuffer, sizeof(m_skydomeBuffer));

		m_skydomeBuffer.color = Math::CColor(1, 1, 1, 1);

		return retCode;
	}
	
	// Method for setting the skydome material for render.
	int CMatSkydome::Render()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = m_pShader->ResetConstantBuffer(1, &m_skydomeBuffer)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE) return retCode;
		
		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CMatSkydome::Release()
	{
		CMaterial::Release();
	}
};