//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatUIImage.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatUIImage.h"

namespace GFX
{
	CMatUIImage::CMatUIImage()
	{
		m_elementCount						= 6;
		m_bufferCount						= 2;
		m_textureCount						= 1;
	}

	CMatUIImage::CMatUIImage(const CMatUIImage& other)
	{
	}

	CMatUIImage::~CMatUIImage()
	{
	}

	// Method for setting up the UI image material shader.
	int CMatUIImage::SetupShader()
	{
		int retCode = Util::Err::NONE;
		
		m_pIed								= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];

		m_pIed[0]	= { "POSITION",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		0,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[1]	= { "TEXCOORD",			0,		DXGI_FORMAT_R32G32_FLOAT,			0,		12,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[2]	= { "INSTPOS",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		0,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[3]	= { "INSTSCALE",		0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		12,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[4]	= { "INSTUV",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		24,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[5]	= { "INSTCOLOR",		0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		40,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };

		m_pVertexResName		= MAKEINTRESOURCE(HLSL_VUIIMAGE);
		m_pGeometryResName		= MAKEINTRESOURCE(HLSL_GUIIMAGE);
		m_pPixelResName			= MAKEINTRESOURCE(HLSL_PUIIMAGE);
		m_pLogFilename			= ".\\UIImageShader.txt";

		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(UIIMAGE_CBUFFER), true)) != Util::Err::NONE)
			return retCode;

		ZeroMemory(&m_imageCBuffer, sizeof(m_imageCBuffer));

		m_imageCBuffer.color			= Math::CColor(1.0f, 1.0f, 1.0f);
		m_imageCBuffer.textured			= true;
		m_imageCBuffer.pointFilter		= false;

		return retCode;
	}
	
	// Method for setting the UI image material for render.
	int CMatUIImage::Render()
	{
		int retCode = Util::Err::NONE;

		m_imageCBuffer.textured = m_pTexture ? true : false;
		
		if((retCode = m_pShader->ResetConstantBuffer(1, &m_imageCBuffer)) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;

		// Bind constant buffers to shader.
		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE) return retCode;

		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CMatUIImage::Release()
	{
		CMaterial::Release();
	}
};