//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CParticleSystem.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CParticleSystem.h"

namespace GFX
{
	CParticleSystem::CParticleSystem()
	{
		m_pTimer							= NULL;
		m_pCamera							= NULL;

		m_pParticleMaterial					= NULL;
		m_pRenderData						= NULL;
	}

	CParticleSystem::CParticleSystem(const CParticleSystem& other)
	{
	}

	CParticleSystem::~CParticleSystem()
	{
	}

	// Method for initializing a particle system and its devices.
	int CParticleSystem::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Allocate memory.
		m_pParticleMaterial					= new CMatParticle();

		m_pParticleMaterial->DirectXData(m_pDX11Data);
		m_pParticleMaterial->RenderData(m_pRenderData);
		m_pParticleMaterial->Camera(m_pCamera);
		if((retCode = m_pParticleMaterial->Initialize()) != Util::Err::NONE)
			return retCode;
			
		// Initialize each if the particle devices stored in this particle system.
		CParticleDevice* pDevice;
		for(u32 i = 0; i < m_sParticleStack.Length(); i++)
		{
			pDevice = m_sParticleStack[i];
			
			pDevice->Timer(m_pTimer);

			if(!pDevice->Material())
				pDevice->Material(m_pParticleMaterial);
			else
				pDevice->Material()->Camera(m_pCamera);
				
			pDevice->RenderData(m_pRenderData);
			pDevice->DirectXData(m_pDX11Data);
			
			if((retCode = pDevice->Initialize()) != Util::Err::NONE)
				return retCode;
		}
		
		return retCode;
	}

	// Method for updating a the particle system and all of its particle devices.
	int CParticleSystem::Update()
	{
		int retCode = Util::Err::NONE;
		
		CParticleDevice* pDevice;
		for(u32 i = 0; i < m_sParticleStack.Length(); i++)
		{
			pDevice = m_sParticleStack[i];

			if((retCode = pDevice->Update()) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}
	
	// Method for rendering a the particle system and all of its particle devices.
	int CParticleSystem::Render()
	{
		int retCode = Util::Err::NONE;
		
		CParticleDevice* pDevice;
		for(u32 i = 0; i < m_sParticleStack.Length(); i++)
		{
			pDevice = m_sParticleStack[i];

			m_pParticleMaterial->m_particleCBuffer.color			= pDevice->m_color;
			m_pParticleMaterial->m_particleCBuffer.softness			= pDevice->m_softness;
			m_pParticleMaterial->m_particleCBuffer.billboardType	= (int)pDevice->m_billboardType;
			m_pParticleMaterial->m_particleCBuffer.time				= pDevice->m_playbackTimer;

			if((retCode = pDevice->Render()) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}

	// Method for releasing the particle system and all of its devices.
	void CParticleSystem::Release()
	{
		for(u32 i = 0; i < m_sParticleStack.Length(); i++)
		{
			SAFE_RELEASE_DELETE(m_sParticleStack[i]);
		}

		SAFE_RELEASE_DELETE(m_pParticleMaterial);
	}
};