//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CParticleSystem.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CPARTICLESYSTEM_H
#define CPARTICLESYSTEM_H

#include "..\Application\CDX11Object.h"
#include "..\Utilities\CDebugError.h"
#include "CParticleDevice.h"
#include "CTextureRenderTo.h"
#include "..\Utilities\CDataStack.h"
#include "..\Utilities\CTimer.h"
#include "CMatParticle.h"
#include "CMatShadowMap.h"
#include "CRenderingData.h"

namespace GFX
{
	// CParticleSystem class.
	// Class for creating a particle system that stores particle devices in it.
	class CParticleSystem : public App::CDX11Object
	{
	public:
		CParticleSystem();
		CParticleSystem(const CParticleSystem& other);
		virtual ~CParticleSystem();

		int									Initialize();
		int									Update();
		int									Render();

		void								Release();

		inline Util::CTimer*				Timer();
		inline void							Timer(Util::CTimer* pTimer);
		inline Math::CCamera*				Camera();
		inline void							Camera(Math::CCamera* pCamera);

		inline void							SetRenderData(RENDERDATA* pRenderData);
		inline void							AddParticleDevice(CParticleDevice*& pParticleDevice);

	private:
		Util::CStack<CParticleDevice*>		m_sParticleStack;

		Util::CTimer*						m_pTimer;
		Math::CCamera*						m_pCamera;
		
		CMatParticle*						m_pParticleMaterial;
		RENDERDATA*							m_pRenderData;
	};

	// Timer getter and setter.

	inline Util::CTimer* CParticleSystem::Timer()
	{
		return m_pTimer;
	}

	inline void CParticleSystem::Timer(Util::CTimer* pTimer)
	{
		m_pTimer = pTimer;
	}

	// Camera getter and setter.

	inline Math::CCamera* CParticleSystem::Camera()
	{
		return m_pCamera;
	}

	inline void CParticleSystem::Camera(Math::CCamera* pCamera)
	{
		m_pCamera = pCamera;
	}

	// Render data setter.
	inline void CParticleSystem::SetRenderData(RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}
	
	// Method for adding a particle device to this particle system.
	inline void CParticleSystem::AddParticleDevice(CParticleDevice*& pParticleDevice)
	{
		m_sParticleStack.Push(pParticleDevice);
	}
};

#endif