//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CPostScreenQuad.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CPostScreenQuad.h"

namespace GFX
{
	CPostScreenQuad::CPostScreenQuad()
	{
		m_pScreenWidth						= NULL;
		m_pScreenHeight						= NULL;

		m_pLighting							= NULL;
		m_pTextureProjection				= NULL;

		m_pTextureRenderTo					= NULL;
	}

	CPostScreenQuad::CPostScreenQuad(const CPostScreenQuad& other)
	{
	}

	CPostScreenQuad::~CPostScreenQuad()
	{
	}

	// Method for setting pointers to the screen width and height values.
	void CPostScreenQuad::SetScreenSize(int* width, int* height)
	{
		m_pScreenWidth = width;
		m_pScreenHeight = height;
	}

	// Method for initializing the post screen quads material, g-buffer render to texture, and its quad.
	int CPostScreenQuad::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Allocate memory.
		m_pMaterial				= new CMatPostScreen();
		m_pTextureRenderTo		= new CTextureRenderTo();

		m_pTransform			= Math::CTransformSharedPtr(new Math::CTransform());
		m_pTransform->Update();

		// Setup the post screen material.
		m_pMaterial->RenderData(m_pRenderData);
		m_pMaterial->DirectXData(m_pDX11Data);
		((CMatPostScreen*)m_pMaterial)->Lighting(m_pLighting);
		((CMatPostScreen*)m_pMaterial)->TextureProjection(m_pTextureProjection);

		if((retCode = m_pMaterial->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pMaterial->Camera()->Initialize((float)*m_pScreenWidth, (float)*m_pScreenHeight);

		// Setup render to textures.
		m_pTextureRenderTo->DirectXData(m_pDX11Data);
		if((retCode = m_pTextureRenderTo->Initialize(*m_pScreenWidth, *m_pScreenHeight, POST_TEXTURE_COUNT, false)) != Util::Err::NONE)
			return retCode;

		m_vertexStride = sizeof(SCREENQUAD_VERTEX);
		m_indexStride = sizeof(u16);
		m_instanceStride = 0;

		// Create the quad.
		
		AddQuad(
			new SCREENQUAD_VERTEX(Math::CVector3(-1.0f, -1.0f, 0.0f), Math::CVector2(0.0f, 1.0f)),
			new SCREENQUAD_VERTEX(Math::CVector3( 1.0f, -1.0f, 0.0f), Math::CVector2(1.0f, 1.0f)),
			new SCREENQUAD_VERTEX(Math::CVector3( 1.0f,  1.0f, 0.0f), Math::CVector2(1.0f, 0.0f)),
			new SCREENQUAD_VERTEX(Math::CVector3(-1.0f,  1.0f, 0.0f), Math::CVector2(0.0f, 0.0f))
		);

		if((retCode = CGeoObject::Initialize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method that starts rendering to texture.
	int CPostScreenQuad::BeginRenderToTexture()
	{
		int retCode = Util::Err::NONE;
		
		m_pTextureRenderTo->SetAsRenderTarget(true);
		m_pTextureRenderTo->ClearRenderTarget(DX11_CLEAR_COLOR, m_pDX11Data->backgroundColor, true);

		return retCode;
	}

	// Method for rendering the screen quad normally.
	int CPostScreenQuad::Render()
	{
		int retCode = Util::Err::NONE;

		// Finish first render to pass.
		m_pTextureRenderTo->ResetRenderTarget(false);

		m_pTransform->Position(Math::CVector3(0, 0, 1));
		m_pTransform->Update();
		
		m_pDevCon->PSSetShaderResources(120, 1, m_pTextureRenderTo->ppShaderResourceView());
		m_pDevCon->PSSetShaderResources(121, 1, m_pTextureRenderTo->ppDepthStencilShaderResourceView());
		
		if((retCode = CGeoObject::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}
	
	// Method for Preparing for window resize.
	int CPostScreenQuad::PreResize()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = m_pTextureRenderTo->PreResize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}
	
	// Method for resetting the screen quad after window resize.
	int CPostScreenQuad::PostResize()
	{
		int retCode = Util::Err::NONE;
		
		m_pMaterial->Camera()->Initialize((float)*m_pScreenWidth, (float)*m_pScreenHeight);
		
		if((retCode = m_pTextureRenderTo->PostResize(*m_pScreenWidth, *m_pScreenHeight)) != Util::Err::NONE)
			return retCode;
			
		return retCode;
	}

	void CPostScreenQuad::Release()
	{
		SAFE_RELEASE_DELETE(m_pTextureRenderTo);
		SAFE_RELEASE_DELETE(m_pMaterial);
		
		CGeoObject::Release();
	}
};