//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CPostScreenQuad.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CPOSTSCREENQUAD_H
#define CPOSTSCREENQUAD_H

#include "CTextureRenderTo.h"
#include "CGeoObject.h"
#include "CMatPostScreen.h"
#include "..\Math\CMathVector2.h"
#include "CLighting.h"

namespace GFX
{
	// CPostScreenQuad class.
	// Class for create a quad that acts as the backbuffer. Primarily for post process effects, and deferred renering.
	class CPostScreenQuad : public CGeoObject
	{
	public:
		// Vertex structure for the post processing screen.
		struct SCREENQUAD_VERTEX
		{
			Math::CVector3					position;
			Math::CVector2					texCoord;

			SCREENQUAD_VERTEX(Math::CVector3 Position, Math::CVector2 TexCoord)
			{
				position = Position;
				texCoord = TexCoord;
			}
		};

	public:
		CPostScreenQuad();
		CPostScreenQuad(const CPostScreenQuad& other);
		virtual ~CPostScreenQuad();
		
		void								SetScreenSize(int* width, int* height);
		int									Initialize() override;
		int									BeginRenderToTexture();
		int									Render() override;
		void								Release() override;

		int									PreResize();
		int									PostResize();
		
		inline CLighting*					Lighting() const;
		inline void							Lighting(CLighting* pLighting);
		inline CTextureProjection*			TextureProjection() const;
		inline void							TextureProjection(CTextureProjection* pProjection);
		inline CTextureRenderTo*			TextureRenderTo() const;
		inline void							TextureRenderTo(CTextureRenderTo* pRenderTexture);

	private:
		int*								m_pScreenWidth;
		int*								m_pScreenHeight;
		
		CLighting*							m_pLighting;
		CTextureProjection*					m_pTextureProjection;

		CTextureRenderTo*					m_pTextureRenderTo;
	};

	// Lighting getter and setter.
		
	inline CLighting* CPostScreenQuad::Lighting() const
	{
		return m_pLighting;
	}

	inline void CPostScreenQuad::Lighting(CLighting* pLighting)
	{
		m_pLighting = pLighting;
	}

	// Texture projection getter and setter.

	inline CTextureProjection* CPostScreenQuad::TextureProjection() const
	{
		return m_pTextureProjection;
	}

	inline void CPostScreenQuad::TextureProjection(CTextureProjection* pProjection)
	{
		m_pTextureProjection = pProjection;
	}

	// Render to texture getter and setter.

	inline CTextureRenderTo* CPostScreenQuad::TextureRenderTo() const
	{
		return m_pTextureRenderTo;
	}

	inline void CPostScreenQuad::TextureRenderTo(CTextureRenderTo* pRenderTexture)
	{
		m_pTextureRenderTo = pRenderTexture;
	}
};

#endif