//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CTexture.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CTexture.h"

// Texture2D
namespace GFX
{
	//---------------------------------------------------------------------------------------------
	// Texture2D
	//---------------------------------------------------------------------------------------------

	CTexture::CTexture()
	{
		m_pTexture							= NULL;
	}

	CTexture::CTexture(const CTexture& other)
	{
	}

	CTexture::~CTexture()
	{
	}

	// Method for creatubg a texture from a file.
	int CTexture::Initialize(const std::string& textureFilename, bool cubemap)
	{
		int retCode = Util::Err::NONE;

		std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> convertor;
		std::wstring wide = convertor.from_bytes(textureFilename);
		
		if(!cubemap)
		{
			if(FAILED(DirectX::CreateWICTextureFromFile(m_pDev, m_pDevCon, wide.c_str(), NULL, &m_pTexture)))
				return Util::Err::ERR_TEXTURE_FAILED_TO_OPEN_TEXTURE_FILE;
		}
		else
		{
			// Create a texture from a cubemap if told to do so.
			ID3D11Texture2D* pTemp;

			if(FAILED(DirectX::CreateDDSTextureFromFileEx(m_pDev, m_pDevCon, wide.c_str(), 0, D3D11_USAGE_DEFAULT, D3D11_BIND_SHADER_RESOURCE, 0, D3D11_RESOURCE_MISC_TEXTURECUBE, 
				0, (ID3D11Resource**)&pTemp, NULL)))
			{
				return Util::Err::ERR_TEXTURE_FAILED_TO_OPEN_TEXTURE_FILE;
			}

			D3D11_TEXTURE2D_DESC desc;
			pTemp->GetDesc(&desc);

			D3D11_SHADER_RESOURCE_VIEW_DESC resDesc;
			ZeroMemory(&resDesc, sizeof(resDesc));
			resDesc.Format = desc.Format;
			resDesc.ViewDimension = D3D11_SRV_DIMENSION_TEXTURECUBE;
			resDesc.TextureCube.MipLevels = desc.MipLevels;
			resDesc.TextureCube.MostDetailedMip = 0;

			if(FAILED(m_pDev->CreateShaderResourceView(pTemp, &resDesc, &m_pTexture)))
				return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_RESOURCE_VIEW;

			pTemp->Release();
		}

		// Check if the texture was actually created.
		if(!m_pTexture)
		{
			return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D;
		}

		return retCode;
	}

	// Method for create a texture from a texture2d object.
	int CTexture::Initialize(ID3D11Texture2D* texture2d)
	{
		int retCode = Util::Err::NONE;

		Initialize(texture2d, NULL);

		return retCode;
	}
	
	// Method for create a texture from a texture2d object w/ an optional shader resource vuew description.
	int CTexture::Initialize(ID3D11Texture2D* texture2d, D3D11_SHADER_RESOURCE_VIEW_DESC* srvd)
	{
		int retCode = Util::Err::NONE;

		if(FAILED(m_pDev->CreateShaderResourceView(texture2d, srvd, &m_pTexture)))
		{
			return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_RESOURCE_VIEW;
		}

		return retCode;
	}

	void CTexture::Release()
	{
		SAFE_RELEASE(m_pTexture);
	}
};

// Texture2D Array.
namespace GFX
{
	//---------------------------------------------------------------------------------------------
	// Texture2D Array
	//---------------------------------------------------------------------------------------------
	CTextureArray::CTextureArray()
	{
		m_pTexture2DArray = NULL;
		m_pTextureArray = NULL;
	}

	CTextureArray::CTextureArray(const CTextureArray& other)
	{
	}

	CTextureArray::~CTextureArray()
	{
	}

	// Method for initializing a texture 2d array from an array of filenames.
	int CTextureArray::Initialize(u32 width, u32 height, u32 arraySize, std::string* filenames)
	{
		int retCode = Util::Err::NONE;

		m_width			= width;
		m_height		= height;
		m_arraySize		= arraySize;
		
		D3D11_TEXTURE2D_DESC td;
		ZeroMemory(&td, sizeof(td));

		// Create the 2d texture(s).
		td.Width							= m_width;
		td.Height							= m_height;
		td.MipLevels						= 1;
		td.ArraySize						= m_arraySize;
		td.Format							= DXGI_FORMAT_B8G8R8A8_UNORM;
		td.SampleDesc.Count					= 1;
		td.SampleDesc.Quality				= 0;
		td.Usage							= D3D11_USAGE_DEFAULT;
		td.BindFlags						= D3D11_BIND_SHADER_RESOURCE;
		td.CPUAccessFlags					= 0;
		
		if(FAILED(m_pDev->CreateTexture2D(&td, NULL, &m_pTexture2DArray)))
		{
			return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D;
		}
		
		ID3D11Resource* resource;
		ID3D11Texture2D* pTemp;

		for(u32 i = 0; i < arraySize; i++)
		{
			std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> convertor;
			std::wstring wide = convertor.from_bytes(filenames[i]);
			
			if(FAILED(DirectX::CreateWICTextureFromFileEx(m_pDev, m_pDevCon, wide.c_str(), 0, D3D11_USAGE_STAGING, 0, D3D11_CPU_ACCESS_READ, 
				0, 0, &resource, NULL)))
			{
				return Util::Err::ERR_TEXTURE_FAILED_TO_OPEN_TEXTURE_FILE;
			}

			resource->QueryInterface(__uuidof(ID3D11Texture2D), (void**)&pTemp);
			D3D11_TEXTURE2D_DESC desc;
			pTemp->GetDesc(&desc);
			
			D3D11_MAPPED_SUBRESOURCE mappedData;
			
			// Map data to texture2d array
			m_pDevCon->Map(resource, 0, D3D11_MAP_READ, 0, &mappedData);

			m_pDevCon->UpdateSubresource(m_pTexture2DArray, D3D11CalcSubresource(0, i, 1), NULL, mappedData.pData, mappedData.RowPitch, 0);

			m_pDevCon->Unmap(resource, 0);

			pTemp->Release();
			resource->Release();
		}

		D3D11_SHADER_RESOURCE_VIEW_DESC		srvd;

		ZeroMemory(&srvd, sizeof(srvd));
		
		// Create the shader resource view(s).
		srvd.Format								= td.Format;
		srvd.ViewDimension						= D3D11_SRV_DIMENSION_TEXTURE2DARRAY;
		srvd.Texture2D.MostDetailedMip			= 0;
		srvd.Texture2D.MipLevels				= 1;
		srvd.Texture2DArray.ArraySize			= m_arraySize;
		srvd.Texture2DArray.MostDetailedMip		= 0;
		srvd.Texture2DArray.MipLevels			= 1;
		srvd.Texture2DArray.FirstArraySlice		= 0;
		
		if(FAILED(m_pDev->CreateShaderResourceView(m_pTexture2DArray, &srvd, &m_pTextureArray)))
		{
			return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_RESOURCE_VIEW;
		}

		return retCode;
	}

	// Method for initializing a texture 2d array from a list of shader resource views.
	int CTextureArray::Initialize(u32 width, u32 height, u32 arraySize, ID3D11ShaderResourceView** shaderResources)
	{
		int retCode = Util::Err::NONE;

		if(arraySize <= 0)
			return retCode;
		
		ID3D11Resource* resource = NULL;
		ID3D11Resource* pNewResource = NULL;
		ID3D11Texture2D* pNewTexture = NULL;
		ID3D11Texture2D* pTemp = NULL;
		D3D11_TEXTURE2D_DESC td;
		ZeroMemory(&td, sizeof(td));

		shaderResources[0]->GetResource(&resource);
		resource->QueryInterface(__uuidof(ID3D11Texture2D), (void**)&pTemp);
		
		pTemp->GetDesc(&td);
		pTemp->Release();
		resource->Release();

		m_width			= td.Width;
		m_height		= td.Height;
		m_arraySize		= arraySize;
		
		// Create the 2d texture(s).
		td.Width							= m_width;
		td.Height							= m_height;
		td.ArraySize						= m_arraySize;
		td.BindFlags						= D3D11_BIND_SHADER_RESOURCE;
		
		if(FAILED(m_pDev->CreateTexture2D(&td, NULL, &m_pTexture2DArray)))
		{
			return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D;
		}
		
		td.BindFlags						= 0;
		td.ArraySize						= 1;
		td.Usage							= D3D11_USAGE_STAGING;
		td.CPUAccessFlags					= D3D11_CPU_ACCESS_READ;

		// Extract and store all texture data from the texture filename array.
		for(u32 i = 0; i < arraySize; i++)
		{
			if(FAILED(m_pDev->CreateTexture2D(&td, NULL, &pNewTexture)))
			{
				return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D;
			}

			shaderResources[i]->GetResource(&resource);
			m_pDevCon->CopyResource(pNewTexture, resource);
			D3D11_TEXTURE2D_DESC desc;
			pNewTexture->GetDesc(&desc);
			
			D3D11_MAPPED_SUBRESOURCE mappedData;
			
			// Map data to texture2d array
			m_pDevCon->Map(pNewTexture, 0, D3D11_MAP_READ, 0, &mappedData);

			m_pDevCon->UpdateSubresource(m_pTexture2DArray, D3D11CalcSubresource(0, i, 1), NULL, mappedData.pData, mappedData.RowPitch, 0);

			m_pDevCon->Unmap(pNewTexture, 0);

			pNewTexture->Release();
			resource->Release();
		}

		D3D11_SHADER_RESOURCE_VIEW_DESC		srvd;

		ZeroMemory(&srvd, sizeof(srvd));
		
		// Create the shader resource view(s).
		srvd.Format								= td.Format;
		srvd.ViewDimension						= D3D11_SRV_DIMENSION_TEXTURE2DARRAY;
		srvd.Texture2D.MostDetailedMip			= 0;
		srvd.Texture2D.MipLevels				= 1;
		srvd.Texture2DArray.ArraySize			= m_arraySize;
		srvd.Texture2DArray.MostDetailedMip		= 0;
		srvd.Texture2DArray.MipLevels			= 1;
		srvd.Texture2DArray.FirstArraySlice		= 0;
		
		if(FAILED(m_pDev->CreateShaderResourceView(m_pTexture2DArray, &srvd, &m_pTextureArray)))
		{
			return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_RESOURCE_VIEW;
		}

		return retCode;
	}

	void CTextureArray::Release()
	{
		SAFE_RELEASE(m_pTextureArray);
		SAFE_RELEASE(m_pTexture2DArray);
	}
};