//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CTextureList.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CTextureList.h"

namespace GFX
{
	// Method for adding global 2d textures to the 2d texture list.
	CTexture* CTextureList::AddTexture(const std::string& filename, bool cubemap)
	{
		CTexture* texture = FindTexture(filename);
		
		if(!texture)
		{
			TEXTURE tex;

			texture = new CTexture();
			texture->DirectXData(m_pDX11Data);
			if((texture->Initialize(filename, cubemap)) != Util::Err::NONE)
				return NULL;
				
			// Create an md5 string for this texture.
			memcpy(tex.md5, Math::CMD5(filename).HexDigest().c_str(), 32);
			tex.arraySize	= 1;
			tex.texture		= texture;

			m_sTextureStack.Push(tex);

			return tex.texture;
		}

		return texture;
	}

	// Method for checking is a texture has already been created.
	CTexture* CTextureList::FindTexture(const std::string& filename)
	{
		std::string md5 = Math::CMD5(filename).HexDigest();
		
		for(u32 i = 0; i < m_sTextureStack.Length(); i++)
		{
			if(strncmp(m_sTextureStack[i].md5, md5.c_str(), 32) == 0)
			{
				return m_sTextureStack[i].texture;
			}
		}

		return NULL;
	}
	
	// Method for adding global 2d texture arrays to the 2d texture list.
	CTextureArray* CTextureList::AddTextureArray(const std::string& name, u32 width, u32 height, u32 arraySize, std::string* filename)
	{
		if(arraySize <= 0)
			return NULL;

		CTextureArray* textureArray = FindTextureArray(name);
		
		if(!textureArray)
		{
			TEXTURE tex;

			textureArray = new CTextureArray();
			textureArray->DirectXData(m_pDX11Data);
			if((textureArray->Initialize(width, height, arraySize, filename)) != Util::Err::NONE)
				return NULL;

			// Create an md5 string for this texture array.
			memcpy(tex.md5, Math::CMD5(name).HexDigest().c_str(), 32);
			tex.arraySize		= arraySize;
			tex.textureArray	= textureArray;

			m_sTextureArrayStack.Push(tex);

			return tex.textureArray;
		}

		return textureArray;
	}

	// Method for checking is a 2d texture array has already been created.
	CTextureArray* CTextureList::FindTextureArray(const std::string& name)
	{
		std::string md5 = Math::CMD5(name).HexDigest();
		
		for(u32 i = 0; i < m_sTextureArrayStack.Length(); i++)
		{
			if(strncmp(m_sTextureArrayStack[i].md5, md5.c_str(), 32) == 0)
			{
				return m_sTextureArrayStack[i].textureArray;
			}
		}

		return NULL;
	}

	// Method for releasing all textures from the texture list.
	void CTextureList::Release()
	{
		// Texture2D Array.
		for(u32 i = 0; i < m_sTextureArrayStack.Length(); i++)
		{
			if(m_sTextureArrayStack[i].textureArray)
				SAFE_RELEASE_DELETE(m_sTextureArrayStack[i].textureArray);
		}

		m_sTextureArrayStack.Release();

		// Texture2D
		for(u32 i = 0; i < m_sTextureStack.Length(); i++)
		{
			if(m_sTextureStack[i].texture)
				SAFE_RELEASE_DELETE(m_sTextureStack[i].texture);
		}

		m_sTextureStack.Release();
	}
};