//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CTextureList.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CTEXTURELIST_H
#define CTEXTURELIST_H

#include "CTexture.h"
#include "..\Math\CMathMD5.h"
#include "..\Utilities\CDataStack.h"

namespace GFX
{
	// CTextureList class (singleton)
	// Class for storing all loaded texutres and texture arrays (checks for existence of audio source before creation of a new one).
	class CTextureList : public App::CDX11Object
	{
	private:

		// Structure for loaded textures w/ md5 string to assist in checking for duplicates.
		struct TEXTURE
		{
			char		md5[32];
			u32			arraySize;
			
			union
			{
				CTexture*		texture;
				CTextureArray*	textureArray;
			};
		};

	public:
		static CTextureList& Instance()
		{
			static CTextureList instance;
			return instance;
		}

	public:
		CTexture*							AddTexture(const std::string& filename, bool cubemap = false);
		CTexture*							FindTexture(const std::string& filename);

		CTextureArray*						AddTextureArray(const std::string& name, u32 width, u32 height, u32 arraySize, std::string* filename);
		CTextureArray*						FindTextureArray(const std::string& name);

		void								Release();

	private:
		CTextureList() {}
		CTextureList(const CTextureList&) = delete;
		virtual ~CTextureList() { Release(); }

		Util::CStack<TEXTURE>				m_sTextureStack;
		Util::CStack<TEXTURE>				m_sTextureArrayStack;
	};
};

#endif