//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CTextureProjection.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CTextureProjection.h"

namespace GFX
{
	CTextureProjection::CTextureProjection()
	{
		m_texProjCount						= 0;

		m_pTextureArray						= NULL;
		m_pTextureArrayGradient				= NULL;

		m_pTextureProjArray					= NULL;
		m_pMatrixPtrArray					= NULL;

		m_pTextureProjCBuffer				= NULL;
	}

	CTextureProjection::CTextureProjection(const CTextureProjection& other)
	{
	}

	CTextureProjection::~CTextureProjection()
	{
	}

	// Method for initializing the texture projection system.
	int CTextureProjection::Initialize()
	{
		int retCode = Util::Err::NONE;
		
		m_texProjCountStr = std::to_string(m_texProjCount);

		// Check if any projected textures have been pushed to the queue.
		if(m_texProjCount > 0)
		{
			m_pTextureProjArray = new TEXTURE_PROJECTOR[m_texProjCount];
			m_pMatrixPtrArray = new MATRIX_POINTERS[m_texProjCount];

			ZeroMemory(m_pTextureProjArray, sizeof(TEXTURE_PROJECTOR));

			// Add pushed projected texture data to the constant buffer array, and the matrix pointer array.
			u32 i = 0;
			TEXTURE_PROJECTOR_QUEUE* data;
			while((data = (TEXTURE_PROJECTOR_QUEUE*)m_qTextureProjQueue.Pop()))
			{
				m_pTextureProjArray[i]			= data->texProjector;
				m_pMatrixPtrArray[i]			= data->matrices;

				SAFE_DELETE(data);
				i++;
			}

			m_pTextureProjCBuffer = new CConstantBuffer();

			m_pTextureProjCBuffer->DirectXData(m_pDX11Data);
			if((retCode = m_pTextureProjCBuffer->Initialize(sizeof(TEXTURE_PROJECTOR) * m_texProjCount, true)) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}

	// Method if resetting the texture projection material constant buffer data.
	int CTextureProjection::ResetBuffer()
	{
		int retCode = Util::Err::NONE;

		// Update all of the texture protection matrices.
		for(u32 i = 0; i < m_texProjCount; i++)
		{
			m_pTextureProjArray[i].projection = *m_pMatrixPtrArray[i].projection;
			m_pTextureProjArray[i].view = *m_pMatrixPtrArray[i].view;
		}

		if(m_pTextureProjCBuffer) if((retCode = m_pTextureProjCBuffer->ResetBuffer(m_pTextureProjArray)) != Util::Err::NONE) return retCode;

		return retCode;
	}
	
	// Method if binding the texture projection material constant buffer data to the shader.
	int CTextureProjection::BindBuffer()
	{
		int retCode = Util::Err::NONE;

		// Bind the constant buffers, and textures for use in shaders,
		if(m_pTextureArray)
			m_pDevCon->PSSetShaderResources(PSSR_PROJ_TEXTURES_INDEX, 1, m_pTextureArray->ppTextureArray());
		if(m_pTextureArrayGradient)
			m_pDevCon->PSSetShaderResources(PSSR_PROJ_GRADIENTS_INDEX, 1, m_pTextureArrayGradient->ppTextureArray());
		
		if(m_pTextureProjCBuffer)
			if((retCode = m_pTextureProjCBuffer->BindBufferPS(CBUFFER_INDEX)) != Util::Err::NONE) return retCode;

		return retCode;
	}

	void CTextureProjection::Release()
	{
		SAFE_RELEASE_DELETE(m_pTextureProjCBuffer);
		SAFE_DELETE_ARRAY(m_pMatrixPtrArray);
		SAFE_DELETE_ARRAY(m_pTextureProjArray);
		m_qTextureProjQueue.Release();
	}
};