//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Math\CMathCamera.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMathCamera.h"

namespace Math
{
	CCamera::CCamera()
	{
		m_pTransform						= NULL;
	}

	CCamera::CCamera(const CCamera& other)
	{
	}

	CCamera::~CCamera()
	{
	}
	
	// Method for initializing a camera in either perspective or orthographic mode with a past screen width, and height.
	void CCamera::Initialize()
	{
		Initialize(m_screenWidth, m_screenHeight);
	}
	
	// Method for initializing a camera in either perspective or orthographic mode with a screen width, and height.
	void CCamera::Initialize(float screenWidth, float screenHeight)
	{
		m_screenWidth = screenWidth;
		m_screenHeight = screenHeight;
		
		Initialize(screenWidth / screenHeight);
	}

	// Method for initializing a camera in either perspective or orthographic mode with an aspect ratio.
	void CCamera::Initialize(float aspect)
	{
		m_aspect = aspect;

		if(m_cameraMode == CAMERA_MODE::Perspective)
		{
			m_projMtx = CMatrix4x4::PerspectiveFovLH(m_fov,
													 m_aspect,
													 m_nearView,
													 m_farView);
		}
		else
		{
			m_projMtx = CMatrix4x4::OrthographicLH(m_width * m_aspect,
												   m_height,
												   m_nearView,
												   m_farView);
		}

		m_projMtxInv = m_projMtx.Inverse();
	}

	// Method for updating a camera's view matrix.
	void CCamera::Update(bool transform)
	{
		if(m_pTransform)
		{
			if(transform)
				m_pTransform->Update();
			
			CVector3 up = CVector3::Up();
			CVector3 lookAt = CVector3::Forward();
			
			CMatrix4x4 tempMtx;
			tempMtx = CMatrix4x4::RotateZ(m_pTransform->Euler().z);
			tempMtx *= CMatrix4x4::RotateY(m_pTransform->Euler().y);
			tempMtx *= CMatrix4x4::RotateX(m_pTransform->Euler().x);

			lookAt = CMatrix4x4::TransformCoord(lookAt, tempMtx) + m_pTransform->Position();
			up = CMatrix4x4::TransformCoord(up, tempMtx);
			
			m_viewMtx = CMatrix4x4::LookAtLH(
				m_pTransform->Position(),
				lookAt,
				up
			);

			m_viewMtxInv = m_viewMtx.Inverse();

			m_frustum.Update(m_viewMtx, m_projMtx);
		}
	}
	
	// Method for transforming a world point to a screen point for orthgraphics cameras.
	Math::CVector2 CCamera::WorldToScreen(const Math::CVector2& point)
	{
		Math::CVector2 pt;

		pt.x = ((point.x * m_projMtx._11 + 1.0f) * m_screenWidth) / 2.0f;
		pt.y = (-(point.y * m_projMtx._22 - 1.0f) * m_screenHeight) / 2.0f;

		return pt;
	}

	// Method for transforming a screen point to a world point for orthgraphics cameras.
	Math::CVector2 CCamera::ScreenToWorld(const Math::CVector2& point)
	{
		Math::CVector2 pt;

		pt.x = ((2.0f * point.x) / m_screenWidth - 1.0f) / m_projMtx._11;
		pt.y = (-(2.0f * point.y) / m_screenHeight + 1.0f) / m_projMtx._22;

		return pt;
	}
};