//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Math\CMathFloat.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMATHFLOAT_H
#define CMATHFLOAT_H

#include <cmath>

namespace Math
{
	// Mathematical constants.
	const float								_1Over3				= 0.33333333f;
	const float								_1Over6				= 0.66666667f;

	const float								_Pi					= 3.14159265f;
	const float								_2Pi				= 2.0f * _Pi;
	const float								_PiOver2			= 0.5f * _Pi;
	const float								_PiOver180			= _Pi / 180.0f;
	const float								_180OverPi			= 180.f / _Pi;

	const float								_ln					= 2.71828183f;
	
	const float								_Epsilon			= 1e-7f;
	const float								_1SubEpsilon		= 1.0f - _Epsilon;
	
	// Sign functions.
	inline float Sign(float f)
	{
		return f < 0.0f ? -1.0f : 1.0f;
	}

	inline int Sign(int i)
	{
		return i < 0 ? -1 : 1;
	}

	// Clamp functions for data type float.
	inline float Clamp(float f, float mn, float mx)
	{
		return f < mn ? mn : f > mx ? mx : f;
	}

	inline void Clamp(float* f, float mn, float mx)
	{
		*f = Clamp(*f, mn, mx);
	}

	// Clamp functions for data type int.
	inline int Clamp(int i, int mn, int mx)
	{
		return i < mn ? mn : i > mx ? mx : i;
	}

	inline void Clamp(int* i, int mn, int mx)
	{
		*i = Clamp(*i, mn, mx);
	}

	// Saturate function.
	inline float Saturate(float f)
	{
		return f < 0.0f ? 0.0f : f > 1.0f ? 1.0f : f;
	}

	inline void Saturate(float* f)
	{
		*f = Saturate(*f);
	}

	// Round to nearest function.
	inline float RoundToNearest(float f, int precision)
	{
		if(precision <= 0)
		{
			return roundf(f);
		}
		else
		{
			float tenPow = powf(10.0f, (float)precision);
			return roundf(f * tenPow) / tenPow;
		}
	}

	inline void RoundToNearest(float* f, int precision)
	{
		if(precision <= 0)
		{
			*f = roundf(*f);
		}
		else
		{
			float tenPow = powf(10.0f, (float)precision);
			*f =  roundf(*f * tenPow) / tenPow;
		}
	}

	//
	// Interpolation functions.
	//

	inline float MoveTowards(float from, float to, float t)
	{
		if(fabsf(to - from) < t)
			return to;

		return from + Sign(to - from) * t;
	}

	inline float Lerp(float from, float to, float t)
	{
		return from + (to - from) * t;
	}
};

#endif