//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Math\CMathFrustum.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMathFrustum.h"

namespace Math
{
	CFrustum::CFrustum()
	{
		m_pView				= NULL;
		m_pProjection		= NULL;
	}

	// Method for updating a view frustums clip planes.
	void CFrustum::Update(const CMatrix4x4& view, const CMatrix4x4& projection)
	{
		CMatrix4x4 viewProjection = (projection) * (view);
		
		// Left plane
		m_clipPlanes[0].a = viewProjection._41 + viewProjection._11;
		m_clipPlanes[0].b = viewProjection._42 + viewProjection._12;
		m_clipPlanes[0].c = viewProjection._43 + viewProjection._13;
		m_clipPlanes[0].d = viewProjection._44 + viewProjection._14;
 
		// Right plane
		m_clipPlanes[1].a = viewProjection._41 - viewProjection._11;
		m_clipPlanes[1].b = viewProjection._42 - viewProjection._12;
		m_clipPlanes[1].c = viewProjection._43 - viewProjection._13;
		m_clipPlanes[1].d = viewProjection._44 - viewProjection._14;
 
		// Top plane
		m_clipPlanes[2].a = viewProjection._41 - viewProjection._21;
		m_clipPlanes[2].b = viewProjection._42 - viewProjection._22;
		m_clipPlanes[2].c = viewProjection._43 - viewProjection._23;
		m_clipPlanes[2].d = viewProjection._44 - viewProjection._24;
 
		// Bottom plane
		m_clipPlanes[3].a = viewProjection._41 + viewProjection._21;
		m_clipPlanes[3].b = viewProjection._42 + viewProjection._22;
		m_clipPlanes[3].c = viewProjection._43 + viewProjection._23;
		m_clipPlanes[3].d = viewProjection._44 + viewProjection._24;
 
		// Near plane
		m_clipPlanes[4].a = viewProjection._31;
		m_clipPlanes[4].b = viewProjection._32;
		m_clipPlanes[4].c = viewProjection._33;
		m_clipPlanes[4].d = viewProjection._34;
 
		// Far plane
		m_clipPlanes[5].a = viewProjection._41 - viewProjection._31;
		m_clipPlanes[5].b = viewProjection._42 - viewProjection._32;
		m_clipPlanes[5].c = viewProjection._43 - viewProjection._33;
		m_clipPlanes[5].d = viewProjection._44 - viewProjection._34;

		for(u8 i = 0; i < 6; i++)
		{
			m_clipPlanes[i].Normalize();
		}
	}

	// Method for testing a sphere against this frustum.
	bool CFrustum::TestSphere(const CSphere& sphere)
	{
		for(u8 i = 0; i < 6; i++)
		{
			float val = m_clipPlanes[i].DotCoord(sphere.center);
			if(val < -sphere.radius)
				return false;
		}

		return true;
	}
	
	// Method for testing an aabb against this frustum.
	bool CFrustum::TestAABB(const CAABB& aabb)
	{
		CVector3 axisVec;

		for(u8 i = 0; i < 6; i++)
		{
			if(m_clipPlanes[i].a < 0)
				axisVec.x = aabb.minBounds.x;
			else
				axisVec.x = aabb.maxBounds.x;

			if(m_clipPlanes[i].b < 0)
				axisVec.y = aabb.minBounds.y;
			else
				axisVec.y = aabb.maxBounds.y;

			if(m_clipPlanes[i].c < 0)
				axisVec.z = aabb.minBounds.z;
			else
				axisVec.z = aabb.maxBounds.z;
       
			if(m_clipPlanes[i].DotCoord(axisVec) < 0)
			{
				return false;
			}
		}
	
		return true;
	}
};