//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Math\CMathMatrix4x4.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMathMatrix4x4.h"

namespace Math
{
	CMatrix4x4::CMatrix4x4()
	{
		memset(m, 0, sizeof(float) * 16);
	}

	CMatrix4x4::CMatrix4x4(const float* M)
	{
		memcpy(m, M, sizeof(float) * 16);
	}
	
	CMatrix4x4::CMatrix4x4(float m_11, float m_12, float m_13, float m_14,
						 float m_21, float m_22, float m_23, float m_24,
						 float m_31, float m_32, float m_33, float m_34,
						 float m_41, float m_42, float m_43, float m_44)
	{
		_11 = m_11; _12 = m_12; _13 = m_13; _14 = m_14;
		_21 = m_21; _22 = m_22; _23 = m_23; _24 = m_24;
		_31 = m_31; _32 = m_32; _33 = m_33; _34 = m_34;
		_41 = m_41; _42 = m_42; _43 = m_43; _44 = m_44;
	}
				  
	CMatrix4x4::CMatrix4x4(const CMatrix4x4& other)
	{
		memcpy(m, other.m, sizeof(float) * 16);
	}

	CMatrix4x4::CMatrix4x4(CMatrix4x4&& other)
	{
		memcpy(m, other.m, sizeof(float) * 16);
		memset(other.m, 0, sizeof(float) * 16);
	}
	
	//
	// CMatrix4x4 operators.
	//

	float& CMatrix4x4::operator () (u32 i, u32 j)
	{
		return _ij[i][j];
	}

	float CMatrix4x4::operator () (u32 i, u32 j) const
	{
		return _ij[i][j];
	}
	
	CMatrix4x4& CMatrix4x4::operator = (float f)
	{
		_11 = f; _12 = f; _13 = f; _14 = f;
		_21 = f; _22 = f; _23 = f; _24 = f;
		_31 = f; _32 = f; _33 = f; _34 = f;
		_41 = f; _42 = f; _43 = f; _44 = f;
		return *this;
	}

	CMatrix4x4& CMatrix4x4::operator = (const CMatrix4x4& M)
	{
		memcpy(m, M.m, sizeof(float) * 16);
		return *this;
	}

	CMatrix4x4 CMatrix4x4::operator - () const
	{
		return CMatrix4x4(
			-_11, -_12, -_13, -_14,
			-_21, -_22, -_23, -_24,
			-_31, -_32, -_33, -_34,
			-_41, -_42, -_43, -_44
		);
	}

	CMatrix4x4 CMatrix4x4::operator + (const CMatrix4x4& M) const
	{
		return CMatrix4x4(
			_11 + M._11, _12 + M._12, _13 + M._13, _14 + M._14,
			_21 + M._21, _22 + M._22, _23 + M._23, _24 + M._24,
			_31 + M._31, _32 + M._32, _33 + M._33, _34 + M._34,
			_41 + M._41, _42 + M._42, _43 + M._43, _44 + M._44
		);
	}

	CMatrix4x4 CMatrix4x4::operator - (const CMatrix4x4& M) const
	{
		return CMatrix4x4(
			_11 - M._11, _12 - M._12, _13 - M._13, _14 - M._14,
			_21 - M._21, _22 - M._22, _23 - M._23, _24 - M._24,
			_31 - M._31, _32 - M._32, _33 - M._33, _34 - M._34,
			_41 - M._41, _42 - M._42, _43 - M._43, _44 - M._44
		);
	}
	
	CMatrix4x4 CMatrix4x4::operator * (const CMatrix4x4& M) const
	{
		return CMatrix4x4(
			_11 * M._11 + _12 * M._21 + _13 * M._31 + _14 * M._41,
			_11 * M._12 + _12 * M._22 + _13 * M._32 + _14 * M._42,
			_11 * M._13 + _12 * M._23 + _13 * M._33 + _14 * M._43,
			_11 * M._14 + _12 * M._24 + _13 * M._34 + _14 * M._44,

			_21 * M._11 + _22 * M._21 + _23 * M._31 + _24 * M._41,
			_21 * M._12 + _22 * M._22 + _23 * M._32 + _24 * M._42,
			_21 * M._13 + _22 * M._23 + _23 * M._33 + _24 * M._43,
			_21 * M._14 + _22 * M._24 + _23 * M._34 + _24 * M._44,

			_31 * M._11 + _32 * M._21 + _33 * M._31 + _34 * M._41,
			_31 * M._12 + _32 * M._22 + _33 * M._32 + _34 * M._42,
			_31 * M._13 + _32 * M._23 + _33 * M._33 + _34 * M._43,
			_31 * M._14 + _32 * M._24 + _33 * M._34 + _34 * M._44,

			_41 * M._11 + _42 * M._21 + _43 * M._31 + _44 * M._41,
			_41 * M._12 + _42 * M._22 + _43 * M._32 + _44 * M._42,
			_41 * M._13 + _42 * M._23 + _43 * M._33 + _44 * M._43,
			_41 * M._14 + _42 * M._24 + _43 * M._34 + _44 * M._44
		);
	}

	CMatrix4x4 CMatrix4x4::operator * (float f) const
	{
		return CMatrix4x4(
			_11 * f, _12 * f, _13 * f, _14 * f,
			_21 * f, _22 * f, _23 * f, _24 * f,
			_31 * f, _32 * f, _33 * f, _34 * f,
			_41 * f, _42 * f, _43 * f, _44 * f
		);
	}

	CMatrix4x4 CMatrix4x4::operator / (float f) const
	{
		return CMatrix4x4(
			_11 / f, _12 / f, _13 / f, _14 / f,
			_21 / f, _22 / f, _23 / f, _24 / f,
			_31 / f, _32 / f, _33 / f, _34 / f,
			_41 / f, _42 / f, _43 / f, _44 / f
		);
	}

	CMatrix4x4& CMatrix4x4::operator += (const CMatrix4x4& M)
	{
		*this = *this + M;
		return *this;
	}

	CMatrix4x4& CMatrix4x4::operator -= (const CMatrix4x4& M)
	{
		*this = *this - M;
		return *this;
	}

	CMatrix4x4& CMatrix4x4::operator *= (const CMatrix4x4& M)
	{
		*this = *this * M;
		return *this;
	}

	CMatrix4x4& CMatrix4x4::operator *= (float f)
	{
		*this = *this * f;
		return *this;
	}

	CMatrix4x4& CMatrix4x4::operator /= (float f)
	{
		*this = *this / f;
		return *this;
	}
	
	//
	// Conditional operators.
	//
	
	bool CMatrix4x4::operator == (const CMatrix4x4& M) const
	{
		return (
			_11 == M._11 && _12 == M._12 && _13 == M._13 && _14 == M._14 &&
			_21 == M._21 && _22 == M._22 && _23 == M._23 && _24 == M._24 &&
			_31 == M._31 && _32 == M._32 && _33 == M._33 && _34 == M._34 &&
			_41 == M._41 && _42 == M._42 && _43 == M._43 && _44 == M._44
		);
	}
	
	bool CMatrix4x4::operator != (const CMatrix4x4& M) const
	{
		return (
			_11 != M._11 || _12 != M._12 || _13 != M._13 || _14 != M._14 ||
			_21 != M._21 || _22 != M._22 || _23 != M._23 || _24 != M._24 ||
			_31 != M._31 || _32 != M._32 || _33 != M._33 || _34 != M._34 ||
			_41 != M._41 || _42 != M._42 || _43 != M._43 || _44 != M._44
		);
	}
	
	//
	// Friend operators.
	//

	CMatrix4x4 operator * (float f, const CMatrix4x4& m)
	{
		return m * f;
	}

	CMatrix4x4 operator / (float f, const CMatrix4x4& m)
	{
		return m / f;
	}

	//
	// Member mathematic methods.
	//

	CMatrix4x4 CMatrix4x4::Transpose()
	{
		CMatrix4x4 mtx;

		mtx(0, 0) = _ij[0][0]; mtx(1, 0) = _ij[0][1]; mtx(2, 0) = _ij[0][2]; mtx(3, 0) = _ij[0][3];
		mtx(0, 1) = _ij[1][0]; mtx(1, 1) = _ij[1][1]; mtx(2, 1) = _ij[1][2]; mtx(3, 1) = _ij[1][3];
		mtx(0, 2) = _ij[2][0]; mtx(1, 2) = _ij[2][1]; mtx(2, 2) = _ij[2][2]; mtx(3, 2) = _ij[2][3];
		mtx(0, 3) = _ij[3][0]; mtx(1, 3) = _ij[3][1]; mtx(2, 3) = _ij[3][2]; mtx(3, 3) = _ij[3][3];

		return mtx;
	}

	float CMatrix4x4::Determinant()
	{
		return (_11 * _22 * _33 * _44) + (_11 * _23 * _34 * _42) + (_11 * _24 * _32 * _43) +
			   (_12 * _21 * _34 * _43) + (_12 * _23 * _31 * _44) + (_12 * _24 * _33 * _41) +
			   (_13 * _21 * _32 * _44) + (_13 * _22 * _34 * _41) + (_13 * _24 * _31 * _42) +
			   (_14 * _21 * _33 * _42) + (_14 * _22 * _31 * _43) + (_14 * _23 * _32 * _41) -
			   (_11 * _22 * _34 * _43) - (_11 * _23 * _32 * _44) - (_11 * _24 * _33 * _42) -
			   (_12 * _21 * _33 * _44) - (_12 * _23 * _34 * _41) - (_12 * _24 * _31 * _43) -
			   (_13 * _21 * _34 * _42) - (_13 * _22 * _31 * _44) - (_13 * _24 * _32 * _41) -
			   (_14 * _21 * _32 * _43) - (_14 * _22 * _33 * _41) - (_14 * _23 * _31 * _42);
	}

	CMatrix4x4 CMatrix4x4::Inverse()
	{
		float det = Determinant();
		if(det == 0)
			return *this;

		CMatrix4x4 mtx;

		mtx._11 = (_22 * _33 * _44) + (_23 * _34 * _42) + (_24 * _32 * _43) - (_22 * _34 * _43) - (_23 * _32 * _44) - (_24 * _33 * _42);
		mtx._12 = (_12 * _34 * _43) + (_13 * _32 * _44) + (_14 * _33 * _42) - (_12 * _33 * _44) - (_13 * _34 * _42) - (_14 * _32 * _43);
		mtx._13 = (_12 * _23 * _44) + (_13 * _24 * _42) + (_14 * _22 * _43) - (_12 * _24 * _43) - (_13 * _22 * _44) - (_14 * _23 * _42);
		mtx._14 = (_12 * _24 * _33) + (_13 * _22 * _34) + (_14 * _23 * _32) - (_12 * _23 * _34) - (_13 * _24 * _32) - (_14 * _22 * _33);
		
		mtx._21 = (_21 * _34 * _43) + (_23 * _31 * _44) + (_24 * _33 * _41) - (_21 * _33 * _44) - (_23 * _34 * _41) - (_24 * _31 * _43);
		mtx._22 = (_11 * _33 * _44) + (_13 * _34 * _41) + (_14 * _31 * _43) - (_11 * _34 * _43) - (_13 * _31 * _44) - (_14 * _33 * _41);
		mtx._23 = (_11 * _24 * _43) + (_13 * _21 * _44) + (_14 * _23 * _41) - (_11 * _23 * _44) - (_13 * _24 * _41) - (_14 * _21 * _43);
		mtx._24 = (_11 * _23 * _34) + (_13 * _24 * _31) + (_14 * _21 * _33) - (_11 * _24 * _33) - (_13 * _21 * _34) - (_14 * _23 * _31);
		
		mtx._31 = (_21 * _32 * _44) + (_22 * _34 * _41) + (_24 * _31 * _42) - (_21 * _34 * _42) - (_22 * _31 * _44) - (_24 * _32 * _41);
		mtx._32 = (_11 * _34 * _42) + (_12 * _31 * _44) + (_14 * _32 * _41) - (_11 * _32 * _44) - (_12 * _34 * _41) - (_14 * _31 * _42);
		mtx._33 = (_11 * _22 * _44) + (_12 * _24 * _41) + (_14 * _21 * _42) - (_11 * _24 * _42) - (_12 * _21 * _44) - (_14 * _22 * _41);
		mtx._34 = (_11 * _24 * _32) + (_12 * _21 * _34) + (_14 * _22 * _31) - (_11 * _22 * _34) - (_12 * _24 * _31) - (_14 * _21 * _32);
		
		mtx._41 = (_21 * _33 * _42) + (_22 * _31 * _43) + (_23 * _32 * _41) - (_21 * _32 * _43) - (_22 * _33 * _41) - (_23 * _31 * _42);
		mtx._42 = (_11 * _32 * _43) + (_12 * _33 * _41) + (_13 * _31 * _42) - (_11 * _33 * _42) - (_12 * _31 * _43) - (_13 * _32 * _41);
		mtx._43 = (_11 * _23 * _42) + (_12 * _21 * _43) + (_13 * _22 * _41) - (_11 * _22 * _43) - (_12 * _23 * _41) - (_13 * _21 * _42);
		mtx._44 = (_11 * _22 * _33) + (_12 * _23 * _31) + (_13 * _21 * _32) - (_11 * _23 * _32) - (_12 * _21 * _33) - (_13 * _22 * _31);
		
		return mtx * (1.0f / det);
	}
	
	//
	// Static matrix constant methods.
	//

	CMatrix4x4 CMatrix4x4::Identity()
	{
		return MTX4X4_IDENTITY;
	}

	//
	// Static matrix mathematic methods.
	//
	
	CMatrix4x4 CMatrix4x4::Translate(const CVector3& position)
	{
		CMatrix4x4 mtx = Identity();

		mtx(0, 3) = position.x;
		mtx(1, 3) = position.y;
		mtx(2, 3) = position.z;

		return mtx;
	}

	CMatrix4x4 CMatrix4x4::Scale(const CVector3& scale)
	{
		CMatrix4x4 mtx = Identity();

		mtx(0, 0) = scale.x;
		mtx(1, 1) = scale.y;
		mtx(2, 2) = scale.z;

		return mtx;
	}

	CMatrix4x4 CMatrix4x4::RotateX(float angle)
	{
		CMatrix4x4 mtx = Identity();

		float s = sinf(angle);
		float c = cosf(angle);

		mtx(1, 1) =  c;
		mtx(2, 1) =  s;
		mtx(1, 2) = -s;
		mtx(2, 2) =  c;

		return mtx;
	}

	CMatrix4x4 CMatrix4x4::RotateY(float angle)
	{
		CMatrix4x4 mtx = Identity();

		float s = sinf(angle);
		float c = cosf(angle);

		mtx(0, 0) =  c;
		mtx(2, 0) = -s;
		mtx(0, 2) =  s;
		mtx(2, 2) =  c;

		return mtx;
	}

	CMatrix4x4 CMatrix4x4::RotateZ(float angle)
	{
		CMatrix4x4 mtx = Identity();

		float s = sinf(angle);
		float c = cosf(angle);

		mtx(0, 0) =  c;
		mtx(1, 0) =  s;
		mtx(0, 1) = -s;
		mtx(1, 1) =  c;

		return mtx;
	}
	
	// Point matrix transformation methods.

	CVector3 CMatrix4x4::TransformCoord(const CVector3& point)
	{
		CVector3 pt;
		
		pt.x = _ij[0][0] * point.x + _ij[0][1] * point.y + _ij[0][2] * point.z;
		pt.y = _ij[1][0] * point.x + _ij[1][1] * point.y + _ij[1][2] * point.z;
		pt.z = _ij[2][0] * point.x + _ij[2][1] * point.y + _ij[2][2] * point.z;

		return pt;
	}

	CVector3 CMatrix4x4::TransformCoord(const CVector3& point, const CMatrix4x4& M)
	{
		CVector3 pt;
		
		pt.x = M(0, 0) * point.x + M(0, 1) * point.y + M(0, 2) * point.z;
		pt.y = M(1, 0) * point.x + M(1, 1) * point.y + M(1, 2) * point.z;
		pt.z = M(2, 0) * point.x + M(2, 1) * point.y + M(2, 2) * point.z;

		return pt;
	}
	
	// View matrix transformation methods.

	CMatrix4x4 CMatrix4x4::LookAtLH(const CVector3& eye, const CVector3& at, const CVector3& up)
	{
		CMatrix4x4 mtx = Identity();
		
		CVector3 zAxis = CVector3(at - eye).Normalized();
		CVector3 xAxis = CVector3::Cross(up, zAxis).Normalized();
		CVector3 yAxis = CVector3::Cross(zAxis, xAxis);
		
		mtx(0, 0) = xAxis.x; mtx(1, 0) = yAxis.x; mtx(2, 0) = zAxis.x;
		mtx(0, 1) = xAxis.y; mtx(1, 1) = yAxis.y; mtx(2, 1) = zAxis.y;
		mtx(0, 2) = xAxis.z; mtx(1, 2) = yAxis.z; mtx(2, 2) = zAxis.z;

		mtx(0, 3) = -CVector3::Dot(xAxis, eye);
		mtx(1, 3) = -CVector3::Dot(yAxis, eye);
		mtx(2, 3) = -CVector3::Dot(zAxis, eye);

		return mtx;
	}

	CMatrix4x4 CMatrix4x4::LookAtRH(const CVector3& eye, const CVector3& at, const CVector3& up)
	{
		CMatrix4x4 mtx = Identity();
		
		CVector3 zAxis = CVector3(at - eye).Normalized();
		CVector3 xAxis = CVector3::Cross(up, zAxis).Normalized();
		CVector3 yAxis = CVector3::Cross(zAxis, xAxis);
		
		mtx(0, 0) = xAxis.x; mtx(1, 0) = yAxis.x; mtx(2, 0) = zAxis.x;
		mtx(0, 1) = xAxis.y; mtx(1, 1) = yAxis.y; mtx(2, 1) = zAxis.y;
		mtx(0, 2) = xAxis.z; mtx(1, 2) = yAxis.z; mtx(2, 2) = zAxis.z;

		mtx(0, 3) = CVector3::Dot(xAxis, eye);
		mtx(1, 3) = CVector3::Dot(yAxis, eye);
		mtx(2, 3) = CVector3::Dot(zAxis, eye);

		return mtx;
	}
	
	// Perspective matrix transformation methods.
	
	CMatrix4x4 CMatrix4x4::PerspectiveFovLH(float fov, float aspect, float nv, float fv)
	{
		CMatrix4x4 mtx;

		float yScale = 1.0f / tanf(fov / 2);
		float xScale = yScale / aspect;

		mtx(0, 0) = xScale;
		mtx(1, 1) = yScale;

		xScale = fv / (fv - nv); // use x scale as temp variable.
		mtx(2, 2) = xScale;
		mtx(2, 3) = -nv * xScale;
		mtx(3, 2) = 1.0f;

		return mtx;
	}	

	CMatrix4x4 CMatrix4x4::PerspectiveFovRH(float fov, float aspect, float nv, float fv)
	{
		CMatrix4x4 mtx;

		float yScale = 1.0f / tanf(fov / 2);
		float xScale = yScale / aspect;

		mtx(0, 0) = xScale;
		mtx(1, 1) = yScale;

		xScale = fv / (fv - nv); // use x scale as temp variable.
		mtx(2, 2) = xScale;
		mtx(2, 3) = nv * xScale;
		mtx(3, 2) = -1.0f;

		return mtx;
	}
	
	// Orthographic matrix transformation methods.

	CMatrix4x4 CMatrix4x4::OrthographicLH(float width, float height, float nv, float fv)
	{
		CMatrix4x4 mtx;

		mtx(0, 0) = 2.0f / width;
		mtx(1, 1) = 2.0f / height;

		mtx(2, 2) = 1.0f / (fv - nv);
		mtx(2, 3) = -nv / (fv - nv);
		mtx(3, 3) = 1.0f;

		return mtx;
	}

	CMatrix4x4 CMatrix4x4::OrthographicRH(float width, float height, float nv, float fv)
	{
		CMatrix4x4 mtx;

		mtx(0, 0) = 2.0f / width;
		mtx(1, 1) = 2.0f / height;

		mtx(2, 2) = 1.0f / (nv - fv);
		mtx(2, 3) = nv / (nv - fv);
		mtx(3, 3) = 1.0f;

		return mtx;
	}
	
	// Method for transposing a 4x4 matrix.
	CMatrix4x4 CMatrix4x4::Transpose(const CMatrix4x4& M)
	{
		CMatrix4x4 mtx;

		mtx(0, 0) = M(0, 0); mtx(1, 0) = M(0, 1); mtx(2, 0) = M(0, 2); mtx(3, 0) = M(0, 3);
		mtx(0, 1) = M(1, 0); mtx(1, 1) = M(1, 1); mtx(2, 1) = M(1, 2); mtx(3, 1) = M(1, 3);
		mtx(0, 2) = M(2, 0); mtx(1, 2) = M(2, 1); mtx(2, 2) = M(2, 2); mtx(3, 2) = M(2, 3);
		mtx(0, 3) = M(3, 0); mtx(1, 3) = M(3, 1); mtx(2, 3) = M(3, 2); mtx(3, 3) = M(3, 3);

		return mtx;
	}
	
	// Method for finding the determinant of a 4x4 matrix.
	float CMatrix4x4::Determinant(const CMatrix4x4& M)
	{
		return (M._11 * M._22 * M._33 * M._44) + (M._11 * M._23 * M._34 * M._42) + (M._11 * M._24 * M._32 * M._43) +
			   (M._12 * M._21 * M._34 * M._43) + (M._12 * M._23 * M._31 * M._44) + (M._12 * M._24 * M._33 * M._41) +
			   (M._13 * M._21 * M._32 * M._44) + (M._13 * M._22 * M._34 * M._41) + (M._13 * M._24 * M._31 * M._42) +
			   (M._14 * M._21 * M._33 * M._42) + (M._14 * M._22 * M._31 * M._43) + (M._14 * M._23 * M._32 * M._41) -
			   (M._11 * M._22 * M._34 * M._43) - (M._11 * M._23 * M._32 * M._44) - (M._11 * M._24 * M._33 * M._42) -
			   (M._12 * M._21 * M._33 * M._44) - (M._12 * M._23 * M._34 * M._41) - (M._12 * M._24 * M._31 * M._43) -
			   (M._13 * M._21 * M._34 * M._42) - (M._13 * M._22 * M._31 * M._44) - (M._13 * M._24 * M._32 * M._41) -
			   (M._14 * M._21 * M._32 * M._43) - (M._14 * M._22 * M._33 * M._41) - (M._14 * M._23 * M._31 * M._42);
	}

	// Method for inverting a 4x4 matrix.
	CMatrix4x4 CMatrix4x4::Inverse(const CMatrix4x4& M)
	{
		float det = Determinant(M);
		if(det == 0)
			return M;

		CMatrix4x4 mtx;

		mtx._11 = (M._22 * M._33 * M._44) + (M._23 * M._34 * M._42) + (M._24 * M._32 * M._43) - (M._22 * M._34 * M._43) - (M._23 * M._32 * M._44) - (M._24 * M._33 * M._42);
		mtx._12 = (M._12 * M._34 * M._43) + (M._13 * M._32 * M._44) + (M._14 * M._33 * M._42) - (M._12 * M._33 * M._44) - (M._13 * M._34 * M._42) - (M._14 * M._32 * M._43);
		mtx._13 = (M._12 * M._23 * M._44) + (M._13 * M._24 * M._42) + (M._14 * M._22 * M._43) - (M._12 * M._24 * M._43) - (M._13 * M._22 * M._44) - (M._14 * M._23 * M._42);
		mtx._14 = (M._12 * M._24 * M._33) + (M._13 * M._22 * M._34) + (M._14 * M._23 * M._32) - (M._12 * M._23 * M._34) - (M._13 * M._24 * M._32) - (M._14 * M._22 * M._33);
		
		mtx._21 = (M._21 * M._34 * M._43) + (M._23 * M._31 * M._44) + (M._24 * M._33 * M._41) - (M._21 * M._33 * M._44) - (M._23 * M._34 * M._41) - (M._24 * M._31 * M._43);
		mtx._22 = (M._11 * M._33 * M._44) + (M._13 * M._34 * M._41) + (M._14 * M._31 * M._43) - (M._11 * M._34 * M._43) - (M._13 * M._31 * M._44) - (M._14 * M._33 * M._41);
		mtx._23 = (M._11 * M._24 * M._43) + (M._13 * M._21 * M._44) + (M._14 * M._23 * M._41) - (M._11 * M._23 * M._44) - (M._13 * M._24 * M._41) - (M._14 * M._21 * M._43);
		mtx._24 = (M._11 * M._23 * M._34) + (M._13 * M._24 * M._31) + (M._14 * M._21 * M._33) - (M._11 * M._24 * M._33) - (M._13 * M._21 * M._34) - (M._14 * M._23 * M._31);
		
		mtx._31 = (M._21 * M._32 * M._44) + (M._22 * M._34 * M._41) + (M._24 * M._31 * M._42) - (M._21 * M._34 * M._42) - (M._22 * M._31 * M._44) - (M._24 * M._32 * M._41);
		mtx._32 = (M._11 * M._34 * M._42) + (M._12 * M._31 * M._44) + (M._14 * M._32 * M._41) - (M._11 * M._32 * M._44) - (M._12 * M._34 * M._41) - (M._14 * M._31 * M._42);
		mtx._33 = (M._11 * M._22 * M._44) + (M._12 * M._24 * M._41) + (M._14 * M._21 * M._42) - (M._11 * M._24 * M._42) - (M._12 * M._21 * M._44) - (M._14 * M._22 * M._41);
		mtx._34 = (M._11 * M._24 * M._32) + (M._12 * M._21 * M._34) + (M._14 * M._22 * M._31) - (M._11 * M._22 * M._34) - (M._12 * M._24 * M._31) - (M._14 * M._21 * M._32);
		
		mtx._41 = (M._21 * M._33 * M._42) + (M._22 * M._31 * M._43) + (M._23 * M._32 * M._41) - (M._21 * M._32 * M._43) - (M._22 * M._33 * M._41) - (M._23 * M._31 * M._42);
		mtx._42 = (M._11 * M._32 * M._43) + (M._12 * M._33 * M._41) + (M._13 * M._31 * M._42) - (M._11 * M._33 * M._42) - (M._12 * M._31 * M._43) - (M._13 * M._32 * M._41);
		mtx._43 = (M._11 * M._23 * M._42) + (M._12 * M._21 * M._43) + (M._13 * M._22 * M._41) - (M._11 * M._22 * M._43) - (M._12 * M._23 * M._41) - (M._13 * M._21 * M._42);
		mtx._44 = (M._11 * M._22 * M._33) + (M._12 * M._23 * M._31) + (M._13 * M._21 * M._32) - (M._11 * M._23 * M._32) - (M._12 * M._21 * M._33) - (M._13 * M._22 * M._31);
		
		return mtx * (1.0f / det);
	}
};