//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Math\CMathQuaternion.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMATHQUATERNION_H
#define CMATHQUATERNION_H

#include "CMathVector3.h"
#include "CMathFloat.h"

namespace Math
{
	// CQuaternion class
	// Class for creating a quaternion. Designed to work well with CVector3 euler vectors.
	class CQuaternion
	{
	public:
		float x;
		float y;
		float z;
		float w;

		CQuaternion();
		CQuaternion(float W, float X, float Y, float Z);
		CQuaternion(const CVector3& euler);
		CQuaternion(const CQuaternion& other);
		CQuaternion(CQuaternion&& other);

		inline void							Identity();

		CQuaternion&						operator =	(const CQuaternion& q);

		CQuaternion							operator *	(const CQuaternion& q);
		CQuaternion&						operator *=	(const CQuaternion& q);

		void								RotateX(float theta);
		void								RotateY(float theta);
		void								RotateZ(float theta);
		void								RotateAxis(const CVector3& axis, float theta);
		void								RotateEuler(CVector3 euler);
		CVector3							ToEuler();

		CQuaternion							Normalize();
		CQuaternion							Conjugate();
		CQuaternion							Inverse();
		float								Dot(const CQuaternion& q);
		inline float						LengthSq();
		inline float						Length();

		static CQuaternion					Normalize(CQuaternion q);
		static CQuaternion					Conjugate(const CQuaternion& q);
		static CQuaternion					Inverse(CQuaternion q);
		static CQuaternion					Slerp(const CQuaternion& from, const CQuaternion& to, float t);
		static float						Dot(const CQuaternion& q0, const CQuaternion& q1);
		static float						LengthSq(const CQuaternion& q);
		static float						Length(const CQuaternion& q);
	};

	// Identity setting method.
	inline void CQuaternion::Identity()
	{
		x = 0.0f;
		y = 0.0f;
		z = 0.0f;
		w = 1.0f;
	}

	// Length getters.

	inline float CQuaternion::LengthSq()
	{
		return (w * w + x * x + y * y + z * z);
	}

	inline float CQuaternion::Length()
	{
		return sqrtf(LengthSq());
	}

	// CMatrix4x4 constants.
	const CQuaternion QUAT_IDENTITY = CQuaternion(
		1.0f, 0.0f, 0.0f, 0.0f
	);
};

#endif
