//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Resources\Shaders\Debug.hlsl
//
//-------------------------------------------------------------------------------------------------

// Matrix constant buffer.
cbuffer MatrixBuffer : register(b0)
{
	float4x4	World;
	float4x4	View;
	float4x4	Projection;
};

// Data constant buffer.
cbuffer ConstantBuffer : register(b1)
{
	float4		Color;
	float4		Scale;
};

// Vertex input structure.
struct VIn
{
	float4 Position			: POSITION;
};

// Vertex output structure.
struct VOut
{
	float4 Position			: SV_POSITION;
	float4 Color			: COLOR;
	float4 ViewPoint		: Position;
};

// Pixel output structure.
struct POut
{
	float4		Color			: SV_TARGET0;
	float4		Diffuse			: SV_TARGET1;
};

// Vertex shader function.
VOut VShader(VIn input)
{
	VOut output;

	input.Position.w	= 1.0f;
	input.Position.xyz	*= Scale.xyz;
	output.Position		= mul(input.Position, World);
	output.Position		= mul(output.Position, View);
	output.ViewPoint	= output.Position;
	output.Position		= mul(output.Position, Projection);

	output.Color		= Color;

	return output;
}

// Pixel shader function.
POut PShader(VOut input) : SV_TARGET
{
	POut output;

	output.Diffuse = float4(1, 1, 1, 0.0f);
	output.Color = input.Color;

	return output;
}