//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUIButton.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CUIButton.h"

namespace UI
{
	CButton::CButton()
	{
		m_bToggleButton						= false;
		m_toggleIndex						= 0;
		m_toggleMax							= 0;
		m_pToggleOptions					= NULL;

		m_bUseBackground					= false;
	}

	CButton::CButton(const CButton& other)
	{
	}

	CButton::~CButton()
	{
	}

	// Method for setting the button's initial data.
	int CButton::SetupButton(App::CInput*& input)
	{
		int retCode = Util::Err::NONE;

		m_pInput = input;

		return retCode;
	}

	// Method for making this button a toggle button, or more reverting it to a regular button.
	int CButton::SetupToggleButton(std::string* toggleOptions, int optionCount, int currentIndex)
	{
		int retCode = Util::Err::NONE;

		if(toggleOptions && optionCount > 0)
		{
			m_bToggleButton					= true;
			m_toggleIndex					= currentIndex;
			m_toggleMax						= optionCount;
			m_pToggleOptions				= toggleOptions;
		}
		else
		{
			m_bToggleButton					= false;
			m_toggleIndex					= 0;
			m_toggleMax						= 0;
			m_pToggleOptions				= NULL;
		}

		return retCode;
	}

	// Method for setting this buttons text (only affects regular buttons).
	int CButton::SetTextData(const std::string& text, const Math::CVector3& offset, float size, const Math::CColor& color, const Math::CColor& selectedColor, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		m_textData.text						= text;
		m_textData.offset					= offset;
		m_textData.size						= size;
		m_textData.color					= color;
		m_textData.selectedColor			= selectedColor;
		m_textData.hAlign					= hAlign;
		m_textData.vAlign					= vAlign;

		return retCode;
	}

	// Method for displaying a button on the screen.
	int CButton::DisplayButton(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, const Math::CColor& selectedColor, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		m_bUseBackground = m_pImage ? true : false;

		if(m_bUseBackground)
		{
			if((retCode = m_pImage->DisplayImage(position, scale, m_bHovered ? selectedColor : color, m_fade, hAlign, vAlign)) != Util::Err::NONE)
				return retCode;

			if(m_pFont)
			{
				if((retCode = m_pFont->Print(m_bToggleButton ? m_pToggleOptions[m_toggleIndex] : m_textData.text, m_textData.offset, m_textData.size, m_bHovered ? m_textData.selectedColor : m_textData.color, m_fade, m_textData.hAlign, m_textData.vAlign, &m_pImage->LastBounds())) != Util::Err::NONE)
					return retCode;
			}

			m_bounds = m_pImage->LastBounds();
		}

		CheckInteraction();

		return retCode;
	}

	// Method for displaying a "text only" on the screen.
	int CButton::DisplayTextButton(const std::string& text, const Math::CVector3& position, float size, const Math::CColor& color, const Math::CColor& selectedColor, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		m_bUseBackground = false;

		if(m_pFont)
		{
			if((retCode = m_pFont->Print(m_bToggleButton ? m_pToggleOptions[m_toggleIndex] : text, position, size, m_bHovered ? selectedColor : color, m_fade, hAlign, vAlign)) != Util::Err::NONE)
				return retCode;
		}

		m_bounds = m_pFont->LastBounds();

		CheckInteraction();

		return retCode;
	}

	// Method for Calculating the bounds of the button, and test user interation with the it.
	void CButton::CheckInteraction()
	{
		Math::CVector2 pos;
		
		m_bHovered = false;

		if(m_pInput)
		{
			if(m_pImage)
				pos = m_pImage->Material()->Camera()->ScreenToWorld(m_pInput->MouseData().pos);
			else if(m_pFont)
				pos = m_pFont->Material()->Camera()->ScreenToWorld(m_pInput->MouseData().pos);

			if(m_bounds.x < pos.x && m_bounds.y < pos.y && 
				m_bounds.z > pos.x &&m_bounds.w > pos.y)
			{
				m_bHovered = true;
			}

			m_bClicked = false;
			
			if(m_pInput->MouseDown(0))
			{
				if(m_bHovered)
				{
					if(m_bToggleButton)
					{
						m_toggleIndex = (m_toggleIndex + 1) % m_toggleMax;
					}

					m_bClicked = true;
				}
			}
		}
	}
};