//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUICheckbox.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CUICheckbox.h"

namespace UI
{
	CCheckbox::CCheckbox()
	{
		m_imageCount						= 2;

		m_bChecked							= false;

		m_bTextHangRight					= false;
	}

	CCheckbox::CCheckbox(const CCheckbox& other)
	{
	}

	CCheckbox::~CCheckbox()
	{
	}
	
	// Method for setting up the checkbox's initial data.
	int CCheckbox::SetupCheckbox(App::CInput*& input)
	{
		int retCode = Util::Err::NONE;

		m_pInput = input;

		return retCode;
	}

	// Method for setting the text that hangs to the left or right of a checkbox.
	int CCheckbox::SetupCheckboxText(const std::string& text, float size, const Math::CColor& color, bool hangRight)
	{
		int retCode = Util::Err::NONE;

		m_text				= text;
		m_bTextHangRight	= hangRight;
		m_textSize			= size;
		m_textColor			= color;

		return retCode;
	}

	// Method for setting the data for the checkbox check icon.
	int CCheckbox::SetupCheckIcon(const Math::CColor& color)
	{
		int retCode = Util::Err::NONE;

		m_checkIconColor = color;
		
		return retCode;
	}

	// Method for displaying a checkbox on the screen.
	int CCheckbox::DisplayCheckbox(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		if(m_pImages)
		{
			if(m_pImages[0])
			{
				if(m_pFont)
				{
					if((retCode = m_pFont->Print(m_text, position + Math::CVector3(m_bTextHangRight ? 1.25f * scale.x : 0.0f, 0, 0), m_textSize, m_textColor, m_fade, hAlign, vAlign)) != Util::Err::NONE)
						return retCode;
					
					m_bounds = m_pFont->LastBounds();
				}

				if((retCode = m_pImages[0]->DisplayImage(Math::CVector3(m_bTextHangRight ? -1.25f * scale.x : 1.25f * scale.x, 0, 0), scale, color, m_fade, m_bTextHangRight ? UI_HALIGN::Left : UI_HALIGN::Right, UI_VALIGN::Center, &m_bounds)) != Util::Err::NONE)
					return retCode;

				m_bounds = m_pImages[0]->LastBounds();

				if(m_pImages[1] && m_bChecked)
				{
					if((retCode = m_pImages[1]->DisplayImage(Math::CVector3::Zero(), scale * 0.85f, m_checkIconColor, m_fade, UI_HALIGN::Center, UI_VALIGN::Center, &m_bounds)) != Util::Err::NONE)
						return retCode;
				}
			}
		}

		CheckInteraction();

		return retCode;
	}

	// Method for calculating the bounds of the checkbox, and test for user interaction with the it.
	void CCheckbox::CheckInteraction()
	{
		Math::CVector2 pos;
		
		m_bHovered = false;

		if(m_pInput)
		{
			if(m_pImages[0])
			{
				pos = m_pImages[0]->Material()->Camera()->ScreenToWorld(m_pInput->MouseData().pos);

				if(m_bounds.x < pos.x && m_bounds.y < pos.y && 
					m_bounds.z > pos.x && m_bounds.w > pos.y)
				{
					m_bHovered = true;
				}
			}

			m_bClicked = false;

			if(m_pInput->MouseDown(0))
			{
				if(m_bHovered)
				{
					m_bClicked = true;
					m_bChecked = !m_bChecked;
				}
			}
		}
	}
};