//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUICheckbox.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CUICHECKBOX_H
#define CUICHECKBOX_H

#include "CUIObject.h"
#include "..\Application\CInput.h"

namespace UI
{
	// CCheckbox class.
	// Class for creating a UI checkbox.
	class CCheckbox : public CUIObject
	{
	public:
		CCheckbox();
		CCheckbox(const CCheckbox& other);
		virtual ~CCheckbox();
		
		int									SetupCheckbox(App::CInput*& input);
		
		int									SetupCheckboxText(const std::string& text, float size, const Math::CColor& color, bool hangRight = false);
		int									SetupCheckIcon(const Math::CColor& color);
		int									DisplayCheckbox(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign);
		
		inline void							Checked(bool checked);
		inline bool							Checked() const;

		inline void							FontRenderer(CFontRenderer* font);
		inline void							Background(CImage* background);
		inline void							CheckIcon(CImage* checkIcon);

	protected:
		void								CheckInteraction() override;

		bool								m_bChecked;

		Math::CColor						m_checkIconColor;

		std::string							m_text;
		bool								m_bTextHangRight;
		float								m_textSize;
		Math::CColor						m_textColor;
	};

	// Checked boolean getter and setter.

	inline void CCheckbox::Checked(bool checked)
	{
		m_bChecked = checked;
	}

	inline bool CCheckbox::Checked() const
	{
		return m_bChecked;
	}

	// Font renderer setter.
	inline void CCheckbox::FontRenderer(CFontRenderer* font)
	{
		m_pFont = font;
	}
	
	// Background image setter.
	inline void CCheckbox::Background(CImage* background)
	{
		m_pImages[0] = background;
	}

	// Check icon image setter.
	inline void CCheckbox::CheckIcon(CImage* checkIcon)
	{
		m_pImages[1] = checkIcon;
	}
};

#endif