//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUIFontRenderer.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CUIFontRenderer.h"

namespace UI
{
	CFontRenderer::CFontRenderer()
	{
		m_kerning							= 1.0f;
		m_pFreeType							= NULL;
	}

	CFontRenderer::CFontRenderer(const CFontRenderer& other)
	{
	}

	CFontRenderer::~CFontRenderer()
	{
	}

	// Method for initializing the font rendering.
	int CFontRenderer::Initialize()
	{
		int retCode = Util::Err::NONE;
		
		m_bEnableIndices		= false;
		m_bEnableInstancing		= true;

		m_vertexStride			= sizeof(FT2_VERTEX);
		m_instanceStride		= sizeof(FT2_INSTANCE);
		
		m_instanceAllocation	= 1000;

		// Create a character instance quad.
		AddVertex(new FT2_VERTEX(Math::CVector3( 0.0f,  0.0f, 0.0f)));
		//AddVertex(new FT2_VERTEX(Math::CVector3( 1.0f,  0.0f, 0.0f)));
		//AddVertex(new FT2_VERTEX(Math::CVector3( 1.0f,  1.0f, 0.0f)));
		//AddVertex(new FT2_VERTEX(Math::CVector3( 0.0f,  1.0f, 0.0f)));

		m_primitiveTopology = D3D11_PRIMITIVE_TOPOLOGY_POINTLIST;

		if((retCode = CGeoObject::Initialize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	int CFontRenderer::Update()
	{
		int retCode = Util::Err::NONE;

		if((retCode = CGeoObject::Update()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	int CFontRenderer::Render()
	{
		int retCode = Util::Err::NONE;

		m_pMaterial->Texture(m_pFreeType->FontTexture());

		if((retCode = CGeoObject::Render()) != Util::Err::NONE)
			return retCode;
			
		// After render, reset the instance count.
		m_instanceCount = 0;
		
		return retCode;
	}

	void CFontRenderer::Release()
	{
		CGeoObject::Release();
	}
	
	// Method for printing out a text string.
	int CFontRenderer::Print(const std::string& text, const Math::CVector3& position, float size, const Math::CColor& color, float fade, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		float width = (m_pMaterial->Camera()->Width() * m_pMaterial->Camera()->Aspect());
		float height = (m_pMaterial->Camera()->Height());
		Math::CVector4 bounds(width / -2, height / -2, width / 2, height / 2);
		if((retCode = Print(text, position, size, color, fade, hAlign, vAlign, &bounds)) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for printing out a text string relative to a target bounds.
	int CFontRenderer::Print(const std::string& text, const Math::CVector3& position, float size, const Math::CColor& color, float fade, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign, Math::CVector4* targetBounds)
	{
		int retCode = Util::Err::NONE;

		if(text.length() <= 0)
			return retCode;
			
		if(m_instanceCount >= m_instanceAllocation)
		{
			Util::Log("Instance count is trying to exceed the allocated amount!");
			return retCode;
		}

		// Create the scale based on the inputted size.
		Math::CVector3 scale = CalculateScaleFromSize(size);

		u32 counter = 0;
		u32 charValue, charIndex, i;

		// Store a flag that state whether this font has kerning in place.
		bool hasKern = FT_HAS_KERNING(m_pFreeType->m_face) ? true : false;

		CFreeType::CharacterData* cd = m_pFreeType->m_characterData;
		
		Math::CVector2 pos = CalculateStartingPoint(text, position, Math::CVector2(scale.x, scale.y), hAlign, vAlign, targetBounds);

		// Cycle through the text string, and create an instance quad for each character.
		for(i = 0; i < text.length(); i++)
		{
			charValue = (u32)text[i];

			if(charValue == 0x20)
			{
				// Handle space ' ' characters.
				pos.x += scale.x * cd[0].width * m_kerning;
			}
			else if(charValue > 0x20 && charValue < FONT_CHAR_COUNT + 0x20)
			{
				charIndex = charValue - 0x20;

				if(i > 0 && hasKern)
				{
					// Handel character kerning if any exists in this font.
					FT_UInt i0 = FT_Get_Char_Index(m_pFreeType->m_face, text[i - 1]);
					FT_UInt i1 = FT_Get_Char_Index(m_pFreeType->m_face, text[i]);
					FT_Vector vecKern;

					FT_Get_Kerning(m_pFreeType->m_face, i0, i1, FT_KERNING_DEFAULT, &vecKern);

					pos.x += scale.x * (float)(vecKern.x >> 6);
				}

				// Set the font instance data for this charater.
				float padding = (((m_pFreeType->m_padding * 1.0f) / m_pFreeType->m_width));

				FT2_INSTANCE* instance = &((FT2_INSTANCE*)m_pInstances)[m_instanceCount + counter];

				instance->uvOffset.x = cd[charIndex].uvDim.x - padding * 0.5f;
				instance->uvOffset.y = cd[charIndex].uvDim.y - padding * 0.5f;
				instance->uvOffset.z = cd[charIndex].uvDim.z + padding;
				instance->uvOffset.w = cd[charIndex].uvDim.w + padding;
				
				pos.x += scale.x * cd[charIndex].kerning.x;
				
				instance->position.x = pos.x;
				instance->position.y = pos.y - cd[charIndex].hang * scale.y;
				instance->position.z = position.z;
				
				float adjustScaleX = 1.0f + ((float)m_pFreeType->m_padding) / cd[charIndex].fontDim.z;
				float adjustScaleY = 1.0f + ((float)m_pFreeType->m_padding) / cd[charIndex].fontDim.w;
				
				instance->color = color;
				instance->color.a *= fade;
				instance->scale = Math::CVector3(scale.x * cd[charIndex].fontDim.z * adjustScaleX, scale.y * cd[charIndex].fontDim.w * adjustScaleY, scale.z);
				
				counter++;

				pos.x += scale.x * ((cd[charIndex].fontDim.z) + cd[charIndex].kerning.y) * m_kerning;
			}
		}

		m_instanceCount += counter;

		return retCode;
	}

	// Method for calculating the starting point for a printed string.
	Math::CVector2 CFontRenderer::CalculateStartingPoint(const std::string& text, const Math::CVector3& position, const Math::CVector2& scale, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign, Math::CVector4* targetBounds)
	{
		if(!targetBounds)
			return Math::CVector2::Zero();

		u32 charValue, i;
		Math::CVector2 pos;

		CFreeType::CharacterData* cd = m_pFreeType->m_characterData;
		
		int charIndex = text[text.length() - 1] - 0x20;
		float maxLineHeight = 0.0f;
		float maxHang = 0.0f;

		// Store a flag that state whether this font has kerning in place.
		bool hasKern = FT_HAS_KERNING(m_pFreeType->m_face) ? true : false;
		
		// Calculate the text line height and hang.
		for(i = 0; i < text.length(); i++)
		{
			charValue = (u32)text[i];

			if(charValue >= 0x20 && charValue < FONT_CHAR_COUNT + 0x20)
			{
				charIndex = charValue - 0x20;

				maxLineHeight = max(maxLineHeight, (cd[charIndex].fontDim.w - cd[charIndex].hang));
				maxHang = max(maxHang, cd[charIndex].hang);
			}
		}
		
		// Calculate the text line length.
		m_lineLength = 0;

		for(i = 0; i < text.length(); i++)
		{
			charValue = (u32)text[i];
			
			if(charValue >= 0x20 && charValue < FONT_CHAR_COUNT + 0x20)
			{
				charIndex = charValue - 0x20;
				
				if(i > 0 && hasKern)
				{
					// Handle character kerning if any exists in this font.
					m_lineLength += scale.x * GetKernValue(text[i - 1], text[i]);
				}

				m_lineLength += scale.x * cd[charIndex].width * m_kerning;
			}
		}

		// Horizontal options.
		if(hAlign == UI_HALIGN::Left)
		{
			pos.x = position.x + targetBounds->x;
		}
		else if(hAlign == UI_HALIGN::Center)
		{
			pos.x = position.x + ((targetBounds->z + targetBounds->x - m_lineLength) / 2);
		}
		else if(hAlign == UI_HALIGN::Right)
		{
			pos.x = -position.x + targetBounds->z - m_lineLength;
		}

		// Vertical options.
		if(vAlign == UI_VALIGN::Top)
		{
			pos.y = -position.y + (targetBounds->w - scale.y * maxLineHeight);
		}
		else if(vAlign == UI_VALIGN::Center)
		{
			pos.y = -position.y + ((targetBounds->w + targetBounds->y - scale.y * maxLineHeight) / 2);
		}
		else if(vAlign == UI_VALIGN::Bottom)
		{
			pos.y = position.y + targetBounds->y;
		}

		m_lastBounds.x = pos.x;
		m_lastBounds.y = pos.y;
		m_lastBounds.z = pos.x + m_lineLength;
		m_lastBounds.w = pos.y + scale.y * maxLineHeight;

		return pos;
	}

	// Method for calculating the kern between two characters.
	float CFontRenderer::GetKernValue(char c0, char c1)
	{
		// Handel character kerning if any exists in this font.
		FT_UInt i0 = FT_Get_Char_Index(m_pFreeType->m_face, c0);
		FT_UInt i1 = FT_Get_Char_Index(m_pFreeType->m_face, c1);
		FT_Vector vecKern;

		FT_Get_Kerning(m_pFreeType->m_face, i0, i1, FT_KERNING_DEFAULT, &vecKern);

		return (float)(vecKern.x >> 6);
	}

	// Method for calculating the length of a given string, and at a given size.
	float CFontRenderer::CalculateLineLength(const std::string& text, float size)
	{
		if(text.length() <= 0)
			return 0.0f;

		Math::CVector3 scale = CalculateScaleFromSize(size);

		u32 charValue, i;
		int charIndex;

		CFreeType::CharacterData* cd = m_pFreeType->m_characterData;
		
		// Store a flag that state whether this font has kerning in place.
		bool hasKern = FT_HAS_KERNING(m_pFreeType->m_face) ? true : false;
		
		// Calculate the text line length.
		float lineLength = 0.0f;

		for(i = 0; i < text.length() - 1; i++)
		{
			charValue = (u32)text[i];
			
			if(charValue >= 0x20 && charValue < FONT_CHAR_COUNT + 0x20)
			{
				charIndex = charValue - 0x20;
				
				if(i > 0 && hasKern)
				{
					// Handle character kerning if any exists in this font.
					lineLength += scale.x * GetKernValue(text[i - 1], text[i]);
				}

				lineLength += scale.x * cd[charIndex].width * m_kerning;
			}
		}

		charIndex = (u32)text[text.length() - 1] - 0x20;
		if(charIndex >= 0 && charIndex < FONT_CHAR_COUNT)
			lineLength += scale.x * (cd[charIndex].width);

		return lineLength;
	}

	int CFontRenderer::ClosestIndexRelativeToPoint(const Math::CVector2& point, const std::string& text, const Math::CVector3 position, float size, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign, Math::CVector4* targetBounds)
	{
		if(text.length() <= 0)
			return 0;

		int index = 0;

		// Create the scale based on the inputted size.
		Math::CVector3 scale = CalculateScaleFromSize(size);

		u32 counter = 0;
		u32 charValue, charIndex, i;

		CFreeType::CharacterData* cd = m_pFreeType->m_characterData;
		
		// Store a flag that state whether this font has kerning in place.
		bool hasKern = FT_HAS_KERNING(m_pFreeType->m_face) ? true : false;
		
		Math::CVector2 pos = CalculateStartingPoint(text, position, Math::CVector2(scale.x, scale.y), hAlign, vAlign, targetBounds);
		
		float lineLength = 0.0f;
		float cp, closestPoint;
		cp = closestPoint = FLT_MAX;

		// Get closest index base on point position.
		for(i = 0; i < text.length(); i++)
		{
			lineLength = 0.0f;
			charValue = (u32)text[i];
			
			if(charValue >= 0x20 && charValue < FONT_CHAR_COUNT + 0x20)
			{
				charIndex = charValue - 0x20;
				
				if(i > 0 && hasKern)
				{
					// Handle character kerning if any exists in this font.
					lineLength += scale.x * GetKernValue(text[i - 1], text[i]);
				}

				lineLength += scale.x * cd[charIndex].width * m_kerning;

				// Check for closer index.
				if(i == 0)
				{
					closestPoint = fabsf(pos.x - point.x);
					index = 0;
				}
				else
				{
					cp = fabsf(pos.x - point.x);
					if(cp < closestPoint)
					{
						index = i;
						closestPoint = cp;
					}
				}

				pos.x += lineLength;
			}
		}

		cp = fabsf(pos.x - point.x);
		if(cp < closestPoint)
		{
			index = i;
			closestPoint = cp;
		}

		return index;
	}
};