//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUIFreeType.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CUIFreeType.h"

namespace UI
{
	CFreeType::CFreeType()
	{
		m_pt								= 0;
		m_ptScale							= 0.0f;
		m_dpi								= 72;
		m_padding							= 0;

		m_width								= 0;
		m_height							= 0;

		m_pixelSize							= 0;
		m_pPixels							= NULL;

		m_library							= NULL;
		m_face								= NULL;
		m_slot								= NULL;

		m_pTexture2d						= NULL;
		m_pTexture							= NULL;
	}

	CFreeType::CFreeType(const CFreeType& other)
	{
	}

	CFreeType::~CFreeType()
	{
	}

	// Method for initializing a font w/ an object to generate a new for or to not.
	int CFreeType::Initialize(bool genFont)
	{
		int retCode = Util::Err::NONE;
		
		FT_Init_FreeType(&m_library);
		
		m_pt								= 300;
		m_ptScale							= 64.0f / m_pt;
		m_padding							= 28;
		m_width								= 2048;
		m_height							= 2048;

		if(genFont)
		{
			if((retCode = CreateFontTexture()) != Util::Err::NONE)
				return retCode;
		}
		else
		{
			// If this function is set to not generate a font file, set all of the needed freetype variables up.
			FT_New_Face(m_library, m_filename.c_str(), 0, &m_face);
			FT_Set_Char_Size(m_face, 0, m_pt * 64, m_dpi, m_dpi);
			m_slot = m_face->glyph;
		}

		if(m_pTexture2d)
		{
			CreateTextureFrom2D();
			//m_pDevCon->PSSetShaderResources(4, 1, &m_pTexture->Texture());
		}

		return retCode;
	}

	// Method for creating a new 2d texture with a font's data in it.
	int CFreeType::CreateFontTexture()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = CreateTexture2D()) != Util::Err::NONE)
			return retCode;

		FT_New_Face(m_library, m_filename.c_str(), 0, &m_face);
		FT_Set_Char_Size(m_face, 0, m_pt * 64, m_dpi, m_dpi);
		m_slot = m_face->glyph;

		D3D11_MAPPED_SUBRESOURCE ms;

		// Add font data to the newly created 2d texture.
		m_pDevCon->Map(m_pTexture2d, NULL, D3D11_MAP_WRITE_DISCARD, NULL, &ms);
		
		m_pixelSize = m_height * ms.RowPitch;
		m_pPixels = new u8[m_pixelSize];
		memset(m_pPixels, 1, m_pixelSize);
		u8* pixels = (u8*)ms.pData;

		u32 i, j, k, l, m;
		u32 x = 0, y = 0;
		bool foundSlot = false;
		u32 paddingHalf = m_padding;

		// Store the data for each character of interest in the font file.
		for(i = 0; i < FONT_CHAR_COUNT; i++)
		{
			FT_Load_Char(m_face, (char)(i + 0x20), FT_LOAD_RENDER | FT_LOAD_TARGET_LIGHT);

			x = 0;
			y = 0;
			foundSlot = false;

			for(j = m_padding; j < ms.RowPitch - paddingHalf; j += 8)
			{
				for(k = m_padding; k < m_height - paddingHalf; k += 8)
				{
					// Check if the top-left pixel is empty.
					if(m_pPixels[k * ms.RowPitch + j] == 1)
					{
						foundSlot = j < ms.RowPitch - paddingHalf && k < m_height - paddingHalf;

						if(foundSlot)
						{
							l = j;
							m = k;

							// Check if the bounds of this pixel can fit in this space.
							for(l = j; l < j + m_slot->bitmap.width + m_padding * 2 && l < ms.RowPitch - paddingHalf; l++)
							{
								for(m = k; m < k + m_slot->bitmap.rows + m_padding * 2 && m < m_height - paddingHalf; m++)
								{
									if(m_pPixels[m * ms.RowPitch + l] != 1)
									{
										l = j + m_slot->bitmap.width + m_padding * 2;
										m = k + m_slot->bitmap.rows + m_padding * 2;
										foundSlot = false;
									}
								}
							}
						}

						if(foundSlot && 
							l + 1 >= j + m_slot->bitmap.width + m_padding * 2 && l < ms.RowPitch - paddingHalf && 
							m + 1 >= k + m_slot->bitmap.rows + m_padding * 2 && m < m_height - paddingHalf)
						{
							x = j + m_padding;
							y = k + m_padding;
							j = m_width;
							k = m_height;
						}
					}
				}
			}
			
			// If an empty space is found, place ths glyph character there.
			if(foundSlot)
				WriteGlyph(i, m_pPixels, m_slot, x, y, m_slot->bitmap, m_slot->bitmap_left, m_slot->bitmap_top, ms.RowPitch);
		}

		// Remove all empty space markers from the texure.
		for(i = 0; i < ms.RowPitch; i++)
		{
			for(j = 0; j < m_height; j++)
			{
				if(m_pPixels[j * ms.RowPitch + i] == 1)
				{
					m_pPixels[j * ms.RowPitch + i] = 0;
				}
			}
		}
		
		memcpy(pixels, m_pPixels, m_pixelSize);

		m_pDevCon->Unmap(m_pTexture2d, NULL);
		
		return retCode;
	}

	// Method for writing a glyph to a 2d texture, and for storing a glyphs character data.
	int CFreeType::WriteGlyph(int index, u8*& bits, FT_GlyphSlot slot, u32 x, u32 y, FT_Bitmap bitmap, int left, int top, int pitch)
	{
		int retCode = Util::Err::NONE;
		
		int i, j;
		u32 p, q;
		int xMax = left + bitmap.width;
		int yMax = top + bitmap.rows;
		u32 yStart = y;

		int advanceX = slot->metrics.horiAdvance;
		int bearingX = slot->metrics.horiBearingX;
		int width = slot->metrics.width;
		int hangX = advanceX - (bearingX + width);

		// Store the characted data of the glyph passed to this function for writing.
		m_characterData[index].width = (float)(slot->metrics.horiAdvance >> 6);
		m_characterData[index].height = (float)(slot->metrics.vertAdvance >> 6);
		m_characterData[index].hang = (float)((slot->metrics.height - slot->metrics.horiBearingY) >> 6);

		m_characterData[index].kerning.x = (float)(slot->metrics.horiBearingX >> 6);
		m_characterData[index].kerning.y = (float)(hangX >> 6);

		m_characterData[index].fontDim.x = ((float)left);
		m_characterData[index].fontDim.y = ((float)top);
		m_characterData[index].fontDim.z = ((float)bitmap.width);
		m_characterData[index].fontDim.w = ((float)bitmap.rows);

		m_characterData[index].uvDim.x = ((float)x) / m_width;
		m_characterData[index].uvDim.y = ((float)y) / m_height;
		m_characterData[index].uvDim.z = ((float)bitmap.width) / m_width;
		m_characterData[index].uvDim.w = ((float)bitmap.rows) / m_height;

		// Add the glyphs pixel data to the 2d texture.
		for(i = left, p = 0; i < xMax; i++, p++, x++)
		{
			for(j = top, q = 0, y = yStart; j < yMax; j++, q++, y++)
			{
				bits[y * pitch + x] = bitmap.buffer[q * bitmap.width + p];
			}
		}

		return retCode;
	}

	void CFreeType::Release()
	{
		SAFE_DELETE_ARRAY(m_pPixels);
		SAFE_RELEASE_DELETE(m_pTexture);
		
		FT_Done_Face(m_face);
		FT_Done_FreeType(m_library);
	}

	// Method for creating a font file purely based on the texture generated.
	int CFreeType::GenerateFontFile(const std::string& filename)
	{
		int retCode = Util::Err::NONE;

		FILE *file;

		fopen_s(&file, filename.c_str(), "wb");

		fwrite(m_pPixels, 1, m_pixelSize, file);

		fwrite(&m_padding, sizeof(u32), 1, file);
		fwrite(m_characterData, sizeof(CharacterData), FONT_CHAR_COUNT, file);

		fclose(file);

		return retCode;
	}
	
	// Method for create a font file based on the texture generated, with a signed distance field in place.
	int CFreeType::GenerateSDFFontFile(const std::string& filename)
	{
		int retCode = Util::Err::NONE;
		
		FILE *file;

		fopen_s(&file, filename.c_str(), "wb");
		
		u8 pixel = 0;
		u8* sdfPixels = new u8[m_pixelSize];
		memset(sdfPixels, 0, m_pixelSize);

		int i, j, k, l;
		int outerArea = 12;
		int innerArea = 8;

		float multiplier = 8.0f;
		float innerMin = 256.0f - multiplier * innerArea;

		float dist = 0.0f;
		float minDist = 0.0f;

		// Generate the signed distance field from the font texture.
		for(i = 0; i < (int)m_width; i++)
		{
			for(j = 0; j < (int)m_height; j++)
			{
				minDist = FLT_MAX;

				if(m_pPixels[j * m_width + i] == 0)
				{
					// Handle black pixels.
					for(k = max(0, i - outerArea); k < min((int)m_width, i + outerArea); k++)
					{
						for(l = max(0, j - outerArea); l < min((int)m_height, j + outerArea); l++)
						{
							if(m_pPixels[l * m_width + k] != 0)
							{
								dist = sqrtf(powf(((float)i - (float)k), 2) + powf(((float)j - (float)l), 2));
								if(dist < minDist)
								{
									minDist = dist;
								}
							}
						}
					}

					if(minDist <= outerArea)
					{
						pixel = (u8)max((int)((innerMin + 8.0f) - Math::Clamp(minDist * multiplier, 0.0f, innerMin + 8.0f)), 0);
						sdfPixels[j * m_width + i] = pixel;
					}
				}
				else
				{
					// Handle pixels > (0, black).
					for(k = max(0, i - innerArea); k < min((int)m_width, i + innerArea); k++)
					{
						for(l = max(0, j - innerArea); l < min((int)m_height, j + innerArea); l++)
						{
							if(m_pPixels[l * m_width + k] == 0)
							{
								dist = sqrtf(powf(((float)i - (float)k), 2) + powf(((float)j - (float)l), 2));
								if(dist < minDist)
								{
									minDist = dist;
								}
							}
						}
					}
					
					if(minDist <= innerArea)
					{
						pixel = (u8)min((int)(Math::Clamp(minDist * multiplier, 0.0f, multiplier * innerArea) + innerMin), 255);
						sdfPixels[j * m_width + i] = pixel;
					}
					else
					{
						// Set a pixel to it's default values if it's not to be modified by the signed distance field.
						sdfPixels[j * m_width + i] = m_pPixels[j * m_width + i];
					}
				}
			}
		}
		
		fwrite(sdfPixels, 1, m_pixelSize, file);

		SAFE_DELETE_ARRAY(sdfPixels);
		
		fwrite(&m_padding, sizeof(u32), 1, file);
		fwrite(m_characterData, sizeof(CharacterData), FONT_CHAR_COUNT, file);

		fclose(file);

		return retCode;
	}

	// Method for openning a custom font file for this application.
	int CFreeType::OpenFontFile(const std::string& filename)
	{
		int retCode = Util::Err::NONE;
		
		FILE *fileIn;

		fopen_s(&fileIn, filename.c_str(), "rb");

		// If pixels are still null allocate memory to it.
		if(!m_pPixels)
		{
			m_pixelSize = m_width * m_height;
			m_pPixels = new u8[m_pixelSize];
		}

		// Read all pixel data.
		fread(m_pPixels, sizeof(u8), m_pixelSize, fileIn);

		fread(&m_padding, sizeof(u32), 1, fileIn);
		fread(&m_characterData, sizeof(CharacterData), FONT_CHAR_COUNT, fileIn);
		
		fclose(fileIn);
		
		if(!m_pTexture2d)
		{
			if((retCode = CreateTexture2D()) != Util::Err::NONE)
				return retCode;
		}

		D3D11_MAPPED_SUBRESOURCE ms;

		// Add font data to the newly created 2d texture.
		m_pDevCon->Map(m_pTexture2d, NULL, D3D11_MAP_WRITE_DISCARD, NULL, &ms);
		
		memcpy(ms.pData, m_pPixels, m_pixelSize);

		m_pDevCon->Unmap(m_pTexture2d, NULL);
		
		// Create the texture object based on the stored data in the 2d texture.
		SAFE_RELEASE(m_pTexture);
		CreateTextureFrom2D();

		return retCode;
	}

	// Method for creating a new 2d texture if one does not already exist.
	int CFreeType::CreateTexture2D()
	{
		int retCode = Util::Err::NONE;

		if(!m_pTexture2d)
		{
			// Create the 2d texture that is primarily for storing font data.
			D3D11_TEXTURE2D_DESC td;

			ZeroMemory(&td, sizeof(td));

			td.Width							= m_width;
			td.Height							= m_height;
			td.MipLevels						= 1;
			td.ArraySize						= 1;
			td.Format							= DXGI_FORMAT_A8_UNORM;
			td.SampleDesc.Count					= 1;
			td.SampleDesc.Quality				= 0;
			td.Usage							= D3D11_USAGE_DYNAMIC;
			td.CPUAccessFlags					= D3D11_CPU_ACCESS_WRITE;
			td.BindFlags						= D3D11_BIND_SHADER_RESOURCE;

			if(FAILED(m_pDev->CreateTexture2D(&td, NULL, &m_pTexture2d)))
			{
				return Util::Err::ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D;
			}
		}

		return retCode;
	}

	// Method for creating the texture object from a 2d texture if one exists.
	int CFreeType::CreateTextureFrom2D()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pTexture2d)
		{
			// Create a texture that can be used with shaders.
			m_pTexture = new GFX::CTexture();

			D3D11_SHADER_RESOURCE_VIEW_DESC srvd;
		
			ZeroMemory(&srvd, sizeof(srvd));

			// Create the shader resource view.
			srvd.Format							= DXGI_FORMAT_A8_UNORM;
			srvd.ViewDimension					= D3D11_SRV_DIMENSION_TEXTURE2D;
			srvd.Texture2D.MostDetailedMip		= 0;
			srvd.Texture2D.MipLevels			= 1;
		
			// Create a shader resource from the 2d texture that now has the font data in it.
			m_pTexture->DirectXData(m_pDX11Data);
			if((retCode = m_pTexture->Initialize(m_pTexture2d, &srvd)) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}
};