//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUIImage.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CUIImage.h"

namespace UI
{
	CImage::CImage()
	{
		m_instanceAllocation				= 1;

		m_pTexture							= NULL;
	}

	CImage::CImage(const CImage& other)
	{
	}

	CImage::~CImage()
	{
	}

	// Method for setting a UI image's initial data w/ a filename.
	int CImage::SetupImage(const std::string& filename, u32 instanceAlloc)
	{
		int retCode = Util::Err::NONE;

		m_filename				= filename;
		m_instanceAllocation	= instanceAlloc;

		return retCode;
	}
	
	// Method for setting a UI image's initial data w/ a texture.
	int CImage::SetupImage(GFX::CTexture* texture, u32 instanceAlloc)
	{
		int retCode = Util::Err::NONE;

		m_pTexture				= texture;
		m_instanceAllocation	= instanceAlloc;

		return retCode;
	}

	// Method for initializing a new UI image.
	int CImage::Initialize()
	{
		int retCode = Util::Err::NONE;
		
		m_bEnableIndices		= false;
		m_bEnableInstancing		= true;

		m_vertexStride			= sizeof(IMAGE_VERTEX);
		m_instanceStride		= sizeof(IMAGE_INSTANCE);
		
		// Create a image instance quad.
		AddVertex(new IMAGE_VERTEX(Math::CVector3( 0.0f,  0.0f, 0.0f)));
		//AddVertex(new IMAGE_VERTEX(Math::CVector3( 1.0f,  0.0f, 0.0f)));
		//AddVertex(new IMAGE_VERTEX(Math::CVector3( 1.0f,  1.0f, 0.0f)));
		//AddVertex(new IMAGE_VERTEX(Math::CVector3( 0.0f,  1.0f, 0.0f)));

		m_primitiveTopology = D3D11_PRIMITIVE_TOPOLOGY_POINTLIST;

		if((retCode = CGeoObject::Initialize()) != Util::Err::NONE)
			return retCode;

		// Create texture.
		if(!m_pTexture && m_filename.length() > 0)
		{
			m_pTexture = new GFX::CTexture();

			m_pTexture->DirectXData(m_pDX11Data);
			if((retCode = m_pTexture->Initialize((char*)m_filename.c_str())) != Util::Err::NONE)
				return retCode;
		}
		else if(m_pTexture)
		{
			m_filename = "";
		}

		return retCode;
	}

	int CImage::Update()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = CGeoObject::Update()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for rendering the UI image.
	int CImage::Render()
	{
		int retCode = Util::Err::NONE;

		m_pMaterial->Texture(m_pTexture);
		
		if((retCode = CGeoObject::Render()) != Util::Err::NONE)
			return retCode;

		m_instanceCount = 0;

		return retCode;
	}

	void CImage::Release()
	{
		if(m_filename.length() > 0)
			SAFE_RELEASE_DELETE(m_pTexture);

		CGeoObject::Release();
	}

	// Method for displaying an image.
	int CImage::DisplayImage(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, float fade, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		float width = (m_pMaterial->Camera()->Width() * m_pMaterial->Camera()->Aspect());
		float height = (m_pMaterial->Camera()->Height());
		Math::CVector4 bounds(width / -2, height / -2, width / 2, height / 2);

		DisplayImage(position, scale, color, fade, hAlign, vAlign, &bounds);

		return retCode;
	}

	// Method for displaying an image.
	int CImage::DisplayImage(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, float fade, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign, Math::CVector4* targetBounds)
	{
		int retCode = Util::Err::NONE;

		if(m_instanceCount >= m_instanceAllocation)
		{
			Util::Log("Instance count is trying to exceed the allocated amount!");
			return retCode;
		}

		IMAGE_INSTANCE* instance = &(((IMAGE_INSTANCE*)m_pInstances)[m_instanceCount]);

		Math::CVector2 pos = GetStartingPoint(position, scale, hAlign, vAlign, targetBounds);

		instance->position = Math::CVector3(pos.x, pos.y, position.z);
		instance->scale = scale;
		instance->uvOffset = Math::CVector4(0.0f, 0.0f, 1.0f, 1.0f);
		instance->color = color;
		instance->color.a *= fade;

		m_lastBounds.x = pos.x;
		m_lastBounds.y = pos.y;
		m_lastBounds.z = m_lastBounds.x + scale.x;
		m_lastBounds.w = m_lastBounds.y + scale.y;

		m_instanceCount++;

		return retCode;
	}

	// Method for getting the anchor point for the image.
	Math::CVector2 CImage::GetStartingPoint(const Math::CVector3& position, const Math::CVector2& scale, UI_HALIGN hAlign, UI_VALIGN vAlign, Math::CVector4* targetBounds)
	{
		if(!targetBounds)
			return Math::CVector2::Zero();

		Math::CVector2 pos;

		// Horizontal options.
		if(hAlign == UI_HALIGN::Left)
		{
			pos.x = position.x + targetBounds->x;
		}
		else if(hAlign == UI_HALIGN::Center)
		{
			pos.x = position.x + ((targetBounds->z + targetBounds->x - scale.x) / 2);
		}
		else if(hAlign == UI_HALIGN::Right)
		{
			pos.x = position.x + targetBounds->z - scale.x;
		}

		// Vertical options.
		if(vAlign == UI_VALIGN::Top)
		{
			pos.y = -position.y + (targetBounds->w - scale.y);
		}
		else if(vAlign == UI_VALIGN::Center)
		{
			pos.y = -position.y + ((targetBounds->w + targetBounds->y - scale.y) / 2);
		}
		else if(vAlign == UI_VALIGN::Bottom)
		{
			pos.y = position.y + targetBounds->y;
		}

		return pos;
	}
};