//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUIImage.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CUIIMAGE_H
#define CUIIMAGE_H

#include "..\Graphics\CGeoObject.h"
#include "..\Math\CMathVector2.h"
#include "CUIFontRenderer.h"

namespace UI
{
	// CImage class.
	// Class for creating a ui image instance.
	class CImage : public GFX::CGeoObject
	{
	public:
		// Image vertex structure.
		struct IMAGE_VERTEX
		{
			Math::CVector3					position;

			IMAGE_VERTEX(Math::CVector3 Position)
			{
				position = Position;
			}
		};

		// Image instance structure.
		struct IMAGE_INSTANCE
		{
			Math::CVector3					position;
			Math::CVector3					scale;
			Math::CVector4					uvOffset;
			Math::CColor					color;
		};

	public:
		CImage();
		CImage(const CImage& other);
		virtual ~CImage();

		int									SetupImage(const std::string& filename, u32 instanceAlloc = 1);
		int									SetupImage(GFX::CTexture* texture, u32 instanceAlloc = 1);

		int									Initialize() override;
		int									Update() override;
		int									Render() override;
		void								Release() override;
		
		int									DisplayImage(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, float fade, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign);
		int									DisplayImage(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, float fade, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign, Math::CVector4* targetBounds);

		inline Math::CVector4				LastBounds() const;

	private:
		Math::CVector2						GetStartingPoint(const Math::CVector3& position, const Math::CVector2& scale, UI_HALIGN hAlign, UI_VALIGN vAlign, Math::CVector4* targetBounds);
		
		Math::CVector4						m_lastBounds;

		std::string							m_filename;
		GFX::CTexture*						m_pTexture;
	};
	
	// Last image bounds getter. (For use with point intersections)
	inline Math::CVector4 CImage::LastBounds() const
	{
		return m_lastBounds;
	}
};

#endif