//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUISlider.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CUISlider.h"

namespace UI
{
	CSlider::CSlider()
	{
		m_imageCount						= 3;

		m_bSelected							= false;

		m_sliderOffset						= 0.0f;
		m_normalizeValue					= 0.0f;

		m_minValue							= 0.0f;
		m_maxValue							= 1.0f;
		m_initialValue						= 0.0f;
		m_finalValue						= 0.0f;

		m_bInitialValueSet					= false;
	}

	CSlider::CSlider(const CSlider& other)
	{
	}

	CSlider::~CSlider()
	{
	}

	// Method for setting up the slider.
	int CSlider::SetupSlider(App::CInput*& input)
	{
		int retCode = Util::Err::NONE;

		m_pInput = input;

		return retCode;
	}

	// Method for setting general slider data.
	int CSlider::SetSliderData(float initialValue, float minValue, float maxValue, const Math::CColor& bgColor, const Math::CColor& levelBarColor, const Math::CColor& sliderBarColor)
	{
		int retCode = Util::Err::NONE;

		m_minValue			= minValue;
		m_maxValue			= max(maxValue, minValue + 0.01f);
		m_initialValue		= Math::Clamp(initialValue, m_minValue, m_maxValue);

		m_bgColor			= bgColor;
		m_levelBarColor		= levelBarColor;
		m_sliderBarColor	= sliderBarColor;

		return retCode;
	}
	
	// Method for setting the slider text data.
	int CSlider::SetTextData(float size, const Math::CColor& color, int precision, bool roundVal, bool hangRight)
	{
		int retCode = Util::Err::NONE;
		
		m_textData.size				= size;
		m_textData.color			= color;
		m_textData.precision		= precision;
		m_textData.roundVal			= roundVal;
		m_textData.hangRight		= hangRight;

		return retCode;
	}

	// Method for displaying the slider.
	int CSlider::DisplaySlider(const Math::CVector3& position, const Math::CVector2& scale, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		if(m_pImages)
		{
			// Render background, level bar, and slider bar.
			if(m_pImages[0])
			{
				if((retCode = m_pImages[0]->DisplayImage(position, scale, m_bgColor, m_fade, hAlign, vAlign)) != Util::Err::NONE)
					return retCode;

				m_bounds = m_pImages[0]->LastBounds();
			}
			if(m_pImages[1])
			{
				if((retCode = m_pImages[1]->DisplayImage(position, scale * Math::CVector2(m_normalizeValue, 1.0f), m_levelBarColor, m_fade, hAlign, vAlign)) != Util::Err::NONE)
					return retCode;
			}
			if(m_pImages[2])
			{
				if((retCode = m_pImages[2]->DisplayImage(position + Math::CVector3(m_sliderOffset, 0, 0), Math::CVector2(2.0f, scale.y * 3.0f), m_sliderBarColor, m_fade, hAlign, vAlign)) != Util::Err::NONE)
					return retCode;

				m_bounds2 = m_pImages[2]->LastBounds();
			}

			// Render the current value text.
			if(m_pFont)
			{
				std::ostringstream out;
				out << std::fixed << std::setprecision(m_textData.precision) << m_finalValue;
				if((retCode = m_pFont->Print(out.str(), Math::CVector3(m_textData.hangRight ? 1.0f * scale.x + 2.5f : -1.0f * scale.x - 2.5f, 0.0f, 0), m_textData.size, m_textData.color, m_fade, m_textData.hangRight ? UI_HALIGN::Left : UI_HALIGN::Right, UI_VALIGN::Bottom, &m_bounds)) != Util::Err::NONE)
					return retCode;
			}
		}

		CheckInteraction();

		return retCode;
	}

	// Method for calculating the bounds of the slider, and test for user interaction with the it.
	void CSlider::CheckInteraction()
	{
		Math::CVector2 pos;
		
		m_bHovered = false;

		if(m_pInput)
		{
			if(m_pImages[0])
			{
				pos = m_pImages[0]->Material()->Camera()->ScreenToWorld(m_pInput->MouseData().pos);

				if(!m_bInitialValueSet)
				{
					float maxVal		= m_bounds.z - m_bounds.x - (m_bounds2.z - m_bounds2.x);

					m_finalValue		= m_initialValue;
					m_normalizeValue	= (m_finalValue - m_minValue) / (m_maxValue - m_minValue);
					m_sliderOffset		= m_normalizeValue * maxVal;

					m_bInitialValueSet	= true;
				}

				if(m_bSelected)
				{
					// Calculate the final value based on the slider bar position.
					float maxVal = m_bounds.z - m_bounds.x - (m_bounds2.z - m_bounds2.x);
					m_sliderOffset = pos.x - m_bounds.x + (m_bounds2.x - m_bounds2.z) / 2;
					m_sliderOffset = Math::Clamp(m_sliderOffset, 0.0f, maxVal);
					m_normalizeValue = m_sliderOffset / maxVal;
					m_finalValue = m_normalizeValue * (m_maxValue - m_minValue) + m_minValue;

					if(m_textData.roundVal)
					{
						m_finalValue = Math::RoundToNearest(m_finalValue, m_textData.precision);
					}
				}

				if((m_bounds.x < pos.x && m_bounds.y < pos.y && 
					m_bounds.z > pos.x && m_bounds.w > pos.y))
				{
					m_bHovered = true;
				}
				else if((m_bounds2.x < pos.x && m_bounds2.y < pos.y && 
					m_bounds2.z > pos.x && m_bounds2.w > pos.y))
				{
					m_bHovered = true;
				}
			}

			m_bClicked = false;

			if(m_pInput->MouseDown(0))
			{
				if(m_bHovered)
				{
					m_bClicked = true;
					m_bSelected = true;
				}
			}
			if(!m_pInput->Mouse(0))
			{
				m_bSelected = false;
			}
		}
	}
};