//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUISlider.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CUISLIDER_H
#define CUISLIDER_H

#include <iomanip>
#include "..\UI\CUIObject.h"

namespace UI
{
	// CSlider class.
	// Class for creating a UI slider.
	class CSlider : public CUIObject
	{
	private:
		struct TextData
		{
			Math::CColor					color;
			float							size;
			int								precision;
			bool							roundVal;
			bool							hangRight;
		};

	public:
		CSlider();
		CSlider(const CSlider& other);
		virtual ~CSlider();
		
		int									SetupSlider(App::CInput*& input);
		
		int									SetSliderData(float initialValue, float minValue, float maxValue, const Math::CColor& bgColor, const Math::CColor& levelBarColor, const Math::CColor& sliderBarColor);
		int									SetTextData(float size, const Math::CColor& color, int precision, bool roundVal = true, bool hangRight = false);
		int									DisplaySlider(const Math::CVector3& position, const Math::CVector2& scale, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign);
		
		inline void							FontRenderer(CFontRenderer* font);
		inline void							Background(CImage* background);
		inline void							LevelBar(CImage* levelBar);
		inline void							SliderBar(CImage* sliderBar);

	private:
		void								CheckInteraction() override;

		bool								m_bSelected;

		Math::CColor						m_bgColor;
		Math::CColor						m_levelBarColor;
		Math::CColor						m_sliderBarColor;

		float								m_sliderOffset;
		float								m_normalizeValue;
		float								m_minValue;
		float								m_maxValue;
		float								m_initialValue;
		float								m_finalValue;

		bool								m_bInitialValueSet;

		Math::CVector4						m_bounds2;

		TextData							m_textData;
	};

	// Font renderer setter.
	inline void CSlider::FontRenderer(CFontRenderer* font)
	{
		m_pFont = font;
	}
	
	// Background image setter.
	inline void CSlider::Background(CImage* background)
	{
		m_pImages[0] = background;
	}
	
	// Level bar image setter.
	inline void CSlider::LevelBar(CImage* levelBar)
	{
		m_pImages[1] = levelBar;
	}
	
	// Slider bar image setter.
	inline void CSlider::SliderBar(CImage* sliderBar)
	{
		m_pImages[2] = sliderBar;
	}
};

#endif
