//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUITextField.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CUITextField.h"

namespace UI
{
	CTextField::CTextField()
	{
		m_caretBlinkRate					= 0.5f;
		m_caretBlinkTimer					= 0.0f;

		m_bSelected							= false;

		m_pInputTyping						= NULL;
	}

	CTextField::CTextField(const CTextField& other)
	{
	}

	CTextField::~CTextField()
	{
	}

	// Method for setting up a UI text field.
	int CTextField::SetupTextField(App::CInput*& input, Util::CTimer*& timer, const std::string& initialText)
	{
		int retCode = Util::Err::NONE;
		
		m_pInput							= input;
		m_pTimer							= timer;

		m_textData.initialText				= initialText;
		m_textData.currentText				= initialText;

		if(!m_pInputTyping)
			m_pInputTyping					= new App::CInputTyping();
		
		if((retCode = m_pInputTyping->Initialize(m_pInput, m_pTimer)) != Util::Err::NONE)
			return retCode;

		m_pInputTyping->TypingFlag(App::TYPING_EVERYTHING);
		m_pInputTyping->Text(initialText);

		return retCode;
	}

	// Method for updating the text field.
	int CTextField::Update()
	{
		int retCode = Util::Err::NONE;

		// Update caret blinking.
		if(m_pTimer)
		{
			if(m_bSelected)
				m_caretBlinkTimer += m_pTimer->Delta();
			else
				m_caretBlinkTimer = 0.0f;
		}

		if(m_bSelected)
		{
			// Reset caret timer is text changes.
			if(m_pInputTyping->Update(m_caretIndex, &m_textData.currentText))
				m_caretBlinkTimer = 0.0f;
		}

		return retCode;
	}

	// Method for setting the text data for the text field.
	int CTextField::SetTextData(float size, const Math::CVector3& offset, const Math::CColor& color)
	{
		int retCode = Util::Err::NONE;

		m_textData.size						= size;
		m_textData.offset					= offset;
		m_textData.color					= color;

		return retCode;
	}

	// Method for displaying the text field.
	int CTextField::DisplayTextField(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign)
	{
		int retCode = Util::Err::NONE;

		if(m_pImage)
		{
			if((retCode = m_pImage->DisplayImage(position, scale, color, m_fade, hAlign, vAlign)) != Util::Err::NONE)
				return retCode;

			m_bounds = m_pImage->LastBounds();

			if(m_pFont)
			{
				if((retCode = m_pFont->Print(m_textData.currentText, m_textData.offset, m_textData.size, m_textData.color, m_fade, UI_HALIGN::Left, UI_VALIGN::Bottom, &m_bounds)) != Util::Err::NONE)
					return retCode;

				if(m_bSelected && fmodf(m_caretBlinkTimer, m_caretBlinkRate * 2) < m_caretBlinkRate)
				{
					m_caretPosition = m_pFont->CalculateLineLength(m_textData.currentText.substr(0, m_caretIndex), m_textData.size);
					if((retCode = m_pFont->Print("|", m_textData.offset + Math::CVector3(m_caretPosition - 0.075f * m_textData.size, -0.05f * m_textData.size, 0.0f), m_textData.size, m_textData.color, m_fade, UI_HALIGN::Left, UI_VALIGN::Bottom, &m_bounds)) != Util::Err::NONE)
						return retCode;
				}
			}
		}

		CheckInteraction();

		return retCode;
	}

	void CTextField::Release()
	{
		SAFE_DELETE_ARRAY(m_pInputTyping);
		CUIObject::Release();
	}
	
	// Method for calculating the bounds of the text field, and test user interation with the field.
	void CTextField::CheckInteraction()
	{
		Math::CVector2 pos;
		
		m_bHovered = false;

		if(m_pInput)
		{
			if(m_pImage)
			{
				pos = m_pImage->Material()->Camera()->ScreenToWorld(m_pInput->MouseData().pos);

				if(m_bounds.x < pos.x && m_bounds.y < pos.y && 
					m_bounds.z > pos.x &&m_bounds.w > pos.y)
				{
					m_bHovered = true;
				}
			}

			m_bClicked = false;
			
			if(m_pInput->MouseDown(0))
			{
				if(m_bHovered)
				{
					m_bClicked = true;
					m_bSelected = true;

					m_caretIndex = m_pFont->ClosestIndexRelativeToPoint(pos, m_textData.currentText, m_textData.offset, m_textData.size, 
						UI_HALIGN::Left, UI_VALIGN::Center, &m_bounds);
				}
				else
				{
					m_bSelected = false;
					m_caretIndex = 0;
				}
			}
		}
	}
};