//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CDataList.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CDataList.h"

namespace Util
{
	CListPtr::CListPtr()
	{
		m_pHead = NULL;
		m_pTail = NULL;
	}

	CListPtr::CListPtr(const CListPtr& other)
	{
	}

	CListPtr::CListPtr(CListPtr&& other)
	{
	}

	CListPtr::~CListPtr()
	{
		Release();
	}

	// Method for pushing data to the back of the list.
	void CListPtr::PushBack(void* data)
	{
		NODE* newNode = new NODE();
		newNode->data = data;

		if(m_pHead == NULL)
		{
			m_pHead = newNode;
			m_pTail = newNode;
		}

		m_pTail->next = newNode;
		newNode->next = NULL;
		m_pTail = newNode;
	}

	// Method for popping data from the front of the list.
	void* CListPtr::PopFront()
	{
		if(m_pHead == NULL)
			return NULL;

		NODE* temp = m_pHead;
		void* data = m_pHead->data;

		// If a head next node exists, replace the head node with it.
		if(m_pHead->next)
		{
			m_pHead = m_pHead->next;
			delete temp;
			return data;
		}

		delete temp;
		m_pHead = NULL;
		m_pTail = NULL;

		return data;
	}

	// Method for inserting data at an index on the list.
	bool CListPtr::InsertAt(void* data, u32 index)
	{
		if(m_pHead)
		{
			u32 counter = 0;

			NODE* newNode = new NODE();
			newNode->data = data;
			NODE* prev = m_pHead;
			NODE* temp = m_pHead;

			while(temp)
			{
				// Follow the list until the index parameter is reached by the counter variable.
				if(counter == index)
				{
					if(temp == m_pHead)
					{
						// Head node data.
						newNode->next = m_pHead;
						m_pHead = newNode;
					}
					else
					{
						// Non-head node data.
						prev->next = newNode;
						newNode->next = temp;
						if(m_pTail == prev)
						{
							m_pTail = newNode;
						}
					}

					break;
				}

				prev = temp;
				temp = temp->next;

				counter++;
			}
		}

		return true;
	}

	// Method for removing an element from the list at an index.
	bool CListPtr::RemoveAt(u32 index)
	{
		if(m_pHead)
		{
			u32 counter = 0;

			NODE* prev = m_pHead;
			NODE* temp = m_pHead;

			while(temp)
			{
				// Follow the list until the index parameter is reached by the counter variable.
				if(counter == index)
				{
					if(temp == m_pHead)
					{
						// Head node data.
						temp = temp->next;

						if(m_pTail == m_pHead)
							m_pTail = temp;

						delete m_pHead;
						m_pHead = temp;
					}
					else
					{
						// Non-head node data.
						prev->next = temp->next;
					
						if(m_pTail == temp)
							m_pTail = prev;

						delete temp;
					}

					break;
				}

				prev = temp;
				temp = temp->next;

				counter++;
			}
		}

		return true;
	}

	// Method for removing specific data from the list.
	void CListPtr::Remove(void* data)
	{
		if(m_pHead)
		{
			NODE* prev = m_pHead;
			NODE* temp = m_pHead;

			while(temp)
			{
				// Run through the list until data is found.
				if(temp->data == data)
				{
					if(temp == m_pHead)
					{
						// Check if data was found at the head node.
						temp = temp->next;

						if(m_pTail == m_pHead)
							m_pTail = temp;

						delete m_pHead;
						m_pHead = temp;
					}
					else
					{
						// Handle is data is found somewhere other than the head node.
						prev->next = temp->next;

						if(m_pTail == temp)
							m_pTail = prev;

						delete temp;
					}

					break;
				}

				prev = temp;
				temp = temp->next;
			}
		}
	}

	// Method for release the list's data.
	void CListPtr::Release()
	{
		NODE *current, *temp;
		current = temp = m_pHead;
		while(current)
		{
			delete temp;
			temp = current = current->next;
		}

		m_pHead = NULL;
		m_pTail = NULL;
	}
};