//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CDataQueue.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDATAQUEUE_H
#define CDATAQUEUE_H

#include "..\Globals\CGlobals.h"

namespace Util
{
	// CQueue class (template)
	// Class for creating a template queue data structure.
	template<typename T>
	class CQueue
	{
	private:
		// Queue node structure.
		typedef struct Node
		{
			Node*		next;
			T			data;

		} NODE;

		NODE *m_pHead, *m_pTail;

	public:
		CQueue()
		{
			m_pHead = NULL;
			m_pTail = NULL;
		}

		CQueue(const CQueue& other)
		{
		}

		CQueue(CQueue&& other)
		{
		}

		virtual ~CQueue()
		{
			Release();
		}

		// Method for pushing data to the back of the queue.
		void Push(T data)
		{
			if(m_pHead == NULL)
			{
				m_pHead = new NODE();
				m_pHead->data = data;
				m_pHead->next = NULL;
				m_pTail = m_pHead;
				return;
			}

			if(m_pTail)
			{
				NODE* ptr = new NODE();
				ptr->data = data;
				ptr->next = NULL;
				m_pTail->next = ptr;
				m_pTail = ptr;
			}
		}

		// Method for popping data from the front of the queue.
		bool Pop(T& data)
		{
			if(m_pHead == NULL)
				return false;

			NODE* temp = m_pHead;
			data = temp->data;
			if(m_pHead->next)
			{
				m_pHead = m_pHead->next;
				delete temp;
			}
			else
			{
				delete temp;
				m_pHead = NULL;
			}

			return true;
		}

		// Method for releasing all data from the queue, and for destroying the queue.
		void Release()
		{
			if(m_pHead == NULL)
				return;

			NODE* cur = m_pHead;
			NODE* ptr;

			while(cur)
			{
				ptr = cur;
				cur = cur->next;
				delete ptr;
			}

			m_pHead = NULL;
			m_pTail = NULL;
		}
		
	private:
	};
	
	// CQueue class
	// Class for creating a queue data structure with a void pointer.
	class CPtrQueue
	{
	private:
		// Pointer queue node structure.
		typedef struct Node
		{
			void*		data;
			Node*		next;

		} NODE;

		NODE *m_pHead, *m_pTail;

	public:
		CPtrQueue();
		CPtrQueue(const CPtrQueue& other);
		CPtrQueue(CPtrQueue&& other);
		virtual ~CPtrQueue();

		void								Push(void* data);
		void*								Pop();
		void								Release();

	private:
	};
};

#endif