//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CDataStack.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDATASTACK_H
#define CDATASTACK_H

#include "..\Globals\CGlobals.h"
#include "CMemoryFree.h"

namespace Util
{
	//---------------------------------------------------------------------------------------------
	// Template stack class.
	//---------------------------------------------------------------------------------------------
	
	// CStack class (template)
	// Class for creating a template stack data structure.
	template<typename T>
	class CStack
	{
	private:
		// Stack data structure.
		struct Stack
		{
			T*			elems;
			u32			length;
			u32			allocLength;
		} m_stack;

	public:
		CStack()
		{
			m_stack.elems = NULL;
		}

		CStack(const CStack& other)
		{
		}

		CStack(CStack&& other)
		{
		}

		virtual ~CStack()
		{
			Release();
		}

		// Method for creating the initial stack object.
		void Initialize()
		{
			m_stack.length					= 0;
			m_stack.allocLength				= 4;
			m_stack.elems					= (T*)malloc(m_stack.allocLength * sizeof(T));
		}
	
		// Method for pushing an element to the stack.
		void Push(T elem)
		{
			if(!m_stack.elems)
				Initialize();

			if(m_stack.length == m_stack.allocLength)
				IncreaseStack();

			m_stack.elems[m_stack.length] = elem;
			m_stack.length++;
		}
		
		// Method for popping an elements from the stack and store in the the elem parameter.
		void Pop(T* elem)
		{
			if(!m_stack.elems)
				return;

			m_stack.length--;
			return m_stack.elems[m_stack.length];
		}
		
		// Get an elements from the stack at an index.
		T& At(u32 index)
		{
			return m_stack.elems[index];
		}

		T& operator [] (u32 index)
		{
			return At(index);
		}

		// Method for destroying the stack
		void Release()
		{
			if(m_stack.length > 0) SAFE_DELETE(m_stack.elems);
			m_stack.length = 0;
		}

		//
		// Stack inline functions.
		//

		inline T* Array()
		{
			return m_stack.elems;
		}

		inline u32 Length()
		{
			return m_stack.length;
		}

		inline u32 AllocLength()
		{
			return m_stack.allocLength;
		}

	private:
		void IncreaseStack()
		{
			m_stack.allocLength		*= 2;
			m_stack.elems			= (T*)realloc(m_stack.elems, m_stack.allocLength * sizeof(T));
		}
	};

	//---------------------------------------------------------------------------------------------
	// Pointer stack class.
	//---------------------------------------------------------------------------------------------
	
	// CPtrStack class
	// Class for creating a stack data structure w/ void pointers.
	class CPtrStack
	{
	private:
		// Point stack data structure.
		struct Stack
		{
			void*		elems;
			u32			elemSize;
			u32			length;
			u32			allocLength;
			void		(*Release) (void*);
		} m_stack;

	public:
		CPtrStack();
		CPtrStack(const CPtrStack& other);
		CPtrStack(CPtrStack&& other);
		virtual ~CPtrStack();

		void								Initialize(int elemSize = sizeof(void**), void(*Release) (void*) = NULL);

		void								Push(void* elem);
		void								Pop(void* elem);
		u8*									At(u32 index);

		u8* operator [] (u32 index)
		{
			return At(index);
		}

		void													Release									();

		inline u32												Length									();
		inline u32												AllocLength								();

	private:
		void													IncreaseStack							();
	};

	//
	// Pointer stack inline methods.
	//

	inline u32 CPtrStack::Length()
	{
		return m_stack.length;
	}

	inline u32 CPtrStack::AllocLength()
	{
		return m_stack.allocLength;
	}
};

#endif