//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CTimer.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CTimer.h"

namespace Util
{
	// Timer constructer handles the timer initialization.
	CTimer::CTimer()
	{
		m_maxTimeStep						= 1.0f / 3.0f;
		m_delta								= 0.0f;
		m_smoothDelta						= 0.0f;
		
		m_currentTime						= 0;

		ZeroMemory(&m_frameTime, sizeof(float) * TIMER_SAMPLE_COUNT);
		m_sampleCount						= 0;

		m_frameRate							= 0;
		m_frameCount						= 0;
		m_timeElapsed						= 0.0f;
		m_time								= 0.0f;
		m_smoothTime						= 0.0f;

		m_targetFrameRate					= 0.0f;

		// Initialize the performance timer.
		QueryPerformanceFrequency((LARGE_INTEGER*)&m_frequency);
		QueryPerformanceCounter((LARGE_INTEGER*)&m_lastTime);
		m_timeScale = 1.0f / m_frequency;
	}

	CTimer::CTimer(const CTimer& other)
	{
	}

	// Method for updating the timer. Should be called above any code that might use the timer.
	void CTimer::Tick()
	{
		float timeElapsed;

		QueryPerformanceCounter((LARGE_INTEGER*)&m_currentTime);

		timeElapsed = (m_currentTime - m_lastTime) * m_timeScale;

		if(m_targetFrameRate > 0.0f)
		{
			// Wait for target frame rate to be reached before moving on (only if 'm_targetFrameRate' > 0.0f).
			if(timeElapsed < (1.0f / m_targetFrameRate))
			{
				std::this_thread::sleep_for(std::chrono::microseconds((__int64)(1000000.0f * ((1.0f / m_targetFrameRate) - timeElapsed))));
				
				QueryPerformanceCounter((LARGE_INTEGER*)&m_currentTime);

				timeElapsed = (m_currentTime - m_lastTime) * m_timeScale;
			}
		}

		m_lastTime = m_currentTime;

		// Calculate the frame rate.
		if(fabsf(timeElapsed - m_delta) < m_maxTimeStep)
		{
			memmove(&m_frameTime[1], m_frameTime, sizeof(float) * (TIMER_SAMPLE_COUNT - 1));
			m_frameTime[0] = timeElapsed;

			m_delta = timeElapsed;
			m_time += m_delta;

			if(m_sampleCount < TIMER_SAMPLE_COUNT)
				m_sampleCount++;
		}

		m_frameCount++;
		m_timeElapsed += timeElapsed;

		if(m_timeElapsed > 1.0f)
		{
			m_frameRate						= m_frameCount;
			m_frameCount					= 0;
			m_timeElapsed					= 0.0f;
		}

		// Calculate the smoothed out time delta.
		m_smoothDelta = 0.0f;

		for(u32 i = 0; i < m_sampleCount; i++)
		{
			m_smoothDelta += m_frameTime[i];
		}

		if(m_sampleCount > 0)
			m_smoothDelta /= m_sampleCount;

		m_smoothTime += m_smoothDelta;
	}
};