//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CTimer.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CTIMER_H
#define CTIMER_H

#include <thread>
#include <chrono>
#include "..\Globals\CGlobals.h"

namespace Util
{
	const int													TIMER_SAMPLE_COUNT						= 60;

	// CTimer class
	// Class for creating a timer that stores helpful data e.g. time delta, time, fps, etc.
	class CTimer
	{
	public:
		CTimer();
		CTimer(const CTimer& other);

		void								Tick();
		
		inline float						Delta() const;
		inline float						Time() const;
		inline float						SmoothDelta() const;
		inline float						SmoothTime() const;

		inline float						TimeScale() const;
		inline void							TimeScale(float scale);
	
		inline float						MaxTimeStep() const;
		inline void							MaxTimeStep(float maxStep);

		inline float						TargetFrameRate() const;
		inline void							TargetFrameRate(float targetFps);

	private:
		float								m_timeScale;
		float								m_maxTimeStep;
		float								m_delta;
		float								m_smoothDelta;

		u64									m_currentTime;
		u64									m_lastTime;
		u64									m_frequency;

		float								m_frameTime[TIMER_SAMPLE_COUNT];
		u32									m_sampleCount;

		u32									m_frameRate;
		u32									m_frameCount;
		float								m_timeElapsed;
		float								m_time;
		float								m_smoothTime;

		float								m_targetFrameRate;
	};
		
	// Time delta getter.
	inline float CTimer::Delta() const
	{
		return m_delta;
	}

	// Time getter.
	inline float CTimer::Time() const
	{
		return m_time;
	}

	// Smooth time delta getter.
	inline float CTimer::SmoothDelta() const
	{
		return m_smoothDelta;
	}

	// Smooth time getter.
	inline float CTimer::SmoothTime() const
	{
		return m_smoothTime;
	}

	// Time scale getter and setter.

	inline float CTimer::TimeScale() const
	{
		return m_timeScale;
	}

	inline void CTimer::TimeScale(float scale)
	{
		m_timeScale = scale;
	}
	
	// Max time step getter and setter.

	inline float CTimer::MaxTimeStep() const
	{
		return m_maxTimeStep;
	}

	inline void CTimer::MaxTimeStep(float maxStep)
	{
		m_maxTimeStep = maxStep;
	}

	// Target framerate getter and setter.

	inline float CTimer::TargetFrameRate() const
	{
		return m_targetFrameRate;
	}

	inline void CTimer::TargetFrameRate(float targetFps)
	{
		m_targetFrameRate = targetFps;
	}
};

#endif