//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Actors\CPlayer.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CPLAYER_H
#define CPLAYER_H

#include <Application\CWinData.h>
#include <Application\CWinUtil.h>
#include <Application\CInput.h>
#include <Application\CInputControls.h>
#include <Graphics\CRenderingData.h>
#include <Graphics\CMesh.h>
#include <Actors\CActor.h>
#include "CPlayerCamera.h"
#include "..\UI\CPlayerHUD.h"
#include "..\UI\CGameUI.h"

namespace Actor
{
	// CPlayer class
	// Class the housing most of the elements for the user of this game.
	class CPlayer : public CActor
	{
	public:
		CPlayer();
		CPlayer(const CPlayer& other);
		virtual ~CPlayer();

		int									Initialize() override;
		int									Update() override;
		int									Render() override;
		int									RenderHUD();

		int									PostResize();

		void								Release();
		
		inline GFX::CMesh*					PlayerMesh() const;
		void								PlayerMesh(GFX::CMesh* playerMesh);

		inline CPlayerCamera*				PlayerCamera() const;
		inline void							PlayerCamera(CPlayerCamera* pCamera);
		inline UI::CPlayerHUD*				PlayerHUD() const;
		inline void							PlayerHUD(UI::CPlayerHUD* pPlayerHUD);

		inline App::CInput*					Input() const;
		inline void							Input(App::CInput* pInput);
		inline App::WINDATA*				WinData() const;
		inline void							WinData(App::WINDATA* pWinData);

		inline GFX::RENDERDATA*				RenderData() const;
		inline void							RenderData(GFX::RENDERDATA* pRenderData);
		inline Util::CTimer*				Timer() const;
		inline void							Timer(Util::CTimer* pTimer);

		inline UI::CGameUI*					GameUI() const;
		inline void							GameUI(UI::CGameUI* pGameUI);

	private:
		void								UpdateRotation();
		void								UpdateMotion();

		Math::CVector2						m_lookSensitivity;

		CPlayerCamera*						m_pCamera;
		UI::CPlayerHUD*						m_pPlayerHUD;

		GFX::CMesh*							m_pPlayerMesh;

		App::CInput*						m_pInput;
		App::WINDATA*						m_pWinData;

		GFX::RENDERDATA*					m_pRenderData;
		Util::CTimer*						m_pTimer;
		UI::CGameUI*						m_pGameUI;
	};
	
	// Player meshh getter.
	inline GFX::CMesh* CPlayer::PlayerMesh() const
	{
		return m_pPlayerMesh;
	}

	// Player camera getter and setter.

	inline CPlayerCamera* CPlayer::PlayerCamera() const
	{
		return m_pCamera;
	}

	inline void CPlayer::PlayerCamera(CPlayerCamera* pCamera)
	{
		m_pCamera = pCamera;
	}

	// Player HUD getter and setter.

	inline UI::CPlayerHUD* CPlayer::PlayerHUD() const
	{
		return m_pPlayerHUD;
	}

	inline void CPlayer::PlayerHUD(UI::CPlayerHUD* pPlayerHUD)
	{
		m_pPlayerHUD = pPlayerHUD;
	}

	// Input getter and setter.

	inline App::CInput* CPlayer::Input() const
	{
		return m_pInput;
	}

	inline void CPlayer::Input(App::CInput* pInput)
	{
		m_pInput = pInput;
	}

	// Window data getter and setter.

	inline App::WINDATA* CPlayer::WinData() const
	{
		return m_pWinData;
	}

	inline void CPlayer::WinData(App::WINDATA* pWinData)
	{
		m_pWinData = pWinData;
	}

	// Render data getter and setter.

	inline GFX::RENDERDATA* CPlayer::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CPlayer::RenderData(GFX::RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}

	// Timer getter and setter.

	inline Util::CTimer* CPlayer::Timer() const
	{
		return m_pTimer;
	}

	inline void CPlayer::Timer(Util::CTimer* pTimer)
	{
		m_pTimer = pTimer;
	}

	// Game UI getter and setter.

	inline UI::CGameUI* CPlayer::GameUI() const
	{
		return m_pGameUI;
	}

	inline void CPlayer::GameUI(UI::CGameUI* pGameUI)
	{
		m_pGameUI = pGameUI;
	}
};

#endif