//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Graphics\CMatFireEffect.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatFireEffect.h"

namespace GFX
{
	CMatFireEffect::CMatFireEffect()
	{
		m_elementCount						= 6;
		m_bufferCount						= 3;
		m_textureCount						= 3;
		m_bUseInverse						= true;
	}

	CMatFireEffect::CMatFireEffect(const CMatFireEffect& other)
	{
	}

	CMatFireEffect::~CMatFireEffect()
	{
	}

	// Method for setting up the fire effect material's shader.
	int CMatFireEffect::SetupShader()
	{
		int retCode = Util::Err::NONE;

		m_pIed								= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];
		
		m_pIed[0]	= { "POSITION",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		0,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[1]	= { "INSTPOS",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		0,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[2]	= { "INSTSCALE",		0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		12,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[3]	= { "INSTUV",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		24,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[4]	= { "INSTCOLOR",		0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		40,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[5]	= { "INSTDATA",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		56,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		
		m_pVertexResName		= MAKEINTRESOURCE(HLSL_VFIREEFFECT);
		m_pGeometryResName		= MAKEINTRESOURCE(HLSL_GFIREEFFECT);
		m_pPixelResName			= MAKEINTRESOURCE(HLSL_PFIREEFFECT);
		m_pLogFilename			= ".\\FireEffectShader.txt";
		
		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(FIREEFFECT_CBUFFER), true)) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pShader->InitializeConstantBuffer(2, sizeof(FIREEFFECT_DISTORTION), true)) != Util::Err::NONE)
			return retCode;

		ZeroMemory(&m_fireCBuffer, sizeof(m_fireCBuffer));

		m_fireCBuffer.color					= Math::CColor(1, 1, 1, 1);
		m_fireCBuffer.softness				= 0.5f;
		m_fireCBuffer.speeds				= Math::CVector3(1.2f, 2.6f, 3.1f);
		m_fireCBuffer.scales				= Math::CVector3(1.0f, 2.0f, 3.0f);

		m_fireDistortion.distortion0		= Math::CVector2(0.1f, 0.2f);
		m_fireDistortion.distortion1		= Math::CVector2(0.1f, 0.3f);
		m_fireDistortion.distortion2		= Math::CVector2(0.1f, 0.1f);

		m_fireDistortion.distortionScale	= 0.8f;
		m_fireDistortion.distortionBias		= 0.5f;

		return retCode;
	}
	
	// Method for setting the fire effect material up for render.
	int CMatFireEffect::Render()
	{
		int retCode = Util::Err::NONE;

		m_fireCBuffer.screenWidth			= m_pDX11Data->viewport.Width;
		m_fireCBuffer.screenHeight			= m_pDX11Data->viewport.Height;
		m_fireCBuffer.cameraPosition		= m_pCamera->Transform()->Position();
		m_mtxInvCBuffer.projectionInv		= m_pCamera->ProjectionMatrixInv();
		m_mtxInvCBuffer.viewInv				= m_pCamera->ViewMatrixInv();
		
		if((retCode = m_pShader->ResetConstantBuffer(1, &m_fireCBuffer)) != Util::Err::NONE)
			return retCode;
		if((retCode = m_pShader->ResetConstantBuffer(2, &m_fireDistortion)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;
		
		// Bind shader constant buffer.
		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pShader->BindCBuffer(2, m_shaderType)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;
		
		return retCode;
	}
};