//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CParticleFire.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CParticleFire.h"

namespace GFX
{
	CParticleFire::CParticleFire()
	{
		m_textureCount						= 3;

		m_colorTextureFilename				= "";
		m_noiseTextureFilename				= "";
		m_alphaTextureFilename				= "";
	}

	CParticleFire::CParticleFire(const CParticleFire& other)
	{
	}

	CParticleFire::~CParticleFire()
	{
	}
	
	// Method for initializing the particle fire device.
	int CParticleFire::Initialize()
	{
		int retCode = Util::Err::NONE;

		m_ppTexture = new GFX::CTexture*[m_textureCount];

		// Create the fire textures.
		for(u32 i = 0; i < m_textureCount; i++)
		{
			m_ppTexture[i] = new GFX::CTexture();
			m_ppTexture[i]->DirectXData(m_pDX11Data);
		}

		if((retCode = m_ppTexture[0]->Initialize((char*)m_colorTextureFilename.c_str())) != Util::Err::NONE)
			return retCode;

		if((retCode = m_ppTexture[1]->Initialize((char*)m_noiseTextureFilename.c_str())) != Util::Err::NONE)
			return retCode;

		if((retCode = m_ppTexture[2]->Initialize((char*)m_alphaTextureFilename.c_str())) != Util::Err::NONE)
			return retCode;

		
		if((retCode = CParticleDevice::Initialize()) != Util::Err::NONE)
			return retCode;
		return retCode;
	}
	
	// Method for updating the particle fire device.
	int CParticleFire::Update()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = CParticleDevice::Update()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}
	
	// Method for rendering the particle fire device.
	int CParticleFire::Render()
	{
		int retCode = Util::Err::NONE;

		((CMatFireEffect*)m_pMaterial)->m_fireCBuffer.color = m_color;
		((CMatFireEffect*)m_pMaterial)->m_fireCBuffer.softness = m_softness;
		((CMatFireEffect*)m_pMaterial)->m_fireCBuffer.billboardType = m_billboardType;
		((CMatFireEffect*)m_pMaterial)->m_fireCBuffer.time = m_playbackTimer;
		
		if((retCode = CParticleDevice::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for releasing data for the particle fire device.
	void CParticleFire::Release()
	{
		CParticleDevice::Release();

		for(u32 i = 0; i < m_textureCount; i++)
		{
			SAFE_RELEASE_DELETE(m_ppTexture[i]);
		}
	}
};