//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: UI\CGameUI.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CGAMEUI_H
#define CGAMEUI_H

#include <UI\CUIFreeType.h>
#include <UI\CUIFontRenderer.h>
#include <UI\CUIImage.h>
#include <Graphics\CTextureList.h>
#include <Graphics\CMatFT2Font.h>
#include <Graphics\CMatUIImage.h>

namespace UI
{
	// CGameUI class.
	// Class for storing UI data that acts as defaults for this application.
	class CGameUI : public App::CDX11Object
	{
	public:
		CGameUI();
		CGameUI(const CGameUI& other);
		virtual ~CGameUI();

		int									Initialize();
		int									Update();
		int									Render();

		void								Release();

		inline UI::CImage*					CheckboxBGImage() const;
		inline void							CheckboxBGImage(UI::CImage* pCheckboxBGImage);
		inline UI::CImage*					CheckboxFGImage() const;
		inline void							CheckboxFGImage(UI::CImage* pCheckboxFGImage);

		inline GFX::CMatFT2Font*			FT2Material() const;
		inline void							FT2Material(GFX::CMatFT2Font* pMatFT2Font);
		inline GFX::CMatUIImage*			UIImageMaterial() const;
		inline void							UIImageMaterial(GFX::CMatUIImage* pMatUIImage);

		inline UI::CFreeType*				FreeType() const;
		inline void							FreeType(UI::CFreeType* pFreeType);
		inline UI::CFontRenderer*			FontRenderer() const;
		inline void							FontRenderer(UI::CFontRenderer* pFontRenderer);

		inline GFX::RENDERDATA*				RenderData() const;
		inline void							RenderData(GFX::RENDERDATA* pRenderData);

	private:
		UI::CImage*							m_pCheckboxBGImage;
		UI::CImage*							m_pCheckboxFGImage;

		GFX::CMatFT2Font*					m_pMatFT2Font;
		GFX::CMatUIImage*					m_pMatUIImage;

		UI::CFreeType*						m_pFreeType;
		UI::CFontRenderer*					m_pFontRenderer;

		GFX::RENDERDATA*					m_pRenderData;
	};

	// Checkbox background image getter and setter.

	inline UI::CImage* CGameUI::CheckboxBGImage() const
	{
		return m_pCheckboxBGImage;
	}

	inline void CGameUI::CheckboxBGImage(UI::CImage* pCheckboxBGImage)
	{
		m_pCheckboxBGImage = pCheckboxBGImage;
	}

	// Checkbox foreground image getter and setter.

	inline UI::CImage* CGameUI::CheckboxFGImage() const
	{
		return m_pCheckboxFGImage;
	}

	inline void CGameUI::CheckboxFGImage(UI::CImage* pCheckboxFGImage)
	{
		m_pCheckboxFGImage = pCheckboxFGImage;
	}

	// FreeType2 material getter and setter.

	inline GFX::CMatFT2Font* CGameUI::FT2Material() const
	{
		return m_pMatFT2Font;
	}

	inline void CGameUI::FT2Material(GFX::CMatFT2Font* pMatFT2Font)
	{
		m_pMatFT2Font = pMatFT2Font;
	}

	// UI Image material getter and setter.

	inline GFX::CMatUIImage* CGameUI::UIImageMaterial() const
	{
		return m_pMatUIImage;
	}

	inline void CGameUI::UIImageMaterial(GFX::CMatUIImage* pMatUIImage)
	{
		m_pMatUIImage = pMatUIImage;
	}

	// FreeType2 object getter and setter.

	inline UI::CFreeType* CGameUI::FreeType() const
	{
		return m_pFreeType;
	}

	inline void CGameUI::FreeType(UI::CFreeType* pFreeType)
	{
		m_pFreeType = pFreeType;
	}

	// Font renderer getter and setter.

	inline UI::CFontRenderer* CGameUI::FontRenderer() const
	{
		return m_pFontRenderer;
	}

	inline void CGameUI::FontRenderer(UI::CFontRenderer* pFontRenderer)
	{
		m_pFontRenderer = pFontRenderer;
	}

	// Render data getter and setter.

	inline GFX::RENDERDATA* CGameUI::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CGameUI::RenderData(GFX::RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}
};

#endif