//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Graphics\CFireObject.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CFireObject.h"

namespace World
{
	CFireObject::CFireObject()
	{
		m_bSoundStarted						= false;

		m_pParticleFire						= NULL;
		m_pParticleSystem					= NULL;

		m_pFireSound						= NULL;
		m_pFireVoice						= NULL;
		m_pPointLight						= NULL;
	}

	CFireObject::CFireObject(const CFireObject& other)
	{
	}

	CFireObject::~CFireObject()
	{
	}
	
	// Particle fade and scale functions.

	Math::CColor ParticleFade(void* game, float lifetime)
	{
		return Math::CColor(1, 1, 1, 1);
	}
	
	Math::CVector3 ParticleScale(void* game, float lifetime)
	{
		return Math::CVector3(1, 1, 1);
	}

	// Method for initializing the fire object, with the fire material, and fire particle device.
	int CFireObject::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Allocate memory.
		m_pParticleFire						= new GFX::CParticleFire();

		// Setup a new transform if one isn't already stored.
		if(!m_pTransform)
		{
			m_pTransform = Math::CTransformSharedPtr(new Math::CTransform());
			m_pTransform->Update();
		}

		// Setup the fire particle device.
		m_pParticleFire->MaxParticles(1);
		m_pParticleFire->SpawnRate(0.0f);
		m_pParticleFire->PlaybackState(GFX::PARTICLE_PLAYBACK_STATE::Play);
		m_pParticleFire->BillboardType(GFX::PARTICLE_BILLBOARD_TYPE::PBT_Flat);
		m_pParticleFire->Loop(true);
		m_pParticleFire->Duration(5.0f);
		m_pParticleFire->Lifetime(0.0f);

		m_pParticleFire->Velocity(Math::CVector3(0, 1, 0));
		m_pParticleFire->InitialScale(Math::CVector3(1.1f, 1.1f, 1.1f));
		m_pParticleFire->InitialColor(Math::CColor(1));

		m_pParticleFire->Softness(0.5f);
		m_pParticleFire->Material(m_pMaterial);

		m_pParticleFire->FunctionSetup(this);
		m_pParticleFire->FadeFunction(&ParticleFade);
		m_pParticleFire->ScaleFunction(&ParticleScale);

		m_pParticleFire->ColorTextureFilename(".\\GFX\\Textures\\Particles\\Fire\\FireColor000.png");
		m_pParticleFire->NoiseTextureFilename(".\\GFX\\Textures\\Particles\\Fire\\FireNoise000.png");
		m_pParticleFire->AlphaTextureFilename(".\\GFX\\Textures\\Particles\\Fire\\FireAlpha000.png");

		if(m_pParticleSystem)
			m_pParticleSystem->AddParticleDevice((GFX::CParticleDevice*&)m_pParticleFire);
			
		m_pFireSound = Audio::CAudioList::Instance().AddAudioSource((char*)m_fireSoundFilename.c_str());

		if(m_pFireSound)
		{
			m_pFireSound->DistanceScale(0.25f);
			m_pFireSound->Sound3D(true);
		}

		return retCode;
	}
	
	// Method for updating the fire object. Flickers the point light, 
	// and makes sure the fire sound is playing and updated..
	int CFireObject::Update()
	{
		int retCode = Util::Err::NONE;
		
		m_pParticleFire->InstanceAt(0).position = m_pTransform->Position() + m_positionOffset;
		if(m_pPointLight)
		{
			m_pPointLight->position = m_pParticleFire->InstanceAt(0).position + Math::CVector3(0, 0.5f, 0);
			m_pPointLight->strength = powf(fabsf(sinf(m_pParticleFire->PlaybackTime() * 8)), 2.0f) / 15 + 1.0f;
		}
		if(m_pFireSound)
		{
			if(!m_bSoundStarted)
			{
				m_pFireSound->Play(1.0f, true, m_pParticleFire->InstanceAt(0).position, Math::CVector3(0, 0, 1), &m_pFireVoice);
				m_bSoundStarted = true;
			}

			m_pFireSound->Update();
		}

		return retCode;
	}

	void CFireObject::Release()
	{
		if(!m_pParticleSystem)
			SAFE_RELEASE_DELETE(m_pParticleFire);
	}
};