//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Graphics\CWorldObject.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CWORLDOBJECT_H
#define CWORLDOBJECT_H

#include <Application\CDX11Object.h>
#include <Math\CMathTransform.h>
#include <Graphics\CMaterial.h>
#include <Graphics\CRenderingData.h>
#include "..\Utilities\CGameError.h"

namespace World
{
	// CWorldObject class.
	// Base class for assisting it creating world objects for the game.
	class CWorldObject : public App::CDX11Object
	{
	public:
		CWorldObject();
		CWorldObject(const CWorldObject& other);
		virtual ~CWorldObject();

		virtual int							Initialize();
		virtual int							Update();
		virtual int							Render();
		virtual void						Release();

		inline GFX::CMaterial*				Material() const;
		inline void							Material(GFX::CMaterial* pMaterial);
		inline Math::CTransformSharedPtr	Transform() const;
		inline void							Transform(Math::CTransformSharedPtr pTransform);
		inline GFX::RENDERDATA*				RenderData() const;
		inline void							RenderData(GFX::RENDERDATA* pRenderData);

	protected:
		GFX::CMaterial*						m_pMaterial;
		Math::CTransformSharedPtr			m_pTransform;
		GFX::RENDERDATA*					m_pRenderData;

	private:
	};
	
	// Material getter and setter.

	inline GFX::CMaterial* CWorldObject::Material() const
	{
		return m_pMaterial;
	}

	inline void CWorldObject::Material(GFX::CMaterial* pMaterial)
	{
		m_pMaterial = pMaterial;
	}
	
	// Transform getter and setter.

	inline Math::CTransformSharedPtr CWorldObject::Transform() const
	{
		return m_pTransform;
	}

	inline void CWorldObject::Transform(Math::CTransformSharedPtr pTransform)
	{
		m_pTransform = pTransform;
	}
	
	// RenderData getter and setter.

	inline GFX::RENDERDATA* CWorldObject::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CWorldObject::RenderData(GFX::RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}
};

#endif